#!/usr/bin/wish

# Change the menus to reflect new files
# Document Realisations
# Allow for color changes

######################################
# Set VERSION/PLATFORM info

global TOOL TOOL_VERSION PLATFORM OPENING_GRAPHIC APPTYPE
set TOOL coder 
set TOOL_VERSION "4.67 (February 2005)"
set PLATFORM $tcl_platform(platform)
set OPENING_GRAPHIC "Coder1.gif"


######################################
# Warn if wrong TCL Version

if { $tcl_version < "8.1"} {
    tk_messageBox -message "Sorry, The $TOOL requires Tcl/Tk 8.2 or above. See http://www.wagsoft.com/TCL/index.html to download."
    exit
}

catch "console hide"

if { $tcl_version < "8.2"} {
    tk_messageBox -message "You are using version $tcl_version of Tcl/Tk. It should work, but if you have problems, see http://www.wagsoft.com/TCL/index.html to download a more recent version."
update
}

# wm protocol . WM_DELETE_WINDOW Quit


###########################
# DEFINE DEFAULT COLORS

global BG_COLOR1 BG_COLOR2
set BG_COLOR1 "#FCD98E"
set BG_COLOR2 "#F6C19E"
tk_setPalette $BG_COLOR1

######################################
# Determine the root directory

global DIR ROOT SRC PICT_DIR

proc root-dir {} {
    if ![catch "info script"] {
	set dir [file dirname [info script]] 
	if { $dir != "." && $dir != ":"} {return $dir}
    } 

    if ![catch "file nativename [pwd]" result] {
	return $result
    } 
    
    return  [pwd]
}

if [info exists ::freewrap::patchLevel] {
    # This is the freewrapped version
    set APPTYPE exec
    set DIR [file dirname [info nameofexecutable]]
    set ROOT /Mick/Tools/CoderStuff/Coder46
} else {
    if {[info nameofexecutable] == [file dirname [info script]]} {
	# We have a starkit
	set APPTYPE exec
	set DIR [file dirname [info nameofexecutable]]
	set ROOT [info nameofexecutable]
    } else {
	set APPTYPE script
	set DIR [root-dir]
	set ROOT $DIR
    }
}


############################
# Define directory defaults

global SRC SCHEME SHARED LIBRARY SCHEMES ANALYSES GRAMMARS TEXTS

# Define the Dirs
set SRC     [file join $ROOT Source]
set PICT_DIR [file join $ROOT Picts]
set SCHEME [file join $SRC Scheme]
set SHARED [file join $SRC Shared]
set LIBRARY [file join $SRC Library]
set SCHEMES [file join $DIR Schemes]
set GRAMMARS $SCHEMES ;# name used by parser
set ANALYSES [file join $DIR Analyses]
if ![file isdir $ANALYSES] {
    set ANALYSES $DIR
}
set TEXTS [file join $DIR Texts]
if ![file isdir $TEXTS] {
    set TEXTS $DIR
}

######################################
# Load in preferences (if they exist)

global PREFS_FILE 
set PREFS_FILE [file join $DIR ".preferences"]

if [file exists $PREFS_FILE] {
    catch "source $PREFS_FILE"
}

######################################
# Load the modules


# Define the files
global LIB_FILES CDR_FILES STATS_FILES GFR_FILES SCHEME_FILES GFR_PATCHES
set LIB_FILES {tcl-extens tk-extens maths getvalue modal-dialog\
	scrolled-text scrolled-text-dial scrolled-listbox busydialog\
	toolbar draw menuPackage popup-choose draw-pdx bug-report Search procs mclistbox}

#Toolbar1
set SHARED_FILES {Interface Help Segment Import load-xml} 
set CDR_FILES {Globals Interface Codings Menus Interface CoderWindow \
    File Load LoadLisp LoadTagged Save Save3 Choices Goto Options Review Filter }
set STATS_FILES {StatsWindow StatsComp StatsCompSystem StatsCompFile\
	StatsDescript StatsSave StatsCell tstats ScatterPlot}
set GFR_FILES {Globals GraphPane GraphWindow GraphEdit Change-EC\
	GraphCommon GraphDraw1 GraphDraw2 HelpDefs Options}
set SCHEME_FILES {Scheme Negation PathsSimple File}
set GFR_PATCHES { GrapherAdds}

switch -- $PLATFORM {
    "windows" {
	lappend SHARED_FILES "Print-win-gdi"
	lappend LIB_FILES tk_getOpenFix2
    }

    "unix" {
	lappend SHARED_FILES "Print-win-linux"
    }
}

# Start the Load
source [file join $SHARED Opener.tcl]
source [file join $LIBRARY ProgressBar.tcl]

set fcount [llength [concat $LIB_FILES $CDR_FILES $GFR_FILES $STATS_FILES\
	$SCHEME_FILES $GFR_PATCHES $SHARED_FILES]]
open-load $fcount


load-module $LIBRARY $LIB_FILES
load-module $SHARED $SHARED_FILES
load-module $SCHEME $SCHEME_FILES
load-module $SCHEME $GFR_FILES
load-module $SRC $CDR_FILES
load-module $SRC $STATS_FILES
load-module $SRC $GFR_PATCHES
load-patches
puts "All Files Loaded"

close-load



# Add the cd2/cd3/scheme extensions to the windows registry
if { $APPTYPE == "exec" && ![info exists REGISTERED] } {
    switch -- [ask-user "Do you want to associate Coder documents (.cd2, .cd3, .scheme, .dst) with this application?"] {
	"yes" {
	    global ROOT REGISTERED
	    source $ROOT/Source/Library/registry.tcl
	    set exec [info nameofexecutable]
	    RegisterFileType .cd2 cdr2File "Systemic Coder File (vers 2)" $exec
	    RegisterFileType .cd3 cdr2File "Systemic Coder File (vers 3)" $exec
	    RegisterFileType .scheme cdr2File "Systemic Coder Scheme (vers 3)" $exec
	    RegisterFileType .dst cdr2File "Systemic Coder DocSet (vers 3)" $exec
	    set REGISTERED 1
	}

	default {
	    set REGISTERED 0
	}
    }
    save-preferences
}

add-saveas-option "CODINGS 3.0" .cd3 save-codings3
add-saveas-option "CODINGS 2.0" .cd2 save-codings2
add-saveas-option "CODINGS 1.0" .cdr save-codings1
# add-saveas-option "Tab Delimited" .tdf save-codings-tab-delimited 0
add-saveas-option "Tab Delimited" .txt save-codings-txt 0
add-load-option "CODINGS 3.0" {.cd3 .cd2} load-codings2
add-load-option "Codings 1.0" .codings load-codings1
add-load-option "Corpus File" .res1 load-res1-file
add-load-option "Tagged text" .txt load-tagged

global argv
if {$argv != {}} {
    set filename [lindex $argv 0]
    regsub -all {\\} $filename "/" filename
    start-coder $filename
} else {
    start-coder
}

puts "Coder Launched Sucessfully"
