######################################
## Systemic Coder
#
#  File: Choices.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.

proc install-coding {cid} {
    global NODE CURRENT SHOW_CONTEXT HISTORY
    set CURRENT $cid
   
    if { $SHOW_CONTEXT >  0 } {
	set-coder-context $cid
    } else {
	set-coder-text $NODE($cid,textstripped) $cid
    }
    set-coder-comment $NODE($cid,comment)
    set features $NODE($cid,features)
    set HISTORY [sort-features $features]

    # if the coding is ignored, change the "Ignore" button.
    if { $NODE($cid,ignore) == 1} {
	.coder.tbar.action.ignore config -text "Unignore"
    } else {	
	.coder.tbar.action.ignore config -text "Ignore"
    }

    set-counts
    offer-choices
}


proc install-current-coding {} {
    global CURRENT CURR_INDEX NODE TEXT_NODES
    
    if { $TEXT_NODES == {} } {return}
    if {$CURRENT == {}} {
	set CURRENT [nth-coding 0]
	set CURR_INDEX 0
    }

    # old - skip ignored items. But if we enter from review, we want them
    if { 1 == 0 && $NODE($CURRENT,ignore) == 1} {
	install-next-coding
    } else {
	install-coding $CURRENT
    }
}


proc install-next-coding {{ignore_complete {}}} {
    global CURRENT TEXT_NODES NODE IGNORE_COMPLETE

    if { $ignore_complete == {}} {
	set ignore_complete $IGNORE_COMPLETE 
    }

    set max [llength $TEXT_NODES] 
    set indx [node-position $CURRENT]
    while { [incr indx] < $max } {
	set id [nth-coding $indx]
	if { $NODE($id,ignore) == 0} {
	    if { $ignore_complete == 0 || $NODE($id,completed-p) == 0 } {
		install-coding $id
		return 1
	    }
	}
    }
    bell
    install-last-coding
    return 0
}

proc install-last-coding {} {
    global TEXT_NODES NODE
    set last [llength $TEXT_NODES] 
    while { [incr last -1] > 0 } {
	set id [nth-coding $last]
	if { $NODE($id,ignore) == 0} {
	    install-coding $id
	    return 1
	}
    }
}
   

proc offer-choices {} {
  global SYSTEM DISPLAYED_SYSTEMS SPACE_LEFT HISTORY
    set HISTORY [select-from-gates $HISTORY]
    update-history
    set to_offer [unresolved-systems $HISTORY]
    
    set DISPLAYED_SYSTEMS {}

    if ![catch "make-system-space"] {
	if { $to_offer == {}} {
	    
	    .coder.right.cdrframe.hist.buttons.select config -text "Next"
	    .coder.right.cdrframe.hist.buttons.select config -command {goto-next}

	} else {
	    .coder.right.cdrframe.hist.buttons.select config -text "Select"
	    .coder.right.cdrframe.hist.buttons.select config -command {select-action}
	    set col0 .coder.right.cdrframe.choices.col0
	    label $col0.msg -text "Choose from the following,\n then hit 'Select'." -relief raised
	    pack $col0.msg -side top
	    incr SPACE_LEFT($col0) -2
	    foreach sys $to_offer {
		display-system $sys [system-active-features $sys]
	    }
	}
    } else {
	global errorInfo
	puts "Error in make-system-space: $errorInfo"
    }
}


proc unresolved-systems {features} {
    # All gates resolved before entering this function 
    global SYSTEM FEATURE ROOT
    set unresolved {}
    set entered_systems {}
    set pendingSystems {}
    foreach feat $features {
	if { $feat != $ROOT } {
	    lappend entered_systems $FEATURE($feat,system)
	}
	concat! pendingSystems $FEATURE($feat,extenders)
    }
    
    # For pending systems eliminate duplicates and already entered
    set pendingSystems [delete-duplicates $pendingSystems]
    set pendingSystems [set-difference $pendingSystems $entered_systems]
    
    while { $pendingSystems != {}} {
	set sys [pop pendingSystems]
	if { $SYSTEM($sys,state) == "active" && [ec-satisfied-p $SYSTEM($sys,ec) $features]} {
	    lappend unresolved $sys 
	}
    }
    
    return $unresolved
}

proc select-from-gates {features} {
    global SYSTEM FEATURE ROOT
    set entered_systems {}
    set pendingSystems {}
    foreach feat $features {
	if { $feat != $ROOT } {
	    lappend entered_systems $FEATURE($feat,system)
	}
	concat! pendingSystems $FEATURE($feat,extenders)
    }

    # For pending systems eliminate duplicates and already entered
    set pendingSystems [delete-duplicates $pendingSystems]
    set pendingSystems [set-difference $pendingSystems $entered_systems]

    while { $pendingSystems != {}} {
	set sys [pop pendingSystems]
	if { [gate-p $sys] && $SYSTEM($sys,state) != "disabled" && [ec-satisfied-p $SYSTEM($sys,ec) $features]} {

	    # if a gate, choose one feature
	    set feat [choose-from-gate $sys]
	    lappend features $feat
	    foreach sys $FEATURE($feat,extenders) {
		if ![member $sys $pendingSystems] {
		    lappend pendingSystems $sys
		}
	    } 
	}
    }

    return $features
}
  

proc choose-from-gate {system} {
    global FEATURE SYSTEM
    foreach feature $SYSTEM($system,features) {
	if { $FEATURE($feature,state) == "active"} {
	    return $feature
	}
    }
}

proc make-system-space {} {
  global CHOOSER_COLS SPACE_LEFT  COL_HEIGHT SYSTEMS_ON_PAGE CODING_COL_WIDTH

  set SYSTEMS_ON_PAGE 0
  set choicespace .coder.right.cdrframe.choices
  if [winfo exists $choicespace] {
       destroy $choicespace
       pack forget .coder.right.cdrframe.hist
  }
    pack .coder.right.cdrframe.hist  -side left -anchor nw -padx 0.1c -fill y
  frame $choicespace
  pack $choicespace -side left -anchor nw -padx 0.1c -expand t -fill both

  tkwait visibility  .coder.right.cdrframe.choices
  set max_cols  [expr floor([winfo width $choicespace]/ $CODING_COL_WIDTH)]
  set COL_HEIGHT [expr int(floor([winfo height $choicespace]/25))]
  set consumed_width 0
  set col 0
  # consumed_width < $max_cols
  set CHOOSER_COLS {}
  while {$col < $max_cols} {
      set frame [frame .coder.right.cdrframe.choices.col$col -width [expr $CODING_COL_WIDTH - 5]]
      pack $frame -side left -anchor nw -padx 0.1c -expand 0
      lappend CHOOSER_COLS $frame
      set SPACE_LEFT($frame) $COL_HEIGHT
      incr col
  }
}

proc choose-parent-frame {space_needed} {
    global CHOOSER_COLS SPACE_LEFT
    
    foreach frame $CHOOSER_COLS {
	# work out how much
	if {$SPACE_LEFT($frame) >= $space_needed} {
	    set SPACE_LEFT($frame) [expr $SPACE_LEFT($frame) - $space_needed]
	    return $frame
	}
    }
    return 0
}

proc display-system {sysname choices} {
    global COL_HEIGHT
   
    set height [llength $choices]
    set space_needed [expr $height + 1]

    if { $space_needed > $COL_HEIGHT } {
	# Special case - this system doesnt fit a single column - split it.
	return [pack-system-in-multiple-frames $sysname $choices $space_needed]
    } else {
	return [pack-system-in-single-frame $sysname $choices $space_needed]
    }
}

proc pack-system-in-multiple-frames {sysname choices space_needed} {
    global COL_HEIGHT CHOOSER_COLS SPACE_LEFT DISPLAYED_SYSTEMS
    # Find consecutive columns with enough space.
    
    # find a column where space left + space following columns
    # is sufficient
    set cols [llength $CHOOSER_COLS]
    for {set col 0} {$col < [expr $cols - 1]} {incr col} {
	set frame [lindex $CHOOSER_COLS $col]
	set this_colspace $SPACE_LEFT($frame)
	
	# Allow one for the system, and at least one feature, and one border
	if {  $this_colspace < 3 } {continue}

	set next_col [expr $col + 1]
	set next_frame [lindex $CHOOSER_COLS $next_col]
	if {[expr $this_colspace + $SPACE_LEFT($next_frame)] >= $space_needed } {
	    # We can layout the system in two cols
	    layout-in-two-columns $sysname $col $next_col
	    lappend DISPLAYED_SYSTEMS $sysname
	    return 1
	} else {
	    puts "INSUFF SPACE $this_colspace / $SPACE_LEFT($next_frame) / $space_needed"
	    continue
	}
    }
    return 0
}

	


proc layout-in-two-columns {sysname col1 col2} {
    global CHOOSER_COLS SPACE_LEFT SYSTEM SYSTEMS_ON_PAGE MAX_LABEL_WIDTH

    incr SYSTEMS_ON_PAGE

    set features [system-active-features $sysname]
    # 1 Layout first column
    set col_frame [lindex $CHOOSER_COLS $col1]
    set this_frame "$col_frame.c$SYSTEMS_ON_PAGE"
    frame $this_frame

    #a) Layout the system name
    set label $sysname
    if {[string length $label] > $MAX_LABEL_WIDTH} {
	set label "[string range $label 0 [expr $MAX_LABEL_WIDTH -1]]..."
    }
    label $this_frame.labelaa -text $label -fg red
    pack $this_frame.labelaa
    

    #b) Layout the space_first features
    set items [expr $SPACE_LEFT($col_frame)-1]
    set features1 [lrange $features 0 [expr $items - 1]]
    pack-features-in-frame $features1 $this_frame $SYSTEMS_ON_PAGE
    set SPACE_LEFT($col_frame) 0
    
    # Layout the remaining features
    set col_frame [lindex $CHOOSER_COLS $col2]
    set this_frame "$col_frame.c$SYSTEMS_ON_PAGE"
    frame $this_frame
    set features1 [lrange $features $items end]
    pack-features-in-frame $features1 $this_frame $SYSTEMS_ON_PAGE
    set SPACE_LEFT($col_frame) [expr $SPACE_LEFT($col_frame) - [llength $features1]]

    # Make the default the first of the first list
    global RB_$SYSTEMS_ON_PAGE
    set RB_$SYSTEMS_ON_PAGE [lindex $features 0]

    lappend DISPLAYED_SYSTEMS $sysname $this_frame 
    return 1
}


proc pack-features-in-frame {choices frame sysnum} {
    global RB_$sysnum CHOICE_FONT PLATFORM  NODE CURRENT DOUB_TEST MAX_LABEL_WIDTH

    # if [info exists DOUB_TEST] {unset DOUB_TEST}
    foreach choice $choices {
	set displayname $choice
	if {[string length $displayname] > $MAX_LABEL_WIDTH} {
	    set displayname "[string range $displayname 0 [expr $MAX_LABEL_WIDTH -1]]..."
	}
	radiobutton $frame.$choice -text $displayname\
		-variable RB_$sysnum -relief flat -value $choice\
		-font $CHOICE_FONT -pady 0\
		-selectcolor white\
		-command "click-on-choice $choice"
	# checkbutton $frame.$choice -text $displayname\
		-variable RB_$sysnum -relief flat \
		-font $CHOICE_FONT -pady 0
	# bind $frame.$choice <Double-Button-1> "bind $frame.$choice <Button-1> {};double-click-select $choice"

	if { $PLATFORM == "windows"} {
	    set pad 0
	} else {
	    set pad 2
	}
	pack $frame.$choice -side top -pady $pad -anchor w
    }
    set previously_chosen [intersection $choices $NODE($CURRENT,tmp_choices)]
    if { $previously_chosen == {} } {
	# set the default value
	set RB_$sysnum [lindex $choices 0]
    } else {
	set RB_$sysnum $previously_chosen
    }
    pack $frame -pady 0.1c -anchor w
}


proc click-on-choice {feature} {
    global SELECT_ON_CLICK DISPLAYED_SYSTEMS AUTO_ADVANCE IGNORE_COMPLETE
    if $SELECT_ON_CLICK  {
	add-to-history $feature
	mark-edited
	offer-choices
	if { $DISPLAYED_SYSTEMS == {} && $AUTO_ADVANCE == 1  } {
	    goto-next $IGNORE_COMPLETE
	}
    }
}

proc pack-system-in-single-frame {sysname choices space_needed} {
    global  DISPLAYED_SYSTEMS SYSTEMS_ON_PAGE 

    set parent_frame [choose-parent-frame $space_needed ]
    
    if { $parent_frame == 0 }  {
	return 0
    }

    incr SYSTEMS_ON_PAGE
    set this_frame "$parent_frame.c$SYSTEMS_ON_PAGE"
    frame $this_frame

    set label $sysname
    if {[string length $label] > 15} {
	set label "[string range $label 0 15]..."
    }
    label $this_frame.labelaa -text $label -fg red
    pack $this_frame.labelaa
    pack-features-in-frame $choices $this_frame  $SYSTEMS_ON_PAGE
    lappend DISPLAYED_SYSTEMS $sysname
    return 1
}


proc select-action {} {
  global  SYSTEMS_ON_PAGE DISPLAYED_SYSTEMS AUTO_ADVANCE IGNORE_COMPLETE

  for {set x 1} {$x <= $SYSTEMS_ON_PAGE} {incr x} {
      global RB_$x
      set var "RB_$x"
      add-to-history  [expr $$var]
   }
   mark-edited
   offer-choices
   if { $DISPLAYED_SYSTEMS == {} && $AUTO_ADVANCE == 1  } {
       goto-next $IGNORE_COMPLETE
   }
}

proc double-click-select {feature} {
    global  DISPLAYED_SYSTEMS AUTO_ADVANCE IGNORE_COMPLETE DOUB_TEST
    # Doub_test is to stop the second of a 2 double clicks to exit
    if [info exists DOUB_TEST($feature)] {return}
    set DOUB_TEST($feature) 1
    update
    after 200
    add-to-history $feature
    mark-edited
    offer-choices
    if { $DISPLAYED_SYSTEMS == {} && $AUTO_ADVANCE == 1  } {
	goto-next $IGNORE_COMPLETE
    }
}

proc store-current-coding {} {
    global CURRENT NODE DISPLAYED_SYSTEMS SYSTEMS_ON_PAGE SYSTEM FEATURE HISTORY
    
    if { $CURRENT == {}} {
	# We have no current coding, exit
	return
    }
    set oldComment $NODE($CURRENT,comment)
    set NODE($CURRENT,comment) [get-coder-comment]
    if { $oldComment != $NODE($CURRENT,comment) } {
	mark-edited
    }

    set NODE($CURRENT,features) $HISTORY

    if [info exists DISPLAYED_SYSTEMS] {
	if { $DISPLAYED_SYSTEMS == {} } {
	    set NODE($CURRENT,completed-p) 1
	} else {
	    set NODE($CURRENT,completed-p) 0
	     # Store the unsaved choices
	    set choices {}
	    for {set x 1} {$x <= $SYSTEMS_ON_PAGE} {incr x} {
		global RB_$x
		set var "RB_$x"
		set choice [expr $$var]

		# Only store nondefault choices
		set system $FEATURE($choice,system)
		if { [lindex [system-active-features $system] 0] != $choice} {
		    lappend choices $choice
		}
	    }
	    set NODE($CURRENT,tmp_choices) $choices
	}
    }
}


proc delete-from-history {feature} {
    global HISTORY ROOT FEATURE SYSTEM

    if { $feature == $ROOT} {
	set HISTORY [list $ROOT]
	return
    }

    delete HISTORY $feature

    foreach system $FEATURE($feature,extenders) {
	set feat2 [car [intersection $HISTORY [system-active-features $system]]]
	if { $feat2 != {}} {
	    if ![ec-satisfied-p $SYSTEM($system,ec) $HISTORY] {
		delete-from-history $feat2
	    }
	}
    }

    if { $HISTORY == {} } {set HISTORY [list $ROOT]} 
    return $HISTORY
}

