######################################
## Systemic Coder
#
#  File: CoderWindow.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


###############Make-Coder-Interface#######################

proc install-coder {} {
    set-counts
    install-current-coding
}


proc make-coder-frame {} {
    global WINHEIGHT WINWIDTH DIR
    set WINHEIGHT [winfo height .]
    set WINWIDTH  [winfo width .]
    if {[winfo exists .coder]} {destroy .coder}
    frame .coder 
    make-coder-toolbar
    pack .coder.tbar -side left  -padx 5 -anchor n -fill y

    frame .coder.right
    make-textframe
    make-commentframe 
    make-historyframe 

    pack .coder.right -side right -fill both -expand t -pady 0.1c
    pack .coder.right.cdrframe -side bottom  -anchor nw -pady 0.1c -fill both -expand t  -padx 0.1c
    pack .coder.right.textframe .coder.right.cmtframe -side top\
	    -anchor nw -pady 0.1c -fill x   -padx 0.1c

     bind . <Configure> { 
	catch {after cancel $config_id} 
	set config_id [after 500 catch-resize-window] 
    }
    return .coder
}

proc catch-resize-window {} {
    global CURRENT_INTERFACE WINHEIGHT WINWIDTH
    bind . <Configure> {}
    if { $CURRENT_INTERFACE == "coder"} {
	if { $WINHEIGHT != [winfo height .] || $WINWIDTH  != [winfo width .] } {
	    store-current-coding
	    install-current-coding
	    update
	    set WINHEIGHT [winfo height .]
	    set WINWIDTH  [winfo width .]
	}
    }
    bind . <Configure> { 
	catch {after cancel $config_id} 
	set config_id [after 400 catch-resize-window] 
    }
}


###############ToolBar#######################

proc make-coder-toolbar {} {
    frame .coder.tbar 

    Toolbar .coder.tbar.motion vertical "Goto"
    ToolbarItem .coder.tbar.motion "Previous"\
	    "Click here to goto the text item before this one" {goto-previous}
    ToolbarItem .coder.tbar.motion "Next"\
	    "Click here to goto the text item before this one" {goto-next}
    ToolbarItem .coder.tbar.motion "Next Incomplete"\
	    "Click here to goto the next text item with features uncoded" {goto-next-incomplete}
    ToolbarItem .coder.tbar.motion "First"\
	    "Click hereto go to the first (unignored) coding" {goto-first}

    Toolbar .coder.tbar.action vertical "Actions"
    ToolbarItem .coder.tbar.action "Ignore"\
	    "Click here to ignore this coding"  {ignore-current-coding} 
    #    ToolbarItem .coder.tbar.action "Update" "" {update-codings}
    
    label .coder.tbar.count -text "No codings loaded"  -bg white\
	    -font "Times 12" -borderwidth 1 -relief sunken
    add-rollover-message .coder.tbar.count\
       "This space displays the current coding index, and the total number of codings"
    pack .coder.tbar.count -side top -pady 25
    pack .coder.tbar.motion .coder.tbar.action -pady 0.2c


}

###############TextFrame#######################

proc make-textframe {} {
    global TEXT_LINES TEXT_FONT_SIZE
    frame .coder.right.textframe -borderwidth 1 -relief sunken
    label .coder.right.textframe.label -text "Text:  " 
    scrolled-text .coder.right.textframe.text -width 80 -height $TEXT_LINES -bg white\
	    -borderwidth 1 -relief sunken -font "Times $TEXT_FONT_SIZE"
    add-rollover-message .coder.right.textframe.text\
	    "Displays The current text (red) with context (black)"
    pack .coder.right.textframe.label -padx 9.6 -side left
    pack .coder.right.textframe.text -side left  -fill x -expand t -padx 4 -padx 0.1c
}

###############COMMENT#######################

proc make-commentframe {} {
    
    frame .coder.right.cmtframe  -borderwidth 1 -relief sunken
    label .coder.right.cmtframe.label -text "Comment:" 
    scrolled-text .coder.right.cmtframe.text -width 80 -height 4  -bg white \
	    -borderwidth 1 -relief sunken
    add-rollover-message .coder.right.cmtframe.text\
	    "Any comments entered here will be kept for your later reading"
    pack .coder.right.cmtframe.label -padx 2 -side left
    pack .coder.right.cmtframe.text -side left  -fill x -expand t  -padx 0.1c
}

###############HISTORY#######################

proc make-historyframe {} {
    global ROOT HISTORY
    frame .coder.right.cdrframe  -relief sunken -borderwidth 3 
    
    frame .coder.right.cdrframe.hist
    label .coder.right.cdrframe.hist.label -text "Selected Features" 
    scrolled-listbox .coder.right.cdrframe.hist.table -width 20 -height 12 -bg white

    add-rollover-message .coder.right.cdrframe.hist.table\
       "Selected Features (double-click to delete choices)"
    Toolbar .coder.right.cdrframe.hist.buttons horizontal
    ToolbarItem .coder.right.cdrframe.hist.buttons "Select" "" {select-action} 
    add-rollover-message .coder.right.cdrframe.hist.buttons\
       "Click here to select all marked features"
    pack .coder.right.cdrframe.hist.buttons -side top -pady 5
    pack .coder.right.cdrframe.hist.label 
    pack .coder.right.cdrframe.hist.table -side top -fill y -expand t
    pack .coder.right.cdrframe.hist -padx 2 -side left -anchor nw -fill y -expand t
    bind .coder.right.cdrframe.hist.table.choices <ButtonRelease-1> {
	set choice [.coder.right.cdrframe.hist.table.choices get @%x,%y]
	delete-from-history $choice 
	mark-edited
	offer-choices 
    }
}


###############Field Set/Get#######################

proc set-coder-text {text nid} {
    global NODE
    .coder.right.textframe.text.text config -state normal
    .coder.right.textframe.text.text delete 1.0 end
    .coder.right.textframe.text.text insert 1.0 $text coltag
    .coder.right.textframe.text.text config -state disabled

    # If this coding is ignored, use color gray
    if { $NODE($nid,ignore) == 1} {
	set color gray
    } else {
	set color black
    }
    .coder.right.textframe.text.text tag configure coltag -foreground $color
}

proc set-coder-context {nid} {
    global NODE TEXT_NODES SHOW_CONTEXT
    .coder.right.textframe.text.text config -state normal
    .coder.right.textframe.text.text delete 1.0 end

    # If this coding is ignored, use color gray
    if { $NODE($nid,ignore) == 1} {
	set color gray
    } else {
	set color red
    }

    set pos  [node-position $nid] 
    set start [expr $pos - $SHOW_CONTEXT]
    set end [expr $pos + $SHOW_CONTEXT]
    if {$end > [expr [llength $TEXT_NODES] -1]} {set end [expr [llength $TEXT_NODES] -1]}
    for {set x $start} {$x<=$end} {incr x} {
      if  {$x >= 0} {
	  set id [nth-coding $x]
	  if { $x == $pos } {
	      .coder.right.textframe.text.text yview end
	      .coder.right.textframe.text.text insert end  "$NODE($id,textstripped) " {this}
	      .coder.right.textframe.text.text tag configure this -foreground $color
        } else {
           .coder.right.textframe.text.text insert end "$NODE($id,textstripped) " 
        }
      }
  }
  .coder.right.textframe.text.text config -state disabled
}

proc get-coder-text {} {
  return [.coder.right.textframe.text.text get 1.0 end]
}

proc set-lines {val} {
  global TEXT_LINES 
  set TEXT_LINES $val
  .coder.right.textframe.text.text config -height $val
}

proc set-font {val} {
  global TEXT_FONT_SIZE
  set TEXT_FONT_SIZE $val
  .coder.right.textframe.text.text config -font "Times $val"
}


proc set-coder-comment {text} {
  .coder.right.cmtframe.text.text delete 1.0 end
  .coder.right.cmtframe.text.text insert 1.0 $text
}

proc get-coder-comment {} {
  return [string trim [.coder.right.cmtframe.text.text get 1.0 end]]
}

proc set-counts {} {
  global TEXT_NODES CURRENT
    set indx [node-position $CURRENT]
  .coder.tbar.count config -text "Item [incr indx] of [llength $TEXT_NODES]"
}



proc get-history {} {
    global HISTORY
    return $HISTORY
}

proc add-to-history {feature} {
    global HISTORY
    lappend HISTORY $feature
}

proc update-history {} {
    global HISTORY
    ScrolledListBoxSet .coder.right.cdrframe.hist.table $HISTORY
}



###############CODER#######################

proc activate-chooser-buttons {} {
  .coder.right.cdrframe.hist.buttons.select configure -state normal
  .coder.right.cdrframe.hist.buttons.next configure -state normal
}

proc deactivate-chooser-buttons {} {
  .coder.right.cdrframe.hist.buttons.select configure -state normal
  .coder.right.cdrframe.hist.buttons.next configure -state normal
}


proc ignore-current-coding {} {
    global CURRENT 

    # Check what its state is
    switch -- [.coder.tbar.action.ignore cget -text] {
	"Ignore" {
	    ignore-coding $CURRENT
	    goto-next
	}
    
	"Unignore" {
	    unignore-coding $CURRENT
	    install-coding $CURRENT
	}
    }
}
