######################################
## Systemic Coder
#
#  File: CourseWindow.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.



# This loads an additional (first) interface which offers a series
# of lessons. 

# Each lesson loads a coding file, (copied first to the users space)
# Then tells them what to do.

# May add an extra permanent space for comments.

proc make-lessons-frame {} {
    global LESSONDIR 
   
    set font1 [font create -size 18]
    set font2 [font create -size 14 -underline 1]
    if [winfo exists .lessons] {destroy .lessons}
    frame .lessons 
    label .lessons.title -text "Available Exercises:" -font $font1
    pack .lessons.title -side top -anchor nw -pady 0.3c -padx 0.2c
    foreach file [lsort [glob -nocomplain "$LESSONDIR/*.cd2"]] {
	set root [file rootname [file tail $file]]
	set path .lessons.[label-to-name $root]
	frame $path
	pack $path -side top -anchor nw -padx 3c
	label $path.blip -text "o"
	regsub -all "_" $root " " root
	label $path.label -text $root -font $font2
	pack $path.blip $path.label -side left
	pack $path -side top -anchor nw -padx 3c
	bind $path.label <Enter> {}
	bind $path.label <Leave> {}
	regsub -all " " $file "\ " file
	bind $path.label <ButtonRelease> "load-lesson \"$file\""
    }

    Toolbar .lessons.buttons -horizontal
    ToolbarItem .lessons.buttons "Start Exercise" "Click here to start your selected exercise"\
	    {start-exercise}
    ToolbarItem .lessons.buttons "Quit" "Click here to quit the program"\
	    {Quit}
    pack .lessons.buttons -side bottom -pady 0.1c -anchor n


    label .lessons.glosslabel -text "Description:"
    pack .lessons.glosslabel -side top -padx 0.2c -pady 0.3c -anchor nw
    scrolled-text .lessons.gloss
    pack .lessons.gloss -side top -padx 0.2c -expand t -fill both

    return .lessons

}

proc start-exercise {} {
    install-interface segment
}

proc load-lesson {file} {
    global DIR
    if ![file exists "$DIR/Work"] {
	file mkdir "$DIR/Work"
    }
    
    set file2 [file rootname [file tail "$file"]]
    set indx 1
    while { [file exists [file join $DIR Work "$file2$indx.tcl"]] == 1 } {
	incr indx
    }
    set file2 [file join $DIR Work "$file2$indx.tcl"]
    file copy "$file" "$file2"
    
    tell-user "Your copy of the exercise file is stored at:\n [file nativename "$file2"]"

    describe-exercise "$file"

    load-codings "$file2"
    install-interface lessons
}
    
proc describe-exercise {file} {
    global LESSON_GLOSS
    set descrfile [file rootname "$file"].txt
    if [file exists "$descrfile"] {
	gloss-exercise [read-file "$descrfile"]
    } else {
	gloss-exercise "No description provided"
    }
}
	
proc gloss-exercise {text} {
    .lessons.gloss.text delete 1.0 end
    .lessons.gloss.text insert end $text
    update
}

# 
