######################################
## Systemic Coder
#
#  File: DocumentSets.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


Menu .menuBar "Document Set"
MenuItem .menuBar "Document Set" "Create New Set" {create-doc-set}
MenuItem .menuBar "Document Set" "Load Existing Set" {use-doc-set}
MenuSeparator .menuBar "Document Set" 
MenuItem .menuBar "Document Set" "Add/Delete Files" {edit-doc-set}
MenuItem .menuBar "Document Set" "Refresh Data" {load-doc-set}
MenuItem .menuBar "Document Set" "About Document Sets" {help docsets}

MenuItemDisable .menuBar "Document Set" "Add/Delete Files" 
MenuItemDisable .menuBar "Document Set" "Refresh Data" 


proc create-doc-set {} {
    global DOCUMENT_SET DS_REFRESH_NEEDED  DS_FILE

    if { [prompt-save-current-files] == "cancelled" } {return cancelled} 

     # Grab a name for the Doc Set
    set file [tk_getSaveFile -initialdir [default-directory analyses] -initialfile "untitled.dst"]
    if { $file == {}} {return "cancelled"}

    set DS_FILE $file
    set DOCUMENT_SET {}
    set DS_REFRESH_NEEDED 0
    disable-normal-coder
    edit-doc-set
}

proc use-doc-set {{file {}}} {
    global DOCUMENT_SET DS_REFRESH_NEEDED DS_FILE
    if { [prompt-save-current-files] == "cancelled" } {return cancelled} 

     # Grab a name for the Doc Set
    
    if { $file == {}} {
	set file [tk_getOpenFile -initialdir [default-directory analyses] -initialfile "untitled.dst"]
    }
    if { $file == {}} {return}

    disable-normal-coder
    set DS_FILE $file
    set DOCUMENT_SET [delete-nulls [split [read-file $file] "\n"]]
    set DS_REFRESH_NEEDED 1
    load-doc-set
    # Change the file display to show this file
}


proc disable-normal-coder {} {
    # catch "hideToolbarItem .top.toolbar Coding"
    catch "hideToolbarItem .top.toolbar Text"
    MenuItemEnable .menuBar "Document Set" "Add/Delete Files" 
    MenuItemEnable .menuBar "Document Set" "Refresh Data" 
    # Save disabled also
    MenuItemDisable .menuBar "Codings" "Save Codings" 
    # Review click also
    # bind  .review.main.table.choices <Double-Button-1> {}
}

proc enable-normal-coder {} {
    global DOCUMENT_SET
    unset DOCUMENT_SET
    packToolbarItemBefore .top.toolbar Coding Review
    packToolbarItemBefore .top.toolbar Text Scheme
    MenuItemDisable .menuBar "Document Set" "Add/Delete Files"
    MenuItemDisable .menuBar "Document Set" "Refresh Data" 
    # Save disabled also
    MenuItemEnable .menuBar "Codings" "Save Codings" 
    # Review click also
    bind  .review.main.table.choices <Double-Button-1>  +{review-edit-current}
}

proc save-doc-set {} {
    global DOCUMENT_SET  DS_FILE
    if [file exists $DS_FILE] {file delete $DS_FILE}

    set str [open $DS_FILE w]
    foreach file $DOCUMENT_SET {
	puts $str $file
    }
    close $str
}


proc load-doc-set {} {
    global DOCUMENT_SET DS_REFRESH_NEEDED DS_DOCUMENT_FEATURES FEATURE
    
    if [info exists DS_NODES] {unset DS_NODES}
    clear-codings

    # Load in the Scheme of the first file
    load-scheme-from-codings-file [car $DOCUMENT_SET]
    build-ordered-features-list

    set DS_DOCUMENT_FEATURES {}
    set count 0
    foreach file $DOCUMENT_SET {
	add-docset-file $file [incr count]
    }

    # Unignore any document features 
    if { $DS_DOCUMENT_FEATURES != {}} {
	foreach feature $DS_DOCUMENT_FEATURES {
	    set FEATURE($feature,state) active
	}
	graph-scheme
    }

    set DS_REFRESH_NEEDED 0
    
}

# NEED TO DISABLE SAVE FILE AS WELL


proc add-docset-file {file filenum} {
    global XMLSEG LASTSEG CURRSEG NODE DS_NODES UNUSED_TAGS LASTTAG TEXT_NODES ORDERED_FEATURES\
	    DS_DOCUMENT_FEATURES

    set DS_NODES($filenum) {}
    set allNew {}

    set codingsform [load-xml-forms $file]

    # Handle any document features
    foreach form [get-xml-elements $codingsform {codings header docfeature}] {
	set val [second [car [second $form]]]
	lappend-new DS_DOCUMENT_FEATURES $val
    }

    ################################
    # PARSE the body
    set body [get-xml-element $codingsform {codings body}]
   
    #3. Load in segment defins
    foreach seg [get-xml-elements $body {body segment}] {
	if {[get-xml-element $seg {segment ignore}] != 1} {
	    set id [new-node]
	    set features [get-xml-element $seg {segment features}]
	    set features [ordered-features $features]
	    set NODE($id,features) [select-from-gates $features]
	    set newFeatures [set-difference $features [concat $NODE($id,features) $allNew]]
	    if { $newFeatures != {} } {
		concat! allNew $newFeatures
	    }
	    set NODE($id,comment)   [xml-decode [get-xml-element $seg {segment comment}]]
	    lappend DS_NODES($filenum) $id
	    lappend TEXT_NODES $id
	    set text [xml-decode [lindex $seg 3]]
	    set NODE($id,text) $text
	    set NODE($id,textstripped) [strip-coder-text $text]
	set NODE($id,file) $file
	    calculate-node-completeness $id
	}
    }

    if { $allNew != {}} {
	tell-user "This file has features not in network!: $allNew"
    }
}



defhelp docsets\
"DOCUMENT SETS
=============

  Document Sets allow you to load in a set of codings files, with the same scheme, and perform statistical operations on all segments in these files.

CREATE NEW SET: Use this option to start a document set. You will be asked to provide a filename for the document set (for re-loading the set later), and also to specify the coding files which belong to the set.

USE EXISTING SET: You will be asked to select a document set file (.dst). The document set will be loaded in.

ADD/DELETE FILES: This option allows you to add or delete files from the document set.

REFRESH DATA:  Once a Document Set is loaded, you cannot edit the codings. If you wish to edit codings while a document set is open, open a second copy of the Coder, and edit the coding files in that program. Within the program with the document set, use the \"Refresh Data\" option to reload the changed files."



# Disable the menu item which loads this
MenuItemDisable .menuBar "Options" "Enable Document Sets"



proc edit-doc-set {} {
    global DOCUMENT_SET DS_FILE TMP_DOCUMENT_SET
    if [winfo exists .editdocset] {destroy .editdocset}
    
    
    modal-dialog .editdocset
    wm title .editdocset  "Document Set"

    label .editdocset.title -text "Document Set: $DS_FILE"
    frame .editdocset.main
    Toolbar .editdocset.main.tbar
    ToolbarItem .editdocset.main.tbar "Add File" "Add another file to the document set" ds-add-document
    ToolbarItem .editdocset.main.tbar "Remove File" "Removes the currently selected file from the document set"\
	    ds-remove-document
    ToolbarItem .editdocset.main.tbar "Done" "Save" dsedit-done
    ToolbarItem .editdocset.main.tbar "Cancel" "Add another file to the document set" {return-from-modal .editdocset 0}

    set TMP_DOCUMENT_SET $DOCUMENT_SET
    scrolled-listbox .editdocset.main.files -height 8 -width 40  -bg white
    ScrolledListBoxSet .editdocset.main.files $TMP_DOCUMENT_SET
    pack .editdocset.main.tbar .editdocset.main.files -side left -expand t -fill both
    pack .editdocset.title .editdocset.main -side top -fill both -expand t

    ScrolledListBoxSelect .editdocset.main.files 0

    post-modal .editdocset
}




proc ds-add-document {} {
    global TMP_DOCUMENT_SET DS_FILE 
    set initdir [file dirname $DS_FILE]
    set cands [glob -nocomplain [file join $initdir "*.cd?"]]
    set cands [set-difference $cands $TMP_DOCUMENT_SET]
    if {$cands != {}} {
	set initfile [car $cands]
    } else {
	set initfile "*.cd3"
    }
    set file [tk_getOpenFile -initialdir $initdir -initialfile $initfile\
	    -filetypes [file-load-types]]

    if { $file == {}} {return}
    lappend TMP_DOCUMENT_SET $file
    
    # Version 8.2.x of Tcl doesnt allow the listvar field
    ScrolledListBoxAdd .editdocset.main.files $file
    ScrolledListBoxSelect .editdocset.main.files 0
    update
}

proc dsedit-done {} {
    global  TMP_DOCUMENT_SET DOCUMENT_SET DS_REFRESH_NEEDED
    if { $DOCUMENT_SET != $TMP_DOCUMENT_SET} {
	set DOCUMENT_SET $TMP_DOCUMENT_SET
	set DS_REFRESH_NEEDED 1
	save-doc-set
    }
    return-from-modal .editdocset 1
}

proc ds-remove-document {} {
    global TMP_DOCUMENT_SET
    set file [ScrolledListBoxSelected .editdocset.main.files]
    delete TMP_DOCUMENT_SET $file
    ScrolledListBoxSet .editdocset.main.files $TMP_DOCUMENT_SET
    ScrolledListBoxSelect .editdocset.main.files 0
    update
}

