######################################
## Systemic Coder
#
#  File: GrapherAdds.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.

proc correct-after-ec-change {system newec} {
    global SYSTEM TEXT_NODES NODE
    # now, correct the codings for this case
    # Simple EC, 

    set features $SYSTEM($system,features)
    foreach nid $TEXT_NODES {
	# if the new system is selected from
	# and the ec is not, then:
	# - select it if this is compat
	# - deselect the extenders elsewise
	if [intersect-p $features $NODE($nid,features)] {

	    if { [ec-satisfied-p $newec $NODE($nid,features)] == 0} {
		if [compat $newec $NODE($nid,features)] {
		    if [simple-ec-p $newec] {
			lappend NODE($nid,features) $newec
		    } else {
			if { [car $newec] == "and" } {
			    foreach ec [cdr $newec] {
				if [simple-ec-p $ec] {
				    lappend NODE($nid,features) $ec
				}
			    }
			}
		    }
		} else {
		    set choice [first [intersection $features $NODE($nid,features)]] 
		    set NODE($nid,features) [delete-from-path $choice $NODE($nid,features)]
		}
		set NODE($nid,features) [select-from-gates $NODE($nid,features)]
	    }
	}
    }

    # Check all nodes to see if they are complete in the new scheme
    # Also checks to see if new gates are enterable
    calculate-all-node-completeness resolve_gates
}


proc delete-from-path {feature path} {
    global HISTORY ROOT
    set HISTORY $path
    delete-from-path1 $feature
    if { $HISTORY == {} } {return "$ROOT"} 
    return $HISTORY
}


proc delete-from-path1 {choice} {
  global FEATURE SYSTEM HISTORY
  delete HISTORY $choice
  foreach system $FEATURE($choice,extenders) {
    set sys_feats $SYSTEM($system,features)
    foreach feature $HISTORY {
       if [member $feature $sys_feats] {
         delete-from-path1 $feature
       }
    }
  }
}

proc coder-change-feature-system {feature system} {
    
    global TEXT_NODES NODE 
    
    # Ensure that the codings are consistent with the move
    
    # newpath is preferred
    set newPath [definite [path $feature]]
    
    foreach nid $TEXT_NODES {
	set features $NODE($nid,features)
	
	if [member $feature $features] {
	    set path  [path $features]
	    if { $path == {} } { 
		# The path is now inconsistent
		# Keep this feature, but abandon all inconsistent ones
		set newfeatures $newPath
		foreach feature1 $features  {
		    if ![member $feature1 $newfeatures] {
			if { [path [list and $feature1 $feature]] != {} } {
			    lappend newfeatures $feature1
			}
		    }
		    set NODE($nid,features) $newfeatures
		}
	    } else {
		set NODE($nid,features) [definite $path]
	    }
	}
	set NODE($nid,features) [select-from-gates $NODE($nid,features)]
	calculate-node-completeness $nid
    }
}


