######################################
## Systemic Coder
#
#  File: Interface.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


# Coder-Specific Code for setting up the Coder Interface

proc make-coder-interface {} {
    reset-interface
    make-coder-modebar
    reset-all
    install-interface scheme
}

proc make-coder-modebar {} {
    def-interface segment "Text" "Segment and Edit your Text"\
	    [make-segment-frame]  install-segmenter uninstall-segmenter
    def-interface scheme  "Scheme" "Define and modify the Coding Scheme"    [make-scheme-frame] 
    def-interface coder   "Coding" "Assign features to each segment of text"\
	    [make-coder-frame] install-coder store-current-coding
    def-interface review  "Review"  "Examine your codings"   [make-review-frame] install-review uninstall-review
    def-interface stats   "Statistics" "Perform statistical tests on your codings"\
	    [make-stats-frame] install-stats uninstall-stats
    
    FileBarAdd .top.filebar.cfile CODINGS_FILE "Codings File"
    FileBarAdd  .top.filebar.sfile GRAMMAR_FILE "Scheme File"
}



proc scroll-current-interface {delta event} {
    global CURRENT_INTERFACE


      if { $event > 1024 } {
	  set orient xview
      } else {
	  set orient yview
      }

    switch -- $CURRENT_INTERFACE {
	segment {
	    .segment.editor.text $orient scroll [expr (0 - $delta) / 120] units
	}

	scheme {
	    global GCANV 
	  
	    $GCANV $orient scroll [expr (0 - $delta) / 120] units
	}

	coder {
	    
	}

	review {
	    .review.main.table.choices $orient scroll [expr (0 - $delta) / 120] units
	}
	stats {
	    .stats.main.table.leftframe.choices $orient scroll [expr (0 - $delta) / 120] units
	}
    }
}


proc start-coder {{openfile {}}} {
    # Sets up the buttons at the bottom of the Open dialog

    if { $openfile != {}} {
	destroy .c .credits
	make-coder-interface
	switch -- [string tolower [file extension $openfile]] {
	    ".cd2" -
	    ".cd3" {
		load-codings $openfile
	    }

	    ".dst" {
		global SRC
		source "$SRC/DocumentSets.tcl"
		use-doc-set $openfile
	    }

	    ".scheme" {
		load-scheme $openfile
	    }

	    default {
		tell-user "Sorry, dont know how to opn files of type:  [file extension $openfile]"
	    }
	}
	return
    }
		
    Toolbar .otbar horizontal
    pack .otbar -side bottom
    
    ToolbarItem .otbar "Import Text File"  ""  {
	destroy .c .otbar .credits
	make-coder-interface
	import-text
    } 2
	    
    ToolbarItem .otbar "Load Codings" ""  {
		destroy .c .otbar .credits
		make-coder-interface
		load-codings
    } 2
	
	if { 1== 0} {
	    ToolbarItem .otbar "Load Document Set" ""  {
		global SRC DOCUMENT_SET
		destroy .c .otbar .credits
		make-coder-interface
		source "$SRC/DocumentSets.tcl"
		use-doc-set
		if [info exists DOCUMENT_SET] {
		    load-doc-set
		}
	    } 2
	    }    
    ToolbarItem .otbar "Quit" ""  {exit} 2
}
		
		

