### GRAPHIC PRIMITIVES ###########################

proc draw-text {window txt x y {options {}}} {
  eval {$window create text} $x $y\
       {-text $txt -anchor nw }\
      $options
}

proc draw-line {window x1 y1 x2 y2 {options {}}} {
    eval {$window create line} $x1 $y1  $x2 $y2\
      $options
}

proc draw-line-between {window p1 {p2 {}}} {
 eval [concat "draw-line" $window $p1 $p2]
}

proc draw-arrow-between {window p1 {p2 {}}} {
 eval [concat "draw-line" $window $p1 $p2]
}

proc draw-rect {window x1 y1 x2 y2 {options {}}} {
  eval [concat "$window create rect $x1 $y1  $x2 $y2 " $options]
}

proc draw-rect-between {window p1 {p2 {}}} {
 eval [concat "draw-rect" $window $p1 $p2]
}

proc draw-arc {window points} {
  set cmd "$window create line"
  set options {-tag line -joinstyle round -smooth true}
  eval [concat $cmd $points $options]
}


proc draw-open-curly-bracket {win xmin ymin xmax ymax} {
    set xmid [average $xmax $xmin]
    set ymid [average $ymax $ymin]
    set y1 [expr $ymin + 5]
    set y2 [average $ymid $ymin]
    set y3 [expr $ymid - 5]
    set y4 [expr $ymid + 5]
    set y5 [average $ymax $ymid]
    set y6 [expr $ymax -5]
    set x1a [expr $xmid - 5]
    set x1b [expr $xmid + 5]
    set y1a [expr $ymid - 5]
    set y1b [expr $ymid + 5]
    set y1a [expr $ymid - 5]
    set y1b [expr $ymid + 5]
    draw-arc $win [list $xmin $ymid $x1a $y3 $xmid $y2]
    draw-arc $win [list $xmid $y2 $x1b $y1 $xmax $ymin]
    draw-arc $win [list $xmin $ymid $x1a $y4 $xmid $y5]
    draw-arc $win [list $xmid $y5 $x1b $y6 $xmax $ymax]
}

proc draw-close-curly-bracket {win xmin ymin xmax ymax} {
    set xmid [average $xmax $xmin]
    set ymid [average $ymax $ymin]
    set y1 [expr $ymin + 5]
    set y2 [average $ymid $ymin]
    set y3 [expr $ymid - 5]
    set y4 [expr $ymid + 5]
    set y5 [average $ymax $ymid]
    set y6 [expr $ymax -5]
    set x1a [expr $xmid - 5]
    set x1b [expr $xmid + 5]
    set y1a [expr $ymid - 5]
    set y1b [expr $ymid + 5]
    set y1a [expr $ymid - 5]
    set y1b [expr $ymid + 5]
    draw-arc $win [list $xmax $ymid $x1b $y3 $xmid $y2]
    draw-arc $win [list $xmid $y2 $x1a $y1 $xmin $ymin]
    draw-arc $win [list $xmax $ymid $x1b $y4 $xmid $y5]
    draw-arc $win [list $xmid $y5 $x1a $y6 $xmin $ymax]
}

proc add-points {p1 p2} {
  list [expr [lindex $p1 0] + [lindex $p2 0]]\
       [expr [lindex $p1 1] + [lindex $p2 1]]
}

proc subtract-points {p1 p2} {
  list [expr [lindex $p1 0] - [lindex $p2 0]]\
       [expr [lindex $p1 1] - [lindex $p2 1]]
}

proc mid-point {p1 p2} {
  add-points $p1 [halve-point [subtract-points $p2 $p1]]
}

proc halve-point {p1} {
  list [expr [lindex $p1 0] / 2]\
       [expr [lindex $p1 1] / 2]
}


proc average {x2 x1} {
    return [expr ($x1 + $x2) / 2]
}
    

proc screen-coords {item canvas} {
# Returns the screen coordes of a canvas item
  set screencorrect "[winfo rootx $canvas] [winfo rooty $canvas]"
  set coords  [$canvas coords $item] 

  set scrollcorrection "[$canvas canvasx 0]\
                  [$canvas canvasy 0]"
  return [add-points [subtract-points $coords $scrollcorrection]\
                     $screencorrect]
}

proc move-item {window item xdelta ydelta} {
  $window move $item $xdelta $ydelta
}



proc intersection-point {line rect} {
    # Tells where the line first intersects the rectangle
    # assumes line starts above rect and ends below 
    # returns {} of no intersection
    
    set lx1 [lindex $line 0]
    set ly1 [lindex $line 1]
    set lx2 [lindex $line 2]
    set ly2 [lindex $line 3]

    set rx1 [lindex $rect 0]
    set ry1 [lindex $rect 1]
    set rx2 [lindex $rect 2]
    set ry2 [lindex $rect 3]

    # Assume intersection on the top line - so the intersect y is the top of the box

    # To find x int. point, find what proportion of the line is covered on the y axis, 
    #     and use the same proportion on the x axis
    set iy $ry1
    set proport [expr 1.0000 * ($iy - $ly1) / ($ly2 - $ly1) ]
    set ix [expr $lx1 + (($lx2 - $lx1) * $proport)]
    if { $ix >= $lx1 && $ix <= $lx2 } {
	return [list $ix $iy]
    }

    # If we get here, the x point was not within the rectangle,
    # Look for an intersection on the side of the box

    if { $lx1 < $rx1 } {
	# Intersection on the lhs
	set ix $rx1
	set proport [expr 1.0000 * ($ix - $lx1) / ($lx2 - $lx1) ]
	set iy [expr $ly1 + (($ly2 - $ly1) * $proport)]
	if { $iy >= $ly1 && $iy <= $ly2 } {
	    return [list $ix $iy]
	} else {
	    # no intersection
	    return {}
	}
    }

    # Intersection on rhs
    set ix $rx2
    set proport [expr 1.0000 * ($ix - $lx1) / ($lx2 - $lx1) ]
    set iy [expr $ly1 + (($ly2 - $ly1) * $proport)]
    if { $iy >= $ly1 && $iy <= $ly2 } {
	return [list $ix $iy]
    } else {
	# no intersection
	return {}
    }
}

