######################################
## Systemic Coder
#
#  File: LoadLisp.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


####################################
## Loading of lisp format (Coder2.x) 

proc load-codings1 {file} {

    start-watch-cursor

    #1. Prompt for and load the scheme file
    load-lisp-scheme
    #2. Load the codings file
    load-lisp-codings $file

    stop-watch-cursor
}

proc load-lisp-scheme {} {
    global SCHEMES
    set file [tk_getOpenFile -initialdir $SCHEMES -initialfile ""\
	    -title "Please select a Scheme file"]

    if { [file extension  $file] == ".scheme"} {
	load-scheme $file
	find-root
	return
    }
    
    set text [read-file $file]

    # Make all features lower case by making file LC
    # (systems will be made upper case automatically in defsystem)
    set text [string tolower $text]

    set lines [split $text "\n"]
    set text ""
    foreach line $lines {
	set pos [string first ";" $line]
	if { [string first "in-package" $line] != -1 } {continue}
	if { $pos == -1 } {
	    append text "$line "
	} else {
	    append text  "[string range $line 0 [expr $pos - 1]] "
	}
    }

    regsub -all {:entry-condition} $text " " text
    regsub -all {\[} $text "-" text
    regsub -all {\]} $text "-" text
    regsub -all {:features} $text " " text
    regsub -all {:name} $text " " text
    regsub -all {\(defsystem} $text "\ndefsystem" text
    regsub -all {\)\)} $text "\}" text
    regsub -all {\(} $text "\{" text    
    regsub -all {\)} $text "\}" text
    while { [string first "  " $text] > -1 } {
	regsub -all {  } $text " " text
    }

    # remove lines following an #+ignore
    set lines [split $text "\n"]
    set ignore_next 0
    foreach line $lines {
	set pos [string first "#+ignore" $line]
	if { $pos == -1 } {
	    if {$ignore_next == 0} {eval $line}
	    set ignore_next 0
	} else {
	    if {$ignore_next == 0} {eval [string range $line 0 [expr $pos - 1]]}
	    set ignore_next 1
	}
    }
    find-root
    return
    
}
    

proc load-lisp-codings {file} {
    global NODE ALL_FEATURES
    set text [read-file $file]
    regsub {\{} $text "<" text
    regsub {\}} $text ">" text
    set lines [split $text "\n"]
    set text ""
    foreach line $lines {
	if { [string first "in-package" [string tolower $line]] != -1 } {continue}

	if { [string first ":text" $line] > -1 } {
	    regsub -all {\(} $line "OPENPAREN" line
	    regsub -all {\)} $line "CLOSEPAREN" line
	    regsub -all {\{} $line "OPENCURLY" line
	    regsub -all {\}} $line "CLOSECURLY" line
	    regsub -all {;} $line "SEMICOLON" line
	}
	set pos [string first ";" $line]
	if { $pos > -1 } {
	    if { $pos == 0} {continue}
	    set line [string range $line  0 [expr $pos - 1]]
	}

	# if this is a text line, special treatment.
	
	append text "$line "
    }
    regsub -all {  } $text " " text
    regsub -all {:text \"} $text " \{" text
    regsub -all {:comment \"} $text " \{" text
    regsub -all {\"} $text "\} " text

    regsub -all {:features} $text " " text   
    regsub -all {\(def-corpus-item} $text "\n\{" text
    regsub -all {\)} $text "\}" text
    regsub -all {'\(} $text "\{" text
    regsub -all {\(} $text "\{" text
    regsub -all {\)} $text "\}" text
    regsub -all {\[} $text "<" text
    regsub -all {\]} $text ">" text

    # Replace the text-embedded punctuations changed above.
    regsub -all "OPENPAREN" $text "(" text
    regsub -all "CLOSEPAREN" $text ")" text
    regsub -all "OPENCURLY" $text "\{"  text
    regsub -all "CLOSECURLY" $text "\}" text
    regsub -all "SEMICOLON" $text ";" text

    while { [string first "  " $text] > -1 } {
	regsub -all {  } $text " " text
    }

    set ALL_FEATURES [all-active-features]

    foreach item $text {
       set text1 [lindex $item 0]
       set feats [lindex $item 1]
       set cmt [lindex $item 2]
       defcoding $text1 $feats $cmt
   }
   
   find-root
   return
}

proc defcoding {text features {comment {}}} {
    global  NODE ROOT TEXT_NODES ALL_FEATURES
    if { $features == {} } { set features [list $ROOT]}
    set feats {}
    foreach feat $features {
	lappend feats [string tolower $feat]
    }
    set feats [path $feats]
    set id [new-node]
    lappend TEXT_NODES $id
    set NODE($id,features) $feats
    set NODE($id,text) $text
    set NODE($id,textstripped) [strip-coder-text $text]
    set NODE($id,comment) $comment
    return $id
}
