######################################
## Systemic Coder
#
#  File: LoadTagged.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.



proc load-tagged {file} {
    global NODE TEXT_NODES TEXT_FILE ROOT CODINGS_FILE
    set TEXT_FILE $file
    set CODINGS_FILE {}
    clear-systems
    defroot unit
    defsystem "ROOT" unit {}

    set text [read-file $file]
    regsub -all "//" $text "\222" text
    set segs [split $text "\222"]

    foreach seg $segs {
	
	# set features [get-xml-element $seg {segment features}]
	# Eliminate features not in the codings
	set seg  [string trim $seg]
	if { $seg == {}} {continue}
	set id [new-node]
	set line [tagged-get-features $seg]
	set seg [car $line]
	set feature [second $line]
	if { $feature == {}} {
	    set NODE($id,features) [list $ROOT]
	} else {
	    if ![feature-exists-p $feature] {
		add-feature $feature "ROOT"
	    }
	    set NODE($id,features) [list $ROOT $feature]
	}

	set NODE($id,comment) {}
	set NODE($id,ignore) 0
	lappend TEXT_NODES $id
	set NODE($id,text) $seg
	.segment.editor.text insert end $seg
        new-segment end $id
	calculate-node-completeness $id
    }

    if { [llength [fetch-system-features ROOT]] == 0} {
	add-feature Root feature1
	add-feature Root feature2
    }
	
}

proc tagged-get-features {line} {
    global ROOT
    if {[string index $line 0] == "#" } {
	set end [string first " " $line]
	set feature [string range $line 1 [expr $end -1]]
	set text [string range $line $end end]
	return [list $text $feature]
    }
    return [list $line {}]
}
