######################################
## Systemic Coder
#
#  File: menus.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


proc make-menubar {} {
    global PLATFORM 

    # Define the control key to use for each platform
    switch -- $PLATFORM {
	windows { set META_KEY "Control"}
	unix { set META_KEY "Meta"}
	default {set META_KEY "Meta"}
    }

    if [winfo exists .menuBar] {destroy .menuBar}
    MenuBar .menuBar
    
    ##################
    # Codings

    Menu .menuBar Codings
    MenuItem .menuBar Codings "Load Codings" {load-codings}  $META_KEY-l
    MenuItem .menuBar Codings "Save Codings" {save-codings} $META_KEY-s
    MenuItem .menuBar Codings "Save Codings As..." {save-codings-as}
    MenuItem .menuBar Codings "Import Text" {import-text}  $META_KEY-i
    MenuItem .menuBar Codings "Save Text As..." {save-text}
    MenuSeparator .menuBar Codings 
    MenuItem .menuBar Codings "Quit" {Quit}


    ######################################
    # 1. The SCHEME Menu
    add-scheme-menu .menuBar
    

    ######################################
    # 2. The Edit Menu
    
    # Menu .menuBar Edit
    # MenuItem .menuBar Edit "Find Coding" {find-coding}
    
    ######################################
    # 4. The View Menu
    
    # Menu .menuBar View
    # MenuItem .menuBar View "Change Font Size" {change-font-size}
    # MenuItem .menuBar View "Change Text Lines" {change-text-lines}
    # set VIEW_CMTS 1
    # MenuCheck .menuBar View "Show Comments" VIEW_CMTS	{toggle-comment-view}
    #	SubMenu .menuBar View "Text Font Size" .sizemenu
    # foreach size {9 10 11 12 14 16 18 24} 
    # SubMenuAdd .sizemenu "$size" "set-font $size"
 
    # SubMenu .menuBar View "Lines of Text" .linesmenu
    # foreach lines {1 2 3 4 5 6 7 8} 
    #    SubMenuAdd .linesmenu "$lines"  "set-lines $lines"

	
    ######################################
    # 4. The Options Menu

    Menu .menuBar Options
    MenuCheck .menuBar Options  "Advance When Complete" AUTO_ADVANCE  {}  
    MenuCheck .menuBar Options  "Skip Complete Codings" IGNORE_COMPLETE  {}  
    MenuCheck .menuBar Options  "Select on Click" SELECT_ON_CLICK  {}  
    MenuItem .menuBar Options "Scheme Storing..." {save-options}
    MenuItem .menuBar Options "Show Context..." {set-show-context}
    MenuItem .menuBar Options "Coding Column Width..." {set-colwidth}
    MenuItem .menuBar Options "Coding Label Width..." {set-labelwidth}
    MenuItem .menuBar Options "Graph Options..." {graph-options}
    MenuItem .menuBar Options "Default Folders"   {choose-default-folders}
    MenuItem .menuBar Options "Enable Document Sets"   {global SRC;source "$SRC/DocumentSets.tcl"}
    

    ######################################
    # 4. The Help Menu
    Menu .menuBar Help
    set helpkey  $META_KEY-h
    if { $PLATFORM  == "windows"} {   
	MenuItem .menuBar Help "Coder Help" {
	    global DIR
	    exec winhelp $DIR/Manual/Coder.hlp &
	}  $helpkey
	set helpkey  $META_KEY-H
    }
    MenuItem .menuBar Help "Manual (HTML)" {
	global DIR 
	show-webpage $DIR/Manual/index.html 
    }  $helpkey
    MenuItem .menuBar Help "About Coder" {aboutBox}
    MenuItem .menuBar Help "Release Notes" {
	global DIR
	set file [file join $DIR Release-Notes.txt]
	if ![file exists  $file] {
	    tell-user "Cannot find file: $file"
	} else {
	    set text [read-file $file]
	    scrolled-text-dialog .rn "Coder Release Notes" $text
	}
    }
    
    ######################################
    # 5. The Developers Menu
    Menu .menuBar Debug
    MenuItem .menuBar Debug "Toggle Debug" {debug}

    MenuItem .menuBar Debug "Show Procs" {global DIR;show-all-procs $DIR/Source}
    
    MenuItem .menuBar Debug "ReLoad All" {
	rename grapher-delete-feature {}
	rename grapher-rename-feature {}
	rename grapher-add-system-under {}
	rename grapher-change-ec {}
	rename grapher-mark-scheme-modified {}
	rename grapher-mark-scheme-unmodified {}
	rename grapher-change-feature-system {}
	load-module $LIBRARY $LIB_FILES
	load-module $SRC $CDR_FILES
	load-module $SRC $STATS_FILES
	load-module $SCHEME $SCHEME_FILES
	load-module $SCHEME $GFR_FILES
	load-module $SRC $GFR_PATCHES
	make-coder-interface
    }
    
    MenuItem .menuBar Debug "ReLoad Coder" {load-module $SRC $CDR_FILES}
    MenuItem .menuBar Debug "ReLoad Stats" {load-module $SRC $STATS_FILES}
    MenuItem .menuBar Debug "ReLoad Grapher" {
	load-module $SCHEME $SCHEME_FILES
	load-module $SCHEME $GFR_FILES
	load-module $SRC $GFR_PATCHES
    }
    MenuItem .menuBar Debug "ReLoad Library" {load-module $LIBRARY $LIB_FILES}
    MenuItem .menuBar Debug "Test" {test}
    MenuItem .menuBar Debug "size" {puts "Window Geom: [wm geometry .] Maxsize: [wm maxsize .]"}
    MenuItem .menuBar Debug "Search..." {search-files-for-string}
    MenuItem .menuBar Debug "Show Console" {console show}

    bind all <Control-d> {show-debug-menu}

    # On the Mac use the specia .apple menu for the about item
    if {$PLATFORM == "macintosh1"} {
	
	.menuBar add cascade -menu .menuBar.apple
	menu .menuBar.apple -tearoff 0
	.menuBar.apple add command -label "About..." \
		-command "aboutBox"
    }

    hide-debug-menu

    # Add any additional menus as required by other startup program
    global MENU_ADDS
    if [info exists MENU_ADDS] {
	foreach item $MENU_ADDS {
	    eval [concat "MenuItem .menuBar" $item]
	}
    }
}

proc hide-debug-menu {} {
    if {[.menuBar index "Debug"] > 0} {
	.menuBar delete "Debug"
    }
}

proc show-debug-menu {} {
    if ![catch {.menuBar index "Debug"}]  {return}
    .menuBar add cascade -menu .menuBar.debug -label "Debug" -underline 0
}

proc aboutBox {} {
    global TOOL_VERSION
    tk_messageBox -icon info -type ok -title "About Systemic Coder" -message \
	    "Systemic Coder $TOOL_VERSION\n
    Mick O'Donnell\n
    micko@wagsoft.com\n
    http://www.wagsoft.com/Coder/"
}


	
puts "Menus Loaded"




