######################################
## Systemic Coder
#
#  File: Options.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.



proc save-options {} {
    global  GRAMMAR_FILE SCHEME_SRC
    
    set f [modal-dialog .options]
    # wm geometry $f +20+40
    label $f.title -text "File Save Options" 
    
    frame $f.scheme -borderwidth 2  -relief sunken
    label $f.scheme.label -text "Scheme:" 
    frame $f.scheme.options 
    
    radiobutton $f.scheme.options.text -text "Save Scheme with text"\
	    -variable SCHEME_SRC  -value 0 \
	    -command disable-save-scheme-selection
    radiobutton $f.scheme.options.master -text "Save to Master:"\
	    -variable SCHEME_SRC  -value 1 \
	    -command enable-save-scheme-selection

    frame $f.scheme.file
    label $f.scheme.file.label -text "Master File:" 
    entry $f.scheme.file.entry  -bg white -relief sunken \
	    -textvariable GRAMMAR_FILE -width 30 -borderwidth 1
    button $f.scheme.file.button -text "Locate" -command {opt-select-scheme-file}
    help-button $f.scheme.help scheme
    
    pack  $f.scheme.options.master $f.scheme.options.text -side top -anchor w
    

    # set one of the buttons
    if { $GRAMMAR_FILE == {} } {
	set SCHEME_SRC 0
	disable-save-scheme-selection 
    } else {
	set SCHEME_SRC 1
	enable-save-scheme-selection
    }

    pack $f.scheme.file.label $f.scheme.file.entry $f.scheme.file.button\
	    -side left  -padx 5
    
    pack $f.scheme.label $f.scheme.options -side left 
    pack $f.scheme.file -side top  -padx 5
    pack $f.scheme.help -side top  -padx 5
    pack $f.title  $f.scheme -fill x -expand t
    
    frame $f.buttons 
    button $f.buttons.ok -text "Done" -underline 0 \
	    -command "set-save-options-action"
    button $f.buttons.cancel -text Cancel -command "return-from-modal $f 0" \
	    -underline 0
    bind $f <Return> "set-save-options-action"
    bind $f <Control-c> "return-from-modal $f 0"
    
    pack $f.buttons.ok $f.buttons.cancel  -padx 15 -side left
    pack $f.buttons -pady 10
    post-modal $f
}

proc disable-save-scheme-selection {} {
    .options.scheme.file.label config -fg gray
    .options.scheme.file.button config -state disabled
    .options.scheme.file.entry config -state disabled
    .options.scheme.file.entry config -fg gray
}

proc enable-save-scheme-selection {} {
    .options.scheme.file.label config -fg black
    .options.scheme.file.button config -state normal
    .options.scheme.file.entry config -state normal
    .options.scheme.file.entry config -fg black
}

proc set-save-options-action {} {
    global  SCHEME_SRC GRAMMAR_FILE
    debug "set-save-options-action"
    if {$SCHEME_SRC == 0 } {
	set GRAMMAR_FILE {}
    } else {
	if {$GRAMMAR_FILE == {} } {
	    return 0
	}
	mark-scheme-modified
    }
    mark-edited
    return-from-modal .options 1
}


proc opt-select-scheme-file {} {
    global GRAMMAR_FILE DIR
    set default [file join $DIR Schemes]
    if ![file exists $default] { set default $DIR }
    set GRAMMAR_FILE [tk_getSaveFile -title "Select Scheme File"\
	    -initialdir $default -initialfile $GRAMMAR_FILE -parent .options\
	    -filetypes {{{Scheme Files} {.scheme}} {{All Files} {.*}}}]
}

proc set-show-context {} {
    global SHOW_CONTEXT CURRENT_INTERFACE CURRENT
    set newvalue [GetValue "How many segments of context to show (0-9)?" $SHOW_CONTEXT]
    if ![integer-p $newvalue] {
	tell-user "Value must be an integer!"
	set-show-context 
	return
    }
    set SHOW_CONTEXT $newvalue
    if { $CURRENT_INTERFACE == "coder"} {
	set-coder-context $CURRENT
    }
}

proc set-colwidth {} {
    global CODING_COL_WIDTH CURRENT_INTERFACE 
    set newvalue [GetValue "Width of Coding column (in pixels) ?" $CODING_COL_WIDTH]
    if { $newvalue == ""} {return}
    if ![string is integer $newvalue] {
	tell-user "Value must be an integer!"
	set-colwidth 
	return
    }

    if { $newvalue < 80 || $newvalue > 300 } {
	tell-user "Value must be between 80 and 300!"
	set-colwidth 
	return
    }
    set CODING_COL_WIDTH $newvalue
    if { $CURRENT_INTERFACE == "coder"} {
	store-current-coding
	make-system-space
	install-current-coding
    }
}


proc set-labelwidth {} {
    global  CURRENT_INTERFACE MAX_LABEL_WIDTH
    set newvalue [GetValue "Max width of feature labels (in chars) ?" $MAX_LABEL_WIDTH]
    if { $newvalue == ""} {return}
    if ![string is integer $newvalue] {
	tell-user "Value must be an integer!"
	set-colwidth 
	return
    }

    if { $newvalue < 10 || $newvalue > 50 } {
	tell-user "Value must be between 10 and 50!"
	set-labelwidth 
	return
    }
    set MAX_LABEL_WIDTH $newvalue
    if { $CURRENT_INTERFACE == "coder"} {
	store-current-coding
	install-current-coding
    }
}



###################
# DEFAULT DIRS

proc choose-default-folders {} {
    global  SCHEMES ANALYSES TEXTS TMP_SCHEMES TMP_ANALYSES TMP_TEXTS
    
    set TMP_SCHEMES $SCHEMES
    set TMP_ANALYSES $ANALYSES
    set TMP_TEXTS $TEXTS

    set f [modal-dialog .options]
    # wm geometry $f +20+40
    label $f.title -text "Set Default Directories" 
    
    frame $f.schemedir -relief sunken  -borderwidth 2
    label $f.schemedir.label -text "Scheme Files:" 
    entry $f.schemedir.entry -bg white -relief sunken \
	    -textvariable TMP_SCHEMES -width 30 -borderwidth 1
    button $f.schemedir.button -text "Locate" -command {opt-choosedir TMP_SCHEMES}
    pack $f.schemedir.label  -side left -anchor w
    pack $f.schemedir.entry  -side left -anchor w -expand t -fill x
    pack $f.schemedir.button -side left -anchor w

    frame $f.analysesdir -relief sunken  -borderwidth 2
    label $f.analysesdir.label -text "Coder Files:" 
    entry $f.analysesdir.entry -bg white -relief sunken \
	    -textvariable TMP_ANALYSES -width 30 -borderwidth 1
    button $f.analysesdir.button -text "Locate" -command {opt-choosedir TMP_ANALYSES}
    pack $f.analysesdir.label  -side left -anchor w
    pack $f.analysesdir.entry  -side left -anchor w -expand t -fill x
    pack $f.analysesdir.button -side left -anchor w


    frame $f.textsdir -relief sunken -borderwidth 2
    label $f.textsdir.label -text "Text Files:" 
    entry $f.textsdir.entry -bg white -relief sunken \
	    -textvariable TMP_TEXTS -width 30 -borderwidth 1
    button $f.textsdir.button -text "Locate" -command {opt-choosedir TMP_TEXTS}
    pack $f.textsdir.label  -side left -anchor w
    pack $f.textsdir.entry  -side left -anchor w -expand t -fill x
    pack $f.textsdir.button -side left -anchor w

    pack $f.schemedir $f.analysesdir $f.textsdir  -side top -expand t -fill x


    frame $f.buttons 
    button $f.buttons.ok -text "Done" -underline 0 \
	    -command "save-directories-action"
    button $f.buttons.cancel -text Cancel -command "return-from-modal $f 0" \
	    -underline 0
    bind $f <Return> "save-directories-action"
    bind $f <Control-c> "return-from-modal $f 0"
    
    pack $f.buttons.ok $f.buttons.cancel  -padx 15 -side left
    pack $f.buttons -pady 10
    post-modal $f
}

proc opt-choosedir {dirvar} {
    global $dirvar
    set dir [set $dirvar]
    set newdir [tk_chooseDirectory -initialdir $dir]
    if { $newdir != {} } {
	set $dirvar $newdir
    }
}

proc save-directories-action {} {
    global  SCHEMES TEXTS ANALYSES  TMP_SCHEMES TMP_TEXTS TMP_ANALYSES 
    set SCHEMES $TMP_SCHEMES
    set ANALYSES $TMP_ANALYSES
    set TEXTS $TMP_TEXTS
    catch "save-preferences"
    return-from-modal .options 1
}

proc save-preferences {} {
    global PREFS_FILE CODINGS_FILE GRAPH_VERSION XINCR YINCR SCHEMES TEXTS ANALYSES\
	    AUTO_ADVANCE SHOW_CONTEXT REGISTERED IGNORE_COMPLETE SELECT_ON_CLICK CODING_COL_WIDTH MAX_LABEL_WIDTH

    if ![file isdir [file dirname $PREFS_FILE]] {
	tell-user "Preferences file directory no longer exists, exiting unsaved."
	return
    }

    if [file exists $PREFS_FILE] {
	if [catch "file delete $PREFS_FILE"] {
	    tell-user "Preferences file is locked. Cannot save."
	    return
	}
    }

    if [catch "open $PREFS_FILE w 0666" str] {
	return
    }

    puts $str "global CODINGS_FILE XINCR YINCR GRAPH_VERSION SCHEMES TEXTS ANALYSES AUTO_ADVANCE SHOW_CONTEXT REGISTERED IGNORE_COMPLETE SELECT_ON_CLICK CODING_COL_WIDTH MAX_LABEL_WIDTH HIDE_SYSTEM_NAMES"

    if [info exists GRAPH_VERSION] {
	puts $str "set GRAPH_VERSION \"$GRAPH_VERSION\""
	puts $str "set XINCR $XINCR"
	puts $str "set YINCR $YINCR"
    }
    puts $str "set SCHEMES \"$SCHEMES\""
    puts $str "set TEXTS \"$TEXTS\""
    puts $str "set ANALYSES \"$ANALYSES\""
    
    puts $str "set AUTO_ADVANCE \"$AUTO_ADVANCE\""
    puts $str "set IGNORE_COMPLETE \"$IGNORE_COMPLETE\""
    puts $str "set SELECT_ON_CLICK \"$SELECT_ON_CLICK\""
    puts $str "set SHOW_CONTEXT \"$SHOW_CONTEXT\""
    puts $str "set CODING_COL_WIDTH \"$CODING_COL_WIDTH\""
    if [info exists HIDE_SYSTEM_NAMES] {
	puts $str "set HIDE_SYSTEM_NAMES \"$HIDE_SYSTEM_NAMES\""
    }
 puts $str "set MAX_LABEL_WIDTH \"$MAX_LABEL_WIDTH\""


    if [info exists REGISTERED] {
	puts $str "set REGISTERED $REGISTERED"
    }

    close $str
}



proc recolor-buttons {{path .}} {
    global BG_COLOR2
    foreach wid [winfo children $path] {
	if { [winfo class $wid] == "Button"} {
	    $wid config -bg $BG_COLOR2
	} else {
	    recolor-buttons $wid
	}
    }
}

