######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


proc make-review-frame {} {
    global REVIEW_FILTER
    if [winfo exists .review] {destroy .review}
    frame .review

    make-review-tbar
    pack .review.tbar -side left  -anchor nw -pady 1c

    ##################
    ## The CONTENT Frame
    frame .review.main

    scrolled-listbox .review.main.table -bg white -relief sunken -font "Courier 12"
    bind  .review.main.table.choices <Double-Button-1> +{review-edit-current}
    bind  .review.main.table.choices <Button-2> {review-tell-file-of-current %x %y}
    bind  .review.main.table.choices <ButtonRelease> {review-set-current %x %y}
    pack .review.main.table -side top -expand 1 -fill both

    frame .review.main.filter -relief sunken -border 2
    frame .review.main.filter.field
    label .review.main.filter.field.label -text "Filter:"
    text .review.main.filter.field.text -relief sunken -bg white -height 1
    bind .review.main.filter.field.text <Return> {review-apply-filter;break} 
    help-button .review.main.filter.field.help review_filter

    pack .review.main.filter.field.label -side left
    pack .review.main.filter.field.text -side left -expand 1 -fill x
    pack .review.main.filter.field.help 
    pack .review.main.filter.field -side top -expand 1 -fill x


    Toolbar .review.main.filter.controls horizontal
    ToolbarItem .review.main.filter.controls "Apply"\
	    "If you click here, only the codings matching the filter will be displayed"\
	    {review-apply-filter}
    ToolbarItem .review.main.filter.controls "Reset"\
	    "Click here to stop filtering" {review-reset-filter}
    pack .review.main.filter.controls -side bottom -anchor n

    pack  .review.main.filter -pady 0.5 -fill x

    pack .review.main -expand 1 -fill both -pady 0.5c
    
    return .review
}

proc review-tell-file-of-current {x y} {
    global NODE
    set line [.review.main.table.choices get @$x,$y]
    set firstspace [expr [string first " " $line] - 1]
    set itm [string range $line 0 $firstspace]
    if { $itm == {}} {bell;return}
    set id [nth-coding [expr $itm - 1]]
    if [info exists NODE($id,file)] {
	tell-user "Item is from file: $NODE($id,file)"
    }
}

proc set-review-filter {text} {
    .review.main.filter.field.text delete 1.0 end
    .review.main.filter.field.text insert end $text
}

proc make-review-tbar {} {
    # Define a toolbar
    if {[winfo exists .review.tbar]} {destroy .review.tbar}
    
    frame .review.tbar 

    
    Toolbar .review.tbar.export vertical "Save Filtered:"
    ToolbarItem .review.tbar.export "As Text"\
	    "Click here to save the text of the displayed codings" {save-filtered-text}
    ToolbarItem .review.tbar.export "Detailed Text"\
	    "Same as 'As Text' but with comments and features as well" {
	save-filtered-text detailed
    }
    ToolbarItem .review.tbar.export "As Codings"\
	    "Click here to save the displayed codings as a new file" {save-filtered-codings}
    pack .review.tbar.export -side top -pady 0.2c


    Toolbar .review.tbar.action vertical "Actions"
    ToolbarItem .review.tbar.action "Ignore"\
	    "Click here to (un)ignore the selected coding" {review-toggle-ignore}
    ToolbarItem .review.tbar.action "Color Code"\
	    "Each coding will be displayed colors depending on features" {review-select-colorcode %x %y}
ToolbarItem .review.tbar.action "Show ScatterPlot"\
	    "Brings up a window showing selections over the unfolding text" {show-scatterplot}
    pack .review.tbar.action -side top     


}


proc save-filtered-text {{detailed {}}} {

    global TEXT_FILE CURRENT_INTERFACE CODINGS_FILE DIR REVIEW_NODES NODE REVIEW_FILTER\
	ENCODING
    
    if { [info exists REVIEW_NODES] == 0 || $REVIEW_NODES == {} } {
	tell-user "No filtered nodes to save"
	return
    }

    if { $CODINGS_FILE != {} } {
	set file $CODINGS_FILE
    } elseif { $TEXT_FILE != {} } {
	set file $TEXT_FILE
    } else {
	set file [file join $DIR "untitled.txt"]
    }

    set default "[file rootname $file]-subset.txt"
    set defdir [file dirname $default]
    set deffile [file tail $default]
    set file [tk_getSaveFile -initialfile $deffile -initialdir $defdir]
    if { $file == {} } {
	return cancelled
    }

    set ostr [open $file w 0600]
    if [info exists ENCODING] {
	if { $ENCODING != "Standard"} {
	    fconfigure $ostr -encoding $ENCODING
	}
    }
    puts $ostr "TEXT OF NODES MATCHING FILTER: $REVIEW_FILTER"
    catch "puts $ostr \"Date: [date]\n\n\""
    foreach item $REVIEW_NODES {
	
	if { $detailed == "detailed"} {

	    puts $ostr "TEXT: $NODE($item,textstripped)"
	    
	    if { [string trim $NODE($item,comment)] != {} } {
		regsub -all "\n" $NODE($item,comment) " " cmt
		puts $ostr "COMMENT: $cmt"
	    }
	    set feats [most-delicate-features $NODE($item,features)]
	    puts $ostr "FEATURES: $feats\n"
	} else {
	    puts $ostr "$NODE($item,textstripped)\n\n"
	}
    }
    close $ostr
}




proc save-filtered-codings {} {

    global TEXT_FILE CURRENT_INTERFACE CODINGS_FILE DIR REVIEW_NODES NODE REVIEW_FILTER\
	    GRAMMAR_FILE TEXT_NODES ROOT ENCODING

    if { [info exists REVIEW_NODES] == 0 || $REVIEW_NODES == {} } {
	tell-user "No filtered nodes to save"
	return
    }

    if { $CODINGS_FILE != {} } {
	set file $CODINGS_FILE
    } elseif { $TEXT_FILE != {} } {
	set file $TEXT_FILE
    } else {
	set file [file join $DIR "untitled.cd3"]
    }

    set default "[file rootname $file]-subset.cd3"
    set defdir [file dirname $default]
    set deffile [file tail $default]
    set file [tk_getSaveFile -initialfile $deffile -initialdir $defdir\
	    -filetypes [file-save-types]]
    if { $file == {} } {
	return cancelled
    }
    
    
    # Open the Codings file
    save-codings3 $file $REVIEW_NODES
}

proc uninstall-review {} {
    focus .
}

proc install-review {} {
    global TEXT_NODES REVIEW_FILTER REVIEW_NODES CURRENT
    if { $REVIEW_FILTER == {} } {
	set matched $TEXT_NODES
    } else {
	set matched [apply-filter $REVIEW_FILTER $TEXT_NODES]
	if { $matched == "cancelled"} {
	    set REVIEW_FILTER {}
	    return	
	}
    }
    clear-review
    set REVIEW_NODES $matched
    foreach nid $matched {
	review-insert $nid
    }
    if { $matched != {} } {
	.review.main.table.choices selection set 0
	
	if { [info exists CURRENT] == 1 && $CURRENT != {} } {
	    set pos [lsearch -exact $matched $CURRENT]
	    if { $pos > -1 } {
		.review.main.table.choices see $pos
	    }
	}
	update-ignore-button [lindex $matched 0]
    }
    focus .review.main.table.choices
    bind .review.main.table.choices "i" {review-toggle-ignore} 
    review-colorcode
}


proc review-insert {nid} {
    global NODE
    set pos [expr [node-position $nid] + 1]
    set mark " "
    if {  $NODE($nid,ignore) == 1 } {
	set mark "I"
    } elseif {  $NODE($nid,completed-p) == 1} {
	set mark "C"
    }

    if { [info exists NODE($nid,comment)] == 1 && [string trim $NODE($nid,comment)] != "" } {
	set add "*"
    } else {
	set add " "
    }
    set mark "$mark$add"

    if { $pos < 100 } {
	set mark " $mark"
	if { $pos < 10 } { set mark " $mark"}
    }
    
    # Clean up the text
    .review.main.table.choices insert end "$pos $mark$NODE($nid,textstripped)"
}

proc clear-review {} {
    .review.main.table.choices delete 0 end
}

proc review-toggle-ignore {} {
    global NODE
    set string [.review.main.table.choices get active]
    set idx [string range $string 0 [string first " " $string]]
    if { $idx == {}} {bell;return}
    set idx [expr $idx - 1]
    set id [nth-coding $idx]
    if { $NODE($id,ignore) == 1} {
	set NODE($id,ignore) 0
    } else {
	set NODE($id,ignore) 1
    }
    mark-edited
    set yview [lindex [.review.main.table.choices yview] 0]
    install-review 
    .review.main.table.choices yview moveto $yview
    .review.main.table.choices selection clear 0 end
    .review.main.table.choices selection set $idx 
    .review.main.table.choices activate $idx
    update-ignore-button $id
}

proc update-ignore-button {id} {
    global NODE
    if { $NODE($id,ignore) == 1} {
	.review.tbar.action.ignore config -text "Unignore"
    } else {
	.review.tbar.action.ignore config -text "Ignore"
    }
}

proc review-set-current {x y} {
    global NODE
    set line [.review.main.table.choices get @$x,$y]
    set firstspace [expr [string first " " $line] - 1]
    set itm [string range $line 0 $firstspace]
    if { $itm == {}} {bell;return}
    set id [nth-coding [expr $itm - 1]]
    if { $NODE($id,ignore) == 1} {
	.review.tbar.action.ignore config -text "Unignore"
    } else {
	.review.tbar.action.ignore config -text "Ignore"
    }
}

proc review-edit-current {} {
    global CURRENT
    set string [.review.main.table.choices get active]
    set idx [string range $string 0 [string first " " $string]]
    if { $idx == {}} {bell;return}
    set CURRENT [nth-coding [expr $idx - 1]]
    install-interface coder
}

proc review-apply-filter {} {
    global REVIEW_FILTER TEXT_NODES DISPLAYED_FILTER
    set filter [.review.main.filter.field.text get 1.0 end]
    set filter [delete-newlines [string trim $filter]]
    
    if {$filter == {} } {bell;return}
    set REVIEW_FILTER  [string trim $filter]
    install-review

    # store the (possibly revised) filter to be reinstated when the Review is reentered
    set REVIEW_FILTER $DISPLAYED_FILTER

    # Redisplay the (possibly) revised filter
    .review.main.filter.field.text delete  1.0 end
    .review.main.filter.field.text insert end $REVIEW_FILTER
}

proc review-reset-filter {} {
    global REVIEW_FILTER REVIEW_COLOR_SYSTEM
    .review.main.filter.field.text delete 1.0 end
    set REVIEW_FILTER ""
    set REVIEW_COLOR_SYSTEM ""
    install-review
}


defhelp review_filter "FILTERING
=========

  This space allows you to view a sub-corpus, for instance, to see only finite clauses.

  SIMPLE FILTERING: To select only those codings with a particular feature, type in the name of the feature in the \"Filter:\" box. Then either hit the return key, or press the Apply button.

  COMPLEX FILTERING: You can select codings matching a complex combination
of features by typing in expressions like:

     finite and material

     mental or verbal

     not passive

  If you want to use 'and' and 'or' in the same filter, use parentheses to state clearly what you mean:

     finite and (mental or verbal) and not passive
     (finite and mental) or verbal

  WILDCARDS: If you are not sure of the name of the feature, type an asterisk in the name, e.g., \"fin*\" will match the feature \"finite\". If the wildcard matches more than one feature in your scheme, you will be asked to select the one you want.  Wildcards can also be used in complex filters, e.g., 

   fin* or ment*

  An asterisk by itself will show you ALL features in your scheme.
"


proc show-examples-with-feature {feature} {
    global REVIEW_FILTER
    set REVIEW_FILTER $feature
    set-review-filter $feature
    install-review
    install-interface review
}

proc review-select-colorcode {x y} {
    global  REVIEW_COLOR_SYSTEM tcl_version

    if [catch ".review.main.table.choices itemconfig 0 -foreground black"] {
	tell-user "Sorry, Colorcoding in review requires Tcl/Tk 8.3 or higher. See http://www.wagsoft.com/TCL/index.html to download."
	return
    }
    set systems [all-active-systems]
    set color [popup-choose-from-list $systems 30 30]
    if { $color != "cancelled"} {
	set REVIEW_COLOR_SYSTEM $color
	review-colorcode
    }
}

proc review-colorcode {} {
    global REVIEW_COLOR_SYSTEM REVIEW_NODES SYSTEM NODE
    if [winfo exists .review.tbar.colcode] {destroy .review.tbar.colcode}

    set colors {red blue brown green "OliveDrab" pink purple yellow} 
    if ![info exists REVIEW_COLOR_SYSTEM] {return}
    if ![info exists SYSTEM($REVIEW_COLOR_SYSTEM,features)] {set REVIEW_COLOR_SYSTEM ""}

    if { $REVIEW_COLOR_SYSTEM == ""} {return}
    set features $SYSTEM($REVIEW_COLOR_SYSTEM,features)

    # place the color code
    frame .review.tbar.colcode

    set count -1
    label .review.tbar.colcode.code -text "CODE:" 
    pack .review.tbar.colcode.code -side top -anchor nw
    foreach feature $features {
	incr count
	set color [lindex $colors $count]
	if { $color == {} } {set color black}
	label .review.tbar.colcode.$count -text $feature -fg $color
	pack .review.tbar.colcode.$count  -anchor nw
    }
    label .review.tbar.colcode.none -text "not applicable" -fg grey
    pack .review.tbar.colcode -pady 0.2c

    set line -1
    foreach nid $REVIEW_NODES {
	incr line
	set feat [intersection $features $NODE($nid,features)]
	if { $feat == {}} {
	    set color grey
	} else {
	    set order [lsearch -exact $features $feat ]
	    set color [lindex $colors $order]
	}
	if { $color == {} } {set color grey}
	.review.main.table.choices itemconfig $line -foreground $color -selectforeground $color
    }
}


