######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


#####################
### SAVE

# We keep track of changes to Codings and to Scheme.
# - If the scheme is internal, then any change to scheme
#   is registered as a change to codings.
# - If the scheme is external, then any change to scheme
#   is registered as a change to the scheme.

# If we switch from external to internal scheme (see Options.tcl)
# Then set SCHEME_MODIF 0, set CODINGS_MODIF 1
# If we swithc from internal to external, 
# Then set SCHEME_MODIF 1, set CODINGS_MODIF 1

# When we save,

proc prompt-save-current-files {} {
    
    global MODIF_FLAG

    # if the resources are not loaded yet
    if ![info exists MODIF_FLAG] {return}

    if {[prompt-save-current-file] == "cancelled"} {
	return cancelled
    }
}
     

proc prompt-save-current-file {} {
    global GRAMMAR_FILE CURRENT_INTERFACE
    # if changes to the codings or scheme, prompt for save.

    # if in the coding interface, store the current options
    if { $CURRENT_INTERFACE == "coder"} {
	store-current-coding
    }

    if [codings-modified-p] {
	set result [tk_dialog .d1 {File Clear} "Save Current Work?"\
		warning 0 {Yes} {No} "Cancel"]
	switch -- $result {
	    0 {return [save-codings]}
	    2 {return cancelled}
	}
    } elseif { [scheme-modified-p] == 1 && $GRAMMAR_FILE != {} } {
	set result [tk_dialog .d1 "Save Scheme" "Scheme has changed? Save changes?"\
		warning 0 {Yes} {No} "Cancel"]
	switch -- $result {
	    0 {
		# First check where the scheme should be saved
		if {[verify-scheme-save-file] == "cancelled"} {return "cancelled"}
		
		# User might have switched to internal storage
		if { $GRAMMAR_FILE == {} } {
		    prompt-save-current-file
		} else {
		    save-scheme $GRAMMAR_FILE
		}
	    }
	    1 {return "quit"}
	    2 {return "cancelled"}
	}
    }
}

proc save-and-clear {} {
  global CURR_INDEX
  # Are there modified codings?
  if { $CURR_INDEX > 0  } {
      if { [prompt-save-current-files] == "cancelled" } {return cancelled} 
      clear-codings
  }
  return 1
}


proc prompt-save-current-scheme {} {
    global GRAMMAR_FILE
    # if changes to the codings or scheme, prompt for save.

    if { [scheme-modified-p] == 1 && $GRAMMAR_FILE != {} } {
	set result [tk_dialog .d1 "Save Grammar" "Grammar has changed. Save changes?"\
		warning 0 {Yes} {No} "Cancel"]
	switch -- $result {
	    0 {
		return [save-scheme-as]
	    }
	    1 {return "quit"}
	    2 {return "cancelled"}
	}
    }
}

proc scheme-default-file {{ext "scheme"}} {
    global GRAMMAR_FILE GRAMMARS DIR
    if { $GRAMMAR_FILE != {} } {
	set default $GRAMMAR_FILE
    } elseif [file isdirectory $GRAMMARS] {
	set default [file join $GRAMMARS "untitled"]
    } else {
	set default [file join $DIR "untitled"]
    }
    return "[file rootname $default].$ext"
}



proc get-scheme-save-name {} {
    global GRAMMAR_FILE DIR GRAMMARS
    if [valid-filevar-p GRAMMAR_FILE] {
	set default [file tail $GRAMMAR_FILE]
	set initialdir [file dirname $GRAMMAR_FILE]
    } else {
	set initialdir [default-directory grammar]
	set default  "Untitled.scheme"
    }
    return  [tk_getSaveFile -initialfile $default -initialdir $initialdir]
}




# The primary format is xml
# to save as another form requires selection from the save-as
# menu.
# maybe prompt if load-form different from xml.

# schemes can be in a master (so they should be updated)
# or in the document.
# if CODINGS_FILE is {}, include

proc save-codings {{file {}}} {
    global CODINGS_FILE CURRENT_INTERFACE FILE_SAVE_FORMS

    if {$file != {} } {
	set CODINGS_FILE $file
    }
    
    if { $CODINGS_FILE == {} } {
	return [save-codings-as]
    }

    # Ensure any changes in segmenter are recorded
    if { $CURRENT_INTERFACE == "segment"} {
	update-structure
    }

    set extension [file extension $CODINGS_FILE]
    foreach type $FILE_SAVE_FORMS {
	if  {[second $type] == $extension} {
	    start-watch-cursor
	    if [catch "call-file-loader [third $type] \"$CODINGS_FILE\"" result] {
		stop-watch-cursor
		BugReport "Error while saving $CODINGS_FILE. Please send context to micko@wagsoft.com including the following:"
		return 0
	    }
	    stop-watch-cursor
	    if { $result == 0} {
		return 0
	    }
	    if {[lindex $type 3] == 1} {
		mark-unedited
		wm title . [file tail $CODINGS_FILE]
	    }
	    return 1
	}
    }
    tell-user "Save-codings: Unknown Format: $extension"
    return cancel
}


proc add-saveas-option {title ext proc {mark_saved 1}} {
    global FILE_SAVE_FORMS 
    # If this type found, override
    if [info exists FILE_SAVE_FORMS] {
	indexed-delete FILE_SAVE_FORMS $title 
    }

    # Add the entry
    lappend FILE_SAVE_FORMS [list $title $ext $proc $mark_saved]
}

proc file-save-types {} {
    global FILE_SAVE_FORMS
    set types {}
    foreach type $FILE_SAVE_FORMS {
	lappend types [list [car $type] [second $type]]
    }
    return $types
}

proc save-codings-as {} {
    global CODINGS_FILE TEXT_FILE DIR FILE_SAVE_FORMS

    set default_ext [second [car $FILE_SAVE_FORMS]]
    if {$CODINGS_FILE != {} } {
	set default $CODINGS_FILE
    } elseif {$TEXT_FILE != {} } {
	set default [file rootname $TEXT_FILE]$default_ext
    } elseif [file exists  [file join $DIR Analyses]] {
	set default [file join $DIR "Analyses" "untitled$default_ext"]
    } else { 
	set default  [file join $DIR "untitled$default_ext"]
    }
    
    set defdir [file dirname $default]
    set deffile [file root [file tail $default]]$default_ext
    
    # Compile out the saveas forms
    set types {}
    foreach type $FILE_SAVE_FORMS {
	lappend types [list [car $type] [second $type]]
    }
    set file [tk_getSaveFile -filetypes $types -initialfile $deffile -initialdir $defdir]
    if { $file == {} } {
	return cancelled
    }
    save-codings $file
}



proc save-codings2 {file} {
    global CODINGS_FILE  GRAMMAR_FILE  NODE TEXT_NODES ROOT SCHEME_MODIFIED ENCODING 

    # First check where the scheme should be saved
    if { [verify-scheme-save-file] == "cancelled"} {return "cancelled"}

    # NEW -- save file to tmp file, then move to correct place if save completes
    set tempfile [file root $file].tmp
    if [file exists $tempfile] {file delete $tempfile}
 
    # Open the Codings file
    set f [open $tempfile w 0600]
    
  
    puts $f "<codings>"
    puts $f "  <header>"
    

    # Register an internationalisation encoding (if any)
    if [info exists ENCODING] {
	if { $ENCODING != "Standard"} {
	    puts $f "    <encoding name=\"$ENCODING\" />"
	    tell-user "Saving in $ENCODING format"
	    fconfigure $f -encoding $ENCODING
	}
    }
    # save the scheme 
    if { $GRAMMAR_FILE == {} } {
	# Save embedded in the file
	embed-scheme-in-file $f
    } else {

	# Save as external file if needed
	if { $SCHEME_MODIFIED == 1 } {
	     save-scheme $GRAMMAR_FILE 
	}
	# Register the master scheme in the codings file
	puts $f "    <scheme file=\"$GRAMMAR_FILE\"></scheme>"
    }
    mark-scheme-unmodified

    puts $f "  </header>"

    # Save the text
    puts $f "  <body>"

    foreach itm $TEXT_NODES {
	set text $NODE($itm,text)
	regsub -all "<" $text {\&lt;} text
	regsub -all ">" $text {\&gt;} text
	set comment [string trim $NODE($itm,comment)]
	if { $comment == ""} {
	    set commentstr ""
	} else {
	    regsub -all "<" $comment {\&lt;} comment
	    regsub -all ">" $comment {\&gt;} comment
	    regsub -all "\"" $comment "'" comment
	    set commentstr " comment=\"$comment\""
	}
	puts $f "    <segment features=\"$NODE($itm,features)\" $commentstr ignore=$NODE($itm,ignore)>$text</segment>"
    }
    
    # Close the file
    puts $f "  </body>"
    puts $f "</codings>"
    close $f
    

    # Move the tempfile to the correct place
    if [file exists $file] {file delete $file}
    file rename $tempfile $file
    mark-unedited
}


proc save-codings-txt {file} {
    # Tab Delimited
    global CODINGS_FILE NODE TEXT_NODES ROOT SCHEME_MODIFIED SYSTEM

    # Decide the features to save
    # (for binary systems, record only the first, else all)
    set saved {}
    foreach system [all-active-systems] {
	if { [llength $SYSTEM($system,features)] > 2 } {
	    set saved [concat $saved $SYSTEM($system,features)]
	} else {
	    lappend saved [car $SYSTEM($system,features)]
	}
    }
    # Open the Codings file
    set f [open $file w 0600]
    
    # Put the header opener
    puts -nonewline $f "Text\tComment"
    foreach feature $saved {
	puts -nonewline $f "\t$feature"
    }

    # Save each line
    foreach itm $TEXT_NODES {
	set text $NODE($itm,text)
	regsub -all "\t" $text " " text
	regsub -all "\n" $text " " text
	set comment [string trim $NODE($itm,comment)]
	regsub -all "\t" $comment " " comment
	regsub -all "\n" $comment " " comment
	regsub -all "\"" $comment "'" comment
	puts -nonewline $f "\n$text\t$comment"
	foreach feature $saved {
		puts -nonewline $f "\t[member $feature $NODE($itm,features)]"
	}
    }
    # Close the file
    close $f
}

proc verify-scheme-save-file {} {
    global SCHEME_MODIFIED GRAMMAR_FILE

    # If the scheme is modified and is stored externally,
    # Check if the user wants to overwrite it.
    if {$SCHEME_MODIFIED == 1 && $GRAMMAR_FILE != {} } {
   
	set msg "The Master scheme associated with this coding file has been changed. What should I do?"
	
	switch -- [tk_dialog .tmpwin "Master Scheme" $msg {} 0\
		"Update" "Save As..." "Save in Codings file" "Cancel Save"] {
	    1 {
		set file [get-scheme-save-name]
		if { $file == {} } {
		    return cancelled
		}
		set GRAMMAR_FILE $file
	    }
	    2 {
		set GRAMMAR_FILE {} 
		mark-edited
	    }
	    3 { return "cancelled" }
	}
    }
}


proc embed-scheme-in-file {strm} {
    global ROOT SCHEME_MODIFIED SAVED_SYSTEMS
    set SAVED_SYSTEMS {}
    puts $strm "    <scheme root=$ROOT>"
    save-xml-feature $ROOT $strm
    puts $strm "    </scheme>"
}


####################################
## SCHEME FILE

proc save-text {{file {}}} {
    global TEXT_FILE CURRENT_INTERFACE CODINGS_FILE
    if ![info exists TEXTFILE1] {set TEXTFILE1 {}}
    
    if {$file == {} } {
	if {$TEXT_FILE != {} } {
	    set default $TEXT_FILE
	} elseif {$CODINGS_FILE != {} } {
	    set default [file rootname $CODINGS_FILE].txt
	} else {
	    set default [file join $DIR "Analyses" "untitled.txt"]
	}
	set defdir [file dirname $default]
	set deffile [file tail $default]
	set file [tk_getSaveFile -initialfile $deffile -initialdir $defdir]
	if { $file == {} } {
	    return cancelled
	}
    }

    set TEXT_FILE $file
    set text [.segment.editor.text get 1.0 end]
    write-file $file $text
}


