######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.



proc save-codings3 {file {nids {}}} {
    global CODINGS_FILE  GRAMMAR_FILE  NODE TEXT_NODES ROOT SCHEME_MODIFIED ENCODING\
	    CURRENT_INTERFACE DOCUMENT_FEATURES DOCUMENT_SET PLATFORM

    # Ensure we have the latest stored
    switch -- $CURRENT_INTERFACE {
	"coder" {
	    store-current-coding
	}
    }

    # If no nodes specified, use all
    if { $nids == {} } {
	set nids $TEXT_NODES
    }

    # First check where the scheme should be saved
    if { [verify-scheme-save-file] == "cancelled"} {return "cancelled"}
 
    # Open the Codings file
    set f [open $file w 0600]
    
    # Put the header opener
      # Put the header opener
    if { ![info exists ENCODING] || $ENCODING == "Standard"} {
	switch -- $PLATFORM {
	    "windows" {
		set encoding "windows-1252"
	    }
	    default {
		set encoding ascii
	    }
	}
    } else {
	switch -- $PLATFORM {
	    "windows" {
		regsub "cp" $ENCODING "windows-" encoding
		regsub "CP" $encoding "windows-" encoding
	    }
	    default {
		set encoding $ENCODING
	    }
	}
    }
    puts $f "<?xml version=\"1.0\" encoding='$encoding'?>"
    
    
    puts $f "<codings>"
    puts $f "  <header>"
    

    # Register an internationalisation encoding (if any)
    if [info exists ENCODING] {
	if { $ENCODING != "Standard"} {
	    puts $f "    <encoding name=\"$ENCODING\" />"
	    tell-user "Saving in $ENCODING format"
	    fconfigure $f -encoding $ENCODING
	}
    }

    # Record any document-level features
    foreach feature $DOCUMENT_FEATURES {
	puts $f "    <docfeature  value=\"$feature\" />"
    }

    # save the scheme 
    if { $GRAMMAR_FILE == {} } {
	# Save embedded in the file
	embed-scheme-in-file3 $f
    } else {

	# Save as external file if needed
	if { $SCHEME_MODIFIED == 1 } {
	     save-scheme $GRAMMAR_FILE 
	}
	# Register the master scheme in the codings file
	puts $f "    <scheme file=\"$GRAMMAR_FILE\"></scheme>"
    }
    mark-scheme-unmodified

    puts $f "  </header>"

    # Save the text
    puts $f "  <body>"

    foreach itm $nids {
	set text [xml-encode $NODE($itm,text)]
	set comment [string trim $NODE($itm,comment)]
	regsub -all "\n" $comment " " comment
	if { $comment == ""} {
	    set commentstr ""
	} else {
	    set comment [xml-encode $comment]
	    set commentstr " comment=\"$comment\""
	}
	puts $f "    <segment features=\"$NODE($itm,features)\" $commentstr ignore=\"$NODE($itm,ignore)\">$text</segment>"
    }
    
    # Close the file
    puts $f "  </body>"
    puts $f "</codings>"
    close $f
    mark-unedited
}

proc embed-scheme-in-file3 {strm} {
    global ROOT SCHEME_MODIFIED SAVED_SYSTEMS
    set SAVED_SYSTEMS {}
    puts $strm "    <scheme root=\"$ROOT\">"
    save-xml-feature $ROOT $strm cd3
    puts $strm "    </scheme>"
}

proc save-xml-feature {feature str {filetype cd3}} {
    global FEATURE SYSTEM SAVED_SYSTEMS
    if ![info exists SAVED_SYSTEMS] {set SAVED_SYSTEMS {}}
    if [info exists FEATURE($feature,realisations)] {
	set reals [string trim $FEATURE($feature,realisations)]
	set reals [xml-encode $reals]
	regsub -all "\n" $reals {;} reals
    } else {
	set reals ""
    }

    if [info exists FEATURE($feature,state)] {
	set state $FEATURE($feature,state)
    } else {
	set state "active"
    }
    
    if { $reals != ""  || $state == "disabled"} {
	puts $str "      <feature name=\"$feature\" realisations=\"$reals\" state=\"$state\" />"
    }
    foreach system $FEATURE($feature,extenders) {
	if ![member $system $SAVED_SYSTEMS] {
	    lappend SAVED_SYSTEMS $system
	    puts -nonewline $str "      <system name=\"$system\" ec=\"$SYSTEM($system,ec)\""
	    puts -nonewline $str " features=\"$SYSTEM($system,features)\" state=\"$SYSTEM($system,state)\""
	    if {$SYSTEM($system,gloss) != {} } {
		puts -nonewline  $str " gloss=\"$SYSTEM($system,gloss)\""
	    }

	    if {$filetype == "cd3"} {
		puts  $str "/>"
	    } else {
		puts $str ">"
	    }

	    foreach feat $SYSTEM($system,features) {
		save-xml-feature $feat $str
	    }
	}
    }    
}


