
proc gr-change-ec {system} {
    global SYSTEM FEATURE EC_OP EC_TYPE EC_FEATS ROOT BG_COLOR2

    set maxsize 4
    set options {1 2 3 4 5 6 7 8 9}
    set EC_FEATS [set-difference [all-features-sorted] [all-features-under-system $system]]

    #    establish the current ec
    set currec $SYSTEM($system,ec)

    # Store elements of the current ec into elements of the starting template
    # (even if they are not to be used)
    
    if [member [car $currec] {and or}] {
	set EC_OP [pop currec]
    } else {
	set EC_OP "and"
    }

    # Define the input EC length
    set EC_TYPE [min [llength $currec] $maxsize]

    # Set the values for each field of the EC (at all possible lengths)
    for {set x 1} {$x<=$maxsize} {incr x} {
	set var "EC_FEAT$x"
	
	#If the value is nil, default to the root feature
	set val [pop currec]
	if { $val == {} } {set val $ROOT}
	global $var
	set $var $val
    }

 
   # Build the Dialog
    set f [modal-dialog .changeec]
    wm protocol .changeec WM_DELETE_WINDOW "return-from-modal .changeec 0"

    # Show the Title
    label .changeec.title -text "Entry condition for system: $system" 

    # 1. Choose complex or simple
    frame .changeec.type -relief sunken -borderwidth 1
    label .changeec.type.label  -text "Number of Terms:" 
    eval [concat "tk_optionMenu .changeec.type.selector EC_TYPE" [lrange $options 0 [expr $maxsize - 1]]]
    for {set x 1} {$x<=$maxsize} {incr x} {
	.changeec.type.selector.menu entryconfigure [expr $x - 1] -command "make-ec-frame $x"
    }
    help-button .changeec.type.help ec_type
    pack .changeec.type.label .changeec.type.selector .changeec.type.help -side left -anchor w -padx 0.3c


    # Pack elements in the window
    pack .changeec.title .changeec.type   -anchor w

    # Display the default EC 
    make-ec-frame $EC_TYPE


    # The button row
    frame .changeec.buttons 
    button .changeec.buttons.ok -text "OK" -command "ec-ok $system" -bg $BG_COLOR2
    button .changeec.buttons.cancel -text "Cancel" -command {return-from-modal .changeec 0}\
	    -bg $BG_COLOR2
    bind .changeec <Return> "ec-ok $system"
    bind .changeec <Control-c> "return-from-modal $f 0"
    pack .changeec.buttons.ok .changeec.buttons.cancel  -padx 15 -side left
    pack .changeec.buttons -pady 10
  
    # Display the modal dialog (the Ok button ends this proc)
    post-modal $f
}


proc make-ec-frame {count} {
    global EC_TYPE EC_OP EC_FEAT1 BG_COLOR2
    set EC_TYPE $count

    # 2. Make the frame for displaying entry conditions
    if {[winfo exists .changeec.ec]} {destroy .changeec.ec}
    frame .changeec.ec    -relief sunken -borderwidth 1
    label .changeec.ec.label  -text "Entry Condition:"  

    # Put in the first feature in the EC
    button .changeec.ec.feat1 -textvariable EC_FEAT1 -bg $BG_COLOR2
    bind .changeec.ec.feat1 <ButtonRelease-1> "ec-choose-feat EC_FEAT1 %x %y"

    pack .changeec.ec.label .changeec.ec.feat1 -side left  -anchor w

    # Add remaining elements
    for {set x 2} {$x<=$count} {incr x} {
	global EC_FEAT$x
	tk_optionMenu .changeec.ec.op$x EC_OP "and" "or"
	.changeec.ec.op$x.menu config 
	button .changeec.ec.feat$x -textvariable EC_FEAT$x -bg $BG_COLOR2
	bind .changeec.ec.feat$x <ButtonRelease-1> "ec-choose-feat EC_FEAT$x  %x %y"
	pack  .changeec.ec.op$x .changeec.ec.feat$x -side left	
    }

    help-button .changeec.ec.help change_ec_complex
    pack .changeec.ec.help -side left -padx 0.3c
    pack .changeec.ec -after .changeec.type
}


proc ec-choose-feat {var x y} {
    global EC_FEATS 
    upvar $var v1
    set result [popup-choose-from-list $EC_FEATS $x $y]
    if {$result != "cancelled"} {
	set v1 $result
    }
}

proc ec-ok {system} {
    global  EC_FEAT1 EC_OP EC_TYPE

    if { $EC_TYPE == 1} {
	set newec $EC_FEAT1
    } else {
	set op [string tolower $EC_OP]
	set newec $op
	for {set x 1} {$x<=$EC_TYPE} {incr x} {
	    global EC_FEAT$x
	    set var "EC_FEAT$x"
	    set newec "$newec [expr $$var]"
	}
    }
    return-from-modal .changeec 1
    update
    change-ec $system $newec
    mark-scheme-modified
    graph-scheme
}




defhelp ec_type "ENTRY CONDITION TYPE

 Select here the number of features you want in your entry condition. Choose '1' for a simple entry condition (e.g., clause), or 2, 3 or 4 if you want the entry condition to be a conjunction or disjunction of features (e.g., finite AND mental). 

  If you need more than 4 terms in an entry condition, first make a gate (a system with a single feature) which has 4 or so features as its entry condition, and then use the feature of the gate as part of the entry condition for the current system."


defhelp change_ec_complex "Use this part to specify the entry condition for the system. 

  - If the entry condition has a single feature, just select it. 

  - If multiple, select the joiner to be either AND or OR (at present, these cannot be mixed, you can only have 'A and B and C', not 'A and B or C'). 

  If you want to mix AND and OR in the same entry condition, you can simulate such by first making a 'gate' (a system with only one feature). For instance, to construct the entry condition 'A and (B or C)', first make a system with one feature (call the feature b-or-c). Then use this feature and the A feature in an AND entry condition for your original system. The entry condition for this system will now be 'A and (B or C)'."



