# Loads in a grammar and compares it.



proc compare-scheme {{file {}}} {
    global GRAMMARS GRAMMAR_FILE INSERTS COMPARE_FILE SYSTEM1 \
	    FEATURE1 SYSTEMS1 FEATURES1 COMPTEXT

    if {$file == {} } {
	if { [info exists GRAMMAR_FILE] == 1 && $GRAMMAR_FILE != ""} {
	    set initialdir  [file dirname $GRAMMAR_FILE]
	} else {
	    set initialdir $GRAMMARS
	}
	    
	set file [tk_getOpenFile -initialdir $initialdir \
		-title "Compare current to which scheme?"]

	if { $file == ""} {return}
    }

    # initialise
    if [info exists SYSTEM1] {unset SYSTEM1}
    if [info exists FEATURE1] {unset FEATURE1}
    set FEATURES1 {}
    set SYSTEMS1 {}

    set text [read-file $file]
    regsub -all "defsystem" $text "defsystem-compare" text
    regsub -all "def-feature" $text "deffeature-compare" text
    eval $text
    set COMPARE_FILE $file

    compare-systems
    display-compare-systems
    
}


proc compsay  {text} {
    global COMPTEXT
    set COMPTEXT "$COMPTEXT\n$text"
}

# Load the grammar into an alternative data strcuture
# also keep track of the system and feature names.
# at end, run through systems and report differences.
# Also run through realisations of each system


# 1. System has same name: 
#  report any differences in EC
#  report any differences in features
#    new feature
#    moved feature
#  for feature in new system, report changes to reals

# 2. System has new name:
#  if features occur elsewhere, report change of name
#  then delete from missing-system list
#        report any change ec
#        report for feature in new system, report changes to reals
#  else call new system and describe
# 
# 3. Report systems in old grammar not in new
# 4. Report features in old grammar not in new


proc display-compare-systems {} {
     global COMPTEXT COMPARE_FILE GRAMMAR_FILE COMP_RESULT COMP_SYSTEM_CHANGES SYSTEM1 SYSTEM
    
    if [winfo exists .compare] {destroy .compare}
    set COMPTEXT  "\n DIFFERENCES IN GRAMMARS\n============================\n"
    compsay "CURRENT GRAMMAR: $GRAMMAR_FILE"
    compsay "\nNEW GRAMMAR: $COMPARE_FILE"

    # Find those systems with the same name
    if { [car $COMP_SYSTEM_CHANGES] != {} } {
	compsay "\nDELETED SYSTEMS:"
	foreach system [car $COMP_SYSTEM_CHANGES] {
	    regsub -all " " $SYSTEM($system,features) "/" features
	    compsay "  $system : \{$SYSTEM($system,ec)\} -> $features"	    
	}
    } else {
	compsay "\nThere were no systems deleted."
    }

    if { [second $COMP_SYSTEM_CHANGES] != {} } {
	compsay "\nTOTALLY NEW SYSTEMS:"
	foreach system [second $COMP_SYSTEM_CHANGES] {
	    regsub -all " " $SYSTEM1($system,features) "/" features
	    compsay "  $system : \{$SYSTEM1($system,ec)\} -> $features"	    
	}
    } else {
	compsay "\nThere were no systems added."
    }


    # Run through the commonSystems and report changes
    foreach sysresult $COMP_RESULT {
	set system [car $sysresult]
	set changes [second $sysresult]
	compsay "\n\nSystem: '$system' has changed:"
	foreach change $changes {
	    switch -- [car $change] {
		changedreals {
		    compsay "\n  * Feature '[second $change]' has changed realisations:"
		    if { [third $change] != {} } {
			set deleted [third $change]
			compsay "      Old: [car $deleted]"
			foreach real [cdr $deleted] {
			    compsay "           $real"
			}
		    }
		    if { [fourth $change] != {} } {
			set new [fourth $change]
			compsay "      New: [car $new]"
			foreach real [cdr $new] {
			    compsay "           $real"
			}
		    }
		}

		ecchange {
		    compsay "\n  * The entry condition has changed:"
		    compsay "    Old: [second $change]"
		    compsay "    New: [third $change]"
		}

		movedinfeature {
		    compsay "\n  * Feature '[second $change]' was moved here from system [third $change]."
		}
		newfeature {
		    compsay "\n  * Feature '[second $change]' was added."
		}
		movedoutfeature{
		    compsay "\n  * Feature '[second $change]' was moved to system [third $change]."
		}
		deletedfeature {
		    compsay "\n  * Feature '[second $change]' was deleted."
		}

		default {
		    compsay "  CHANGE: $change"
		}
	    }
	}
    }


    compare-scrolled-text-dialog .compare "Scheme Comparison" $COMPTEXT
}

proc compare-systems {} {
     global SYSTEM1 SYSTEMS1 FEATURE1 FEATURES1 ROOT SYSTEM FEATURE COMP_RESULT COMP_SYSTEM_CHANGES
    
    # Find those systems with the same name
    set allSystems [all-systems-under $ROOT]
    set commonSystems [intersection $allSystems $SYSTEMS1]
    set newSystems [set-difference $SYSTEMS1 $commonSystems]
    set deletedSystems [set-difference $allSystems $commonSystems]

    # Find the systems which have been renamed
    foreach system1 $newSystems {
	foreach system2 $deletedSystems {
	    if { $SYSTEM1($system1,features) == $SYSTEM($system2,features) } {
		set changes [compare-system $system1 $system2]
		push [list namechange $system1 $system2] changes
		lappend COMP_RESULT [list $system1 $changes]
		delete newSystems $system1
		delete deletedSystems $system2
	    }
	}
    }

    set COMP_SYSTEM_CHANGES [list $deletedSystems $newSystems]


    # Run through the commonSystems and record changes
    set COMP_RESULT {}
    foreach system $commonSystems {
	set sysresult [compare-system $system]
	if { $sysresult != {} } {
	    lappend COMP_RESULT [list $system $sysresult]
	}
    }
}


proc compare-system {system {system2 {}} } {
    global SYSTEM1 SYSTEMS1 FEATURE1 FEATURES1 ROOT SYSTEM FEATURE

    if { $system2 == {} } { set system2 $system}
    set sysresult {}
    
    # Record any change of EC
    if { $SYSTEM($system2,ec) != $SYSTEM1($system,ec)} {
	lappend sysresult [list ecchange $SYSTEM($system2,ec)  $SYSTEM1($system,ec)]
    }
    
    # Record any change of features
    set oldFeatures $SYSTEM($system2,features)
    set newFeatures $SYSTEM1($system,features)
    if { $oldFeatures != $newFeatures } {
	set newFeatures1 [set-difference $newFeatures $oldFeatures]
	foreach feature $newFeatures1 {
	    if [info exists FEATURE($feature,system)] {
		lappend sysresult [list movedinfeature $feature $FEATURE($feature,system)]
	    } else {
		lappend sysresult [list newfeature $feature]
	    }
	}
	
	set deletedFeatures [set-difference $oldFeatures $newFeatures]
	foreach feature $deletedFeatures {
	    if [info exists FEATURE1($feature,system)] {
		lappend sysresult [list movedoutfeature $feature $FEATURE1($feature,system)]
	    } else {
		lappend sysresult [list deletedfeature $feature]
	    }
	}
    }	
    
    # for each feature in the new system, report changed realisations
    foreach feature $newFeatures {
	set newReals {}
	set oldReals {}
	if  [info exists FEATURE1($feature,realisations)] {
	    set newReals [convert-reals $FEATURE1($feature,realisations)]
	}
	if  [info exists FEATURE($feature,realisations)] {
	    set oldReals [convert-reals $FEATURE($feature,realisations)]
	}
	
	set commonReals [intersection $oldReals $newReals]
	set newReals1 {}
	foreach real [set-difference $newReals $commonReals] {
	    lappend newReals1 [screen-realisation $real]
	}

	set deletedReals {}
	foreach real [set-difference $oldReals $commonReals] {
	    lappend deletedReals [screen-realisation $real]
	}

	if { $newReals1 != {} || $deletedReals != {} } {
	    lappend sysresult [list changedreals $feature $deletedReals $newReals1]
	}
    }
    return $sysresult
}




proc defsystem-compare {system ec features} {
    global SYSTEM1 SYSTEMS1 FEATURES1 FEATURE1
    set SYSTEM1($system,ec) $ec
    set SYSTEM1($system,features) $features
    lappend SYSTEMS1 $system
    concat! FEATURES1 $features
    foreach feature $features {
	set FEATURE1($feature,system) $system
    }
}

proc deffeature-compare {feature realisations} {
    global FEATURE1
    set FEATURE1($feature,realisations) $realisations
}


proc compare-scrolled-text-dialog {name title text args} {
    global std-exit
# Returns a scrollable text widget
# $name.text is the actual text widger
# $name.title is the title
#
# Args: (all optional)
# -height <integer>
# -font <font-spec>
# -titlevar varname : variable containing the name of the frame title

    toplevel $name 
    wm title $name $title 
    frame $name.textwindow
    text  $name.text  -bg white -relief sunken -wrap word\
	    -yscrollcommand "$name.scroll set"
	
    scrollbar $name.scroll -command "$name.text yview"

    set font [getarg -font $args]
    if { $font != {} } {
	$name.text configure -font $font
    }

    set height [getarg -height $args]
    if { $height != {} } {
	$name.text configure -height $height
    }

    $name.text insert end $text
    pack $name.textwindow -side top -expand 1 -fill both
    pack $name.text -in $name.textwindow -fill both -expand 1 -side left
    pack $name.scroll -in $name.textwindow -side right -fill y 

    frame $name.buttons
    button $name.buttons.save -text "Save as HTML" -command "compare-save-html"
    button $name.buttons.close -text "Close" -command "destroy $name"
    pack $name.buttons.save $name.buttons.close -side left -anchor n
    pack $name.buttons -side bottom

    # 4. Set up a binding for <Return>
    # set a grab, and claim the focus too.
    bind $name <Return> "destroy $name"
}


proc compare-save-html {} {
    global GRAMMAR_FILE COMPARE_FILE  COMP_RESULT COMP_SYSTEM_CHANGES SYSTEM1 SYSTEM DIR
    set file [tk_getSaveFile -title "Save Comparison as"\
	    -initialfile [file join $DIR "SchemeChanges.html"] -filetypes { {HTML *.html}}]

    if { $file == {} } { return }
    
    if [file exists $file] {file delete -force $file}
    set str [open $file w]

    puts $str "<html><title>Scheme Comparison</title><body BGCOLOR=\"#FFCC66\">"
    
    puts $str "<h1>Scheme Comparison</h1>"
    puts $str "<p><b>This File</b>: $GRAMMAR_FILE"
    puts $str "<p><b>New File</b>: $COMPARE_FILE"
    catch "puts $str \"<p><b>Date</b>: [date]</p>\""


    # Find those systems with the same name
    puts $str "<hr><h2>DELETED SYSTEMS</h2>"
    if { [car $COMP_SYSTEM_CHANGES] != {} } {
	puts $str "<ul>"
	foreach system [car $COMP_SYSTEM_CHANGES] {
	    regsub -all " " $SYSTEM($system,features) "/" features
	    puts $str "<li>$system : \{$SYSTEM($system,ec)\} -> $features"	    
	}
	puts $str "</ul>"
    } else {
	puts $str "<p>There were no systems deleted."
    }

    puts $str "<hr><h2>TOTALLY NEW SYSTEMS</h2>"
    if { [second $COMP_SYSTEM_CHANGES] != {} } {
	puts $str "<ul>"
	foreach system [second $COMP_SYSTEM_CHANGES] {
	    regsub -all " " $SYSTEM1($system,features) "/" features
	    puts $str "<li>$system : \{$SYSTEM1($system,ec)\} -> $features"	    
	}
	puts $str "</ul>"
    } else {
	puts $str "<p>There were no systems added."
    }

    puts $str "<hr><h2>CHANGED SYSTEMS</h2>"
    # Run through the commonSystems and report changes
    foreach sysresult $COMP_RESULT {
	set system [car $sysresult]
	set changes [second $sysresult]
	puts $str "<h3>System: '$system'</h3><ul>"
	foreach change $changes {
	    switch -- [car $change] {
		changedreals {
		    puts $str "<li>Feature '[second $change]' has changed realisations:"
		    if { [third $change] != {} } {
			set deleted [third $change]
			puts $str "<br>&nbsp;Deleted: <i>[car $deleted]</i>"
			foreach real [cdr $deleted] {
			    puts $str "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>$real</i>"
			}
		    }
		    if { [fourth $change] != {} } {
			set new [fourth $change]
			puts $str "<br>&nbsp;Added: <i>[car $new]</i>"
			foreach real [cdr $new] {
			    puts $str "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>$real</i>"
			}
		    }
		}

		ecchange {
		    puts $str "<li>The entry condition has changed:"
		    puts $str "<br>    Old: <i>\{[second $change]\}</i>"
		    puts $str "<br>    New: <i>\{[third $change]\}</i>"
		}

		movedinfeature {
		    puts $str "<li>Feature '[second $change]' was moved here from system [third $change]."
		}

		newfeature {
		    puts $str "<li>Feature '[second $change]' was added."
		}
		movedoutfeature {
		    puts $str "<li>Feature '[second $change]' was moved to system [third $change]."
		}

		deletedfeature {
		    puts $str "<li>Feature '[second $change]' was deleted."
		}
		default {
		    puts $str "<li>  CHANGE: $change"
		}
	    }
	    puts $str "<br>&nbsp;"
	}
	puts $str "</ul>"
    }
    puts $str "</body></html>"
    close $str
}



		
		
		
		

		

		
		
