########################3
# SCHEME LOAD and SAVE


###############
# LOAD

proc load-scheme {{file {}} {graphp 0}} {
    global GRAMMARS GRAMMAR_FILE INSERTS

    if {$file == {} } {
	set initialdir [default-directory analyses]
	if ![valid-filevar-p GRAMMAR_FILE] {
	    set GRAMMAR_FILE ""
	}
	set file [tk_getOpenFile -initialdir $initialdir -initialfile $GRAMMAR_FILE\
		-title "Please select a Grammar file"\
		-filetypes {{"Scheme Files" {.scheme .gram}}}]

	if { $file == ""} {return}
    }
    clear-systems
    eval [read-file $file]
    # the following has problem in freewrapped versions
    #source $file
    set GRAMMAR_FILE $file
    find-root
    
    # Compile the lookup table (allows us to find all references to a feature in a presel)
    compile-out-realisations 
    if $graphp {
	graph-scheme
    }
}

######################################
##  SAVE SCHEME


proc save-scheme-as {} {

    set file [get-scheme-save-name]
    if { $file == {} } {
	return cancelled
    }
    save-scheme $file
}

proc save-scheme { {file {} }} {
    
    global GRAMMAR_FILE SAVED_SYSTEMS ROOT SCHEME_MODIFIED
    
    if { $file != {} } {
	set GRAMMAR_FILE $file
    }

    if { $GRAMMAR_FILE == {} } {
	# no file defined yet
	return [save-scheme-as]
    }

    # Now save the file
    # delete the old to stop overwriting problem
    # (if this version is shorter than the last, some of the old remains)

    if [file exists $GRAMMAR_FILE] {file delete $GRAMMAR_FILE}
    set str [open $GRAMMAR_FILE w 0600]
    puts $str "defroot $ROOT\n"

    set SAVED_SYSTEMS {}
    set SCHEME_MODIFIED 0
    save-feature $ROOT $str
    close $str
}

proc save-feature {feature str} {
    global FEATURE SYSTEM SAVED_SYSTEMS

    if [info exists FEATURE($feature,realisations)] {
	set reals [string trim $FEATURE($feature,realisations)]
	regsub -all  {\(} $reals "\{" reals
	regsub -all  {\)} $reals "\}" reals
	regsub -all  {"} $reals "\\\"" reals
	regsub -all  {\[} $reals "\\\[" reals
	regsub -all  {\]} $reals "\\\]" reals
	regsub -all  {$} $reals "\\\$" reals
    } else {
	set reals {}
    }
    set state $FEATURE($feature,state)

    if { $reals != "" || $state == "disabled"} {
	puts $str "def-feature $feature \"$reals\" $state\n"
    }

    foreach system $FEATURE($feature,extenders) {
	if ![member $system $SAVED_SYSTEMS] {
	    lappend SAVED_SYSTEMS $system
	    set ec $SYSTEM($system,ec)
	    set features $SYSTEM($system,features)
	    set state $SYSTEM($system,state)
	    puts $str "defsystem $system \"$ec\" \{$features\} $state\n"
	    foreach feat $SYSTEM($system,features) {
		save-feature $feat $str
	    }
	}
    }
}


