# Code to register an new graphing presesentation algorithm.

global GRAPH_REGISTER

proc register-grapher {name proc} {
    global GRAPH_REGISTER GRAPH_TYPES
    set GRAPH_REGISTER($name) $proc
    lappend-new GRAPH_TYPES $name
}

#####################
# PROCESSING

proc graph-scheme {} {

    global ROOT TEMPROOT
    if { $TEMPROOT == {} } {
	graphf $ROOT
    } else {
	graphf $TEMPROOT
    }
	     
}

proc graphf {feature} {
    global GRAPH_VERSION GRAPH_REGISTER
    init-graph $feature
    eval "$GRAPH_REGISTER($GRAPH_VERSION) $feature"
    fix-grapher-scrollbars
}



proc init-graph {feature} {
    global SPOS FPOS GCANV 
    catch [pack-graphframe]
    $GCANV delete all
    if [info exists SPOS] {unset SPOS}
    if [info exists FPOS] {unset FPOS}
    
    # 1. predraw all text widgets to derive real parameters
    predraw-feature $feature 0
}

proc fix-grapher-scrollbars {} {

    global GCANV 
    # Fix up the scrollbars
    set bbox [$GCANV bbox all]
    set x1 [expr [lindex $bbox 0] - 5]
    set y1 [expr [lindex $bbox 1] - 5]
    set x2 [expr [lindex $bbox 2] + 5]
    set y2 [expr [lindex $bbox 3] + 5]
    $GCANV config -scrollregion  [list $x1 $y1 $x2 $y2]
}


#######################################################
## PREDRAW

# Creates all the text widgets, and measures their size

proc predraw-feature {feature depth} {
    global GCANV FEAT_FONT BFEAT_FONT REALISATION_FONT FPOS FEATURE SPOS TOOL
    set string $feature
    if [depth-limit-reached $depth] {
	
	# Mark features which will need ...
	if { $FEATURE($feature,extenders) != {} } {
	    set FPOS($feature,more) 1
	    # append "..." to the sting to use"
	    set string [concat $feature "..."]
	} else {
	    set FPOS($feature,more) 0
	}
    }

    if [ignored-feature-p $feature] {
	set color gray
    } else {
	set color black
    }
    
    set font $FEAT_FONT
    if { $TOOL == "coder" } {
	global DOCUMENT_FEATURES
	if [member $feature $DOCUMENT_FEATURES] {
	    set font $BFEAT_FONT
	}
    }
    set FPOS($feature,wgt) [draw-text $GCANV $string 20 20\
	    "-font $font -tags {feature} -fill $color"]
    set bbox [$GCANV bbox $FPOS($feature,wgt)]
    set FPOS($feature,width) [expr [lindex $bbox 2] - [lindex $bbox 0]]
    set FPOS($feature,height) [expr [lindex $bbox 3] - [lindex $bbox 1]]

    # Predraw any realisations
    set reals [fetch-feature-realisations $feature]
    if {$reals != {}} {
	set FPOS($feature,rwgt)\
		[draw-text $GCANV $reals 20 20\
		"-font $REALISATION_FONT -tags \"$feature realisation\" -fill $color"]
	set bbox [$GCANV bbox $FPOS($feature,rwgt)]
	set FPOS($feature,rwidth) [expr [lindex $bbox 2] - [lindex $bbox 0]]
	set FPOS($feature,rheight) [expr [lindex $bbox 3] - [lindex $bbox 1]]
    }

    if [depth-limit-reached $depth] {
	return
    }
    
    incr depth
    # Predraw systems
    foreach system $FEATURE($feature,extenders) {
	if ![info exists SPOS($system,width)] {
	    predraw-system $system $depth
	}
    }
}

proc predraw-system {system depth} {
    global GCANV SPOS SYSTEM SYS_FONT HIDE_SYSTEM_NAMES
    if {[info exists HIDE_SYSTEM_NAMES] && $HIDE_SYSTEM_NAMES == 1} {
	set SYSTEM($system,string) ""
    } else {
	set SYSTEM($system,string) [split-system-name $system]
    }
    set SPOS($system,wgt) [draw-text $GCANV $SYSTEM($system,string) 20 20\
	    "-font $SYS_FONT -tags {system} -fill black"]
    set bbox [$GCANV bbox $SPOS($system,wgt)]
    set SPOS($system,width) [expr [lindex $bbox 2] - [lindex $bbox 0]]
    set SPOS($system,height) [expr [lindex $bbox 3] - [lindex $bbox 1]]
	 
    foreach feature $SYSTEM($system,features) {
	predraw-feature $feature $depth
    }
}



proc depth-limit-reached {depth} {
    global GRAPH_DEPTH
    if { $GRAPH_DEPTH == 0 || $GRAPH_DEPTH == "all" || $depth < $GRAPH_DEPTH } {
	return 0
    } else {
	return 1
    }
}
    

