# BLOCK LAYOUT

# CODE TO ALLOW layout of the graph using blocks: the layout of each subnetwork
# is always relative to the root of the sublayout.
# Each block representes its PROFILE: the top and bottom contour.
# These contours are fitted together to acheive closest fit.
# (old code just put the highest part of the current block under the lowest
# part of the graph so far).

# Each block consists of the feature or system name, and also a set of subblocks
# for the dependent systems (or features).

# DATA STRUCTURES

# FPOS($feat,yoffset): distance from block-top to top-lhs of feature
# FPOS($feat,prespace):  distance from block-lhs to where subblocks start
# FPOS($feat,blocks):  list of {$sysname $yoffset}, one for each subblock.
#                                  (the rest of the info for the block is stored with the system)
# FPOS($feat,topprofile):  list of points defining the top profile of the subnet, 
#          in terms of a list of points {x y}. The highest point has a y value of 0.
# FPOS($feat,botprofile):  list of points defining the bottom profile of the subnet, 
#          in terms of a list of points {x y}. 

# FPOS($feat,yoffset): distance from block-top to top-lhs of feature
# FPOS($feat,prespace):  distance from block-lhs to where subblocks start
# FPOS($feat,blocks):  list of {$sysname $yoffset}, one for each subblock.
#                                  (the rest of the info for the block is stored with the system)
# FPOS($feat,topprofile):  list of points defining the top profile of the subnet, 
#          in terms of a list of points {x y}. The highest point has a y value of 0.
# FPOS($feat,botprofile):  list of points defining the bottom profile of the subnet, 
#          in terms of a list of points {x y}. 


register-grapher "Simplified Graph" block-layout

proc block-layout {feature} {
    global SYSTEM_QUEUE GCANV  FPOS SPOS DONE_COMPLEXES SYSTEM VISITED
    init-graph $feature
    
    set SYSTEM_QUEUE {}
    set DONE_COMPLEXES {}

    block-layout-feature $feature

    layout-feature-block $feature 10 10

    # Now graph it all
    if [info exists VISITED] {unset VISITED}
    block-show-feature $feature

    # Place the systems with complex ecs
    set profile $FPOS($feature,botprofile)
    while { $SYSTEM_QUEUE != {} } {
	set system [pop SYSTEM_QUEUE]

	lappend DONE_COMPLEXES $system 

	# We create a dummy feature to represent the complex ec
	set feature1 [make-complex-ec-feature $system]
	place-complex-system $system $feature1 $profile

	# Now, place the new system absolutely
	set yoffset [profile-maxheight $profile]
	set correction [mindistance $profile $FPOS($feature1,topprofile) $yoffset]
	set yoffset [expr $yoffset - $correction]

	layout-feature-block $feature1 5 $yoffset

	# Draw the system 
	block-show-feature $feature1

	# Place hyperlinks from the inputs to the complex
	foreach feature $SYSTEM($system,entry_features) {
	    add-hypermark $feature $system
	}

	# Now, update the profile to include the added network
	set newprofile [shift-profile-down $FPOS($feature1,botprofile) $yoffset]
	set profile [merge-profiles $profile $newprofile max]
    }

    fix-grapher-scrollbars
}


proc make-complex-ec-feature {system} {
    global SYSTEM
    set ec $SYSTEM($system,ec)
    set op [pop ec]
    set name "tmp_[car $ec]"
    set div "_"
    foreach feature [cdr $ec] {
	set name "$name$div$op$div$feature"
    }
    return $name
}


proc place-complex-system {system feature profile} {
    global SYSTEM FEAT_FONT FPOS GCANV FEATURE
    #1. Draw the complex ec string (e.g., finite&lexverb)

    regsub -all "tmp_" $feature "... " cstring
    regsub -all "_and_" $cstring { \& } cstring
    regsub -all "_or_" $cstring " / " cstring
    
    set FPOS($feature,wgt) [draw-text $GCANV $cstring 20 20\
	    "-font $FEAT_FONT -fill black -tags {complex}"]
    set bbox [$GCANV bbox $FPOS($feature,wgt)]
    set FPOS($feature,width) [expr [lindex $bbox 2] - [lindex $bbox 0]]
    set FPOS($feature,height) [expr [lindex $bbox 3] - [lindex $bbox 1]]
    set FEATURE($feature,string) $cstring

    # Layout the feature block
    block-layout-feature-1 $feature [list $system]
}
    

proc block-layout-feature {feature} {
     global FEATURE  SPOS SYSTEM_QUEUE DONE_COMPLEXES
    # find which systems need to be layed out

    set child_systems {}
    foreach system $FEATURE($feature,extenders) {

      # Ignore systems which are not drawn
      if  ![info exists SPOS($system,wgt)] {continue}

      # put systems with complex ecs on the SYSTEM_QUEUE
      if [system-has-complex-ec $system] {
	  if ![member $system [concat $SYSTEM_QUEUE $DONE_COMPLEXES]] {
	      lappend SYSTEM_QUEUE $system
	  }
      } else {
        lappend child_systems $system
      }
    }

    block-layout-feature-1 $feature $child_systems
}


proc block-layout-feature-1 {feature child_systems} {
    global FEATURE SYSTEM FPOS SPOS SYSTEM_QUEUE XINCR YINCR
    # Add a block for each subsystem
    set FPOS($feature,blocks) {}
    set botprofile {}
    foreach system $child_systems {
	block-layout-system $system
	if { $botprofile == {} } {
	    set yoffset 0
	    set botprofile $SPOS($system,botprofile)
	    set topprofile $SPOS($system,topprofile)
	} else {
	    # the default yoffset is the maxwidth of the entire system blocks so far
	    # However, we can subtract from this when two profiles can fit together
	    set yoffset [profile-maxheight $botprofile]
	    set correction  [mindistance $botprofile $SPOS($system,topprofile) $yoffset]
	    set yoffset [expr $yoffset - $correction]
	    set botprofile2 [shift-profile-down $SPOS($system,botprofile) $yoffset]
	    set botprofile [merge-profiles $botprofile $botprofile2 max]
	    set topprofile2 [shift-profile-down $SPOS($system,topprofile) $yoffset]
	    set topprofile [merge-profiles $topprofile $topprofile2 min]
	}
	lappend FPOS($feature,blocks) [list $system $yoffset]
   }

   # Now, place the feature 
   
   # Adjust if there are realisations
   set featwidth $FPOS($feature,width)
   set featheight $FPOS($feature,height)
   if { [fetch-feature-realisations $feature] != {}} {
        set featwidth [max $featwidth [expr $FPOS($feature,rwidth) + 7]]
        incr featheight [expr $FPOS($feature,rheight) + $YINCR + 3]
   }   

   set FPOS($feature,prespace) [expr $featwidth + $XINCR]

   # If a curly bracket is needed, add an increment for it.
   set FPOS($feature,cbn) 0
   switch -- [llength $child_systems] {
       0 {
	   set yoffset 0
       }
       1 {
	   # The midpoint of  the feature should be the midpoint its only system name
	   set sys [car $child_systems]
	   set ymid [expr $SPOS($sys,yoffset) + $SPOS($sys,height)/2]
	   set yoffset [expr $ymid - $FPOS($feature,height)/2]
       }

       default {
	   # The feature ymid should be the midpoint of the first+last systems midpoints
	   # This means a) take he midpoint of the first+last systems's toppoints
	   #            b) add a half system height to each
	   #            c) subtract half the feature's height
	   set FPOS($feature,cbn) 1
	   set block1 [car $FPOS($feature,blocks)]
	   set block2 [last $FPOS($feature,blocks)]
	   set sys1 [car $block1]
	   set sys2 [car $block2]
	   set ymin [expr [second $block1] + $SPOS($sys1,yoffset)]
	   set ymax [expr [second $block2] + $SPOS($sys2,yoffset)]

	   set ymid1 [expr $ymin + ($SPOS($sys1,height) / 2)]
	   set ymid2 [expr $ymax + ($SPOS($sys2,height) / 2)]
	   set yoffset [expr $ymid1 + (($ymid2 - $ymid1)  /  2)]
	   # This is the feature midpoint, take off half its height to get the top
	   set yoffset [expr $yoffset  - ($FPOS($feature,height) / 2)]
	  
	   # Calculate the curly bracket sizes
	   # change ymax to the bottom of the last system name
	   incr ymax $SPOS($sys2,height)

	   set cbheight [expr $ymax - $ymin]
	   set cbwidth [min 40 [expr (($ymax -$ymin) / 13) + 16] ]
	   incr FPOS($feature,prespace) $cbwidth
	   set  FPOS($feature,cbwidth) $cbwidth
	   set  FPOS($feature,cbheight) $cbheight
	   set  FPOS($feature,cbystart) $ymin
       }
   }

    # Now adjust the profiles to take into account  the feature profile
    # Take off the starting point, and add in this features profile
    set feattopprofile  [list [list 0 $yoffset] [list $FPOS($feature,prespace) $yoffset]]
    set featmaxy [expr $yoffset + $featheight + $YINCR]
    set featbotprofile [list [list 0 $featmaxy] [list $FPOS($feature,prespace) $featmaxy]]

    if { $child_systems != {} } {
	# We have child profiles to merge
	set botprofile [shift-profile-right $botprofile $FPOS($feature,prespace)] 
	set topprofile [shift-profile-right $topprofile $FPOS($feature,prespace)] 
	set feattopprofile [concat $feattopprofile [cdr $topprofile]]
	set featbotprofile [concat $featbotprofile [cdr $botprofile]]
    }
    # Store the results
    set FPOS($feature,topprofile) $feattopprofile
    set FPOS($feature,botprofile) $featbotprofile
    set FPOS($feature,yoffset) $yoffset

}


          
proc shift-profile-right {profile offset} {
    if { $profile == {}} {return {}}
    set head [car $profile]
    return [concat [list [list [expr [car $head] + $offset] [second $head]]]\
         [shift-profile-right [cdr $profile] $offset]]
}

  
proc shift-profile-down {profile offset} {
    if { $profile == {}} {return {}}
    set head [car $profile]
    return [concat [list [list [car $head]  [expr [second $head] + $offset]]]\
         [shift-profile-down [cdr $profile] $offset]]
}


proc block-layout-system {system} {
     global FEATURE SYSTEM FPOS SPOS XINCR YINCR
      set features $SYSTEM($system,features)

    # Add a block for each subsystem
    set SPOS($system,blocks) {}
    set botprofile {}
    foreach feature $features {
       block-layout-feature $feature
       if { $botprofile == {} } {
	   set yoffset 0
	   set botprofile $FPOS($feature,botprofile)
	   set topprofile $FPOS($feature,topprofile)
       } else {

	   # the default yoffset is the maxwidth of the entire feature blocks so far
	   # However, we can subtract from this when two profiles can fit together
	   set yoffset [profile-maxheight $botprofile]
	   set correction  [mindistance $botprofile $FPOS($feature,topprofile) $yoffset]
	   set yoffset [expr $yoffset - $correction]
	   set botprofile2 [shift-profile-down $FPOS($feature,botprofile) $yoffset]
	   set botprofile [merge-profiles $botprofile $botprofile2 max]
	   set topprofile2 [shift-profile-down $FPOS($feature,topprofile) $yoffset]
	   set topprofile [merge-profiles $topprofile $topprofile2 min]
       }
       lappend SPOS($system,blocks) [list $feature $yoffset]
    }

    # Now, place the system name 

    set syswidth $SPOS($system,width) 
    set sysheight $SPOS($system,height)

    # if the system has a complex entry condition, allow space for it
    if [system-has-complex-ec $system] {
       incr syswidth  8
    }

    set SPOS($system,prespace) [expr $syswidth + $XINCR]

    # If a curly bracket is needed, add an increment for it.
    switch -- [llength $features] {
      0 {set yoffset 0}
      1 {

	  # The midpoint of  the system should be the midpoint its only feature name
	   set feat [car $features]
	   set ymid [expr $FPOS($feat,yoffset) + $FPOS($feat,height)/2]
	   set yoffset [expr $ymid - $SPOS($system,height)/2]
      }
      default {

	  # The system ymid should be the midpoint of the first+last feature midpoints
	  # This means a) take he midpoint of the first+last feature's toppoints
	  #            b) add a half feature height
	  #            c) subtract half the system height
	  set block1 [car $SPOS($system,blocks)]
	  set block2 [last $SPOS($system,blocks)]
	  set feat1 [car $block1]
	  set feat2 [car $block2]
	  set pos1 [expr [second $block1] + $FPOS($feat1,yoffset)]
	  set pos2 [expr [second $block2] + $FPOS($feat2,yoffset)]
	  set yoffset [expr $pos1 + (($pos2 - $pos1)  /  2)]
	  set yoffset [expr $yoffset + ( $FPOS($feat1,height) / 2) - ($SPOS($system,height) / 2)]
      }
    }

    # Now adjust the profiles to take into account  the feature profile
    # Take off the starting point, and add in this systems profile
    set systopprofile  [list [list 0 $yoffset] [list $SPOS($system,prespace) $yoffset]]
    set sysmaxy [expr $yoffset + $sysheight + $YINCR]
    set sysbotprofile [list [list 0 $sysmaxy] [list $SPOS($system,prespace) $sysmaxy]]

    if { $features != {} } {
	# We have child profiles to merge
	set topprofile [shift-profile-right $topprofile $SPOS($system,prespace)] 
	set botprofile [shift-profile-right $botprofile $SPOS($system,prespace)] 
	set systopprofile [concat $systopprofile [cdr $topprofile]]
	set sysbotprofile [concat $sysbotprofile [cdr $botprofile]]
    }
    set SPOS($system,yoffset) $yoffset
    set SPOS($system,topprofile) $systopprofile
    set SPOS($system,botprofile) $sysbotprofile

}

proc merge-profiles {profile1 profile2 minOrMax} {
    # Take each 
    set p1 [pop profile1]
    set p2 [pop profile2]

    # Calculate the new start point
    # Both profiles should start at x=0, so assume this.
    # Keep the ypoint which is minormax
    set x 0
    set y [$minOrMax [second $p1] [second $p2]]
    set newprofile  [list [list $x $y]]
    set lasty -9

    while { $profile1 != {} && $profile2 != {} } {
	
	#Now, take the next point off each list, and keep the highest/lowest
	# But only upto the xend of the other
	set p1 [pop profile1]
	set p2 [pop profile2]

	set x1 [car $p1]
	set x2 [car $p2]
	set y1 [second $p1]
	set y2 [second $p2]

	set x [min $x1 $x2]
	set y [$minOrMax $y1 $y2]

	# We have the new vector. If it continues the last, drop the last.
	if { $lasty == $y } {
	    set newprofile [lrange $newprofile 0 end-1]
	}
	lappend newprofile [list $x $y]
	set lasty $y

	# Now, workout which points to put backon the queues
	if { $x1 > $x } {
	    push [list $x1 $y1] profile1
	}
	if { $x2 > $x } {
	    push [list $x2 $y2] profile2
	}
    }
    # One list is finished, so append the other list to the new one.
    set toAppend {}
    if { $profile1 != {} } {
	set toAppend $profile1
    }
    if { $profile2 != {} } {
	set toAppend $profile2
    }
    
    # If the first point of the toAppend list continues, drop last
    if { $lasty == [second [car $toAppend]] } {
	set newprofile [lrange $newprofile 0 end-1]
    }

    set newprofile [concat $newprofile $toAppend]
    # Return the profile
    return $newprofile
}
	

proc mindistance {profile1 profile2 prof1height} {
#    return 0
    # Distance is smallest distance between two vectors 
    # The first points are redundant - remove
    pop profile1
    pop profile2
    
    # Cycle through the vectors, recording the minimums
    # Note the first points can be expected to overlap, and this relation
    # is maintained in the loop
    set mindiff 999
    set p1 [pop profile1]
    set p2 [pop profile2]
    while { $p1 != {} && $p2 != {} } {
	set mindiff [min $mindiff [expr $prof1height - [second $p1] + [second $p2]]]
	# Decide which vector is consumed
	set x1 [car $p1]
	set x2 [car $p2]
	if { $x1 < $x2 } {
	    set p1 [pop profile1]
	} elseif { $x1 > $x2 } {
	    set p2 [pop profile2]
	} else {
	    set p1 [pop profile1]
	    set p2 [pop profile2]
	}
    }

    return $mindiff
}

proc profile-maxheight {profile} {
    set max 0
    foreach point $profile {
	set max [max $max [second $point]]
    }
    return $max
}


###################################################
#### DRAWING

proc draw-block {feature} {
    global FPOS
    draw-profile $FPOS($feature,topprofile)
    draw-profile $FPOS($feature,botprofile)
}

proc layout-feature-block {feature x y} {
    global FPOS 
    set FPOS($feature,x) $x
    set FPOS($feature,y) [expr $y + $FPOS($feature,yoffset)]
    set FPOS($feature,xend) [expr $FPOS($feature,x) + $FPOS($feature,width)]
    set FPOS($feature,ymid) [expr $FPOS($feature,y) + ($FPOS($feature,height) / 2)]
    set FPOS($feature,yend) [expr $FPOS($feature,y) + $FPOS($feature,height)]
    set newx [expr $x + $FPOS($feature,prespace)]

    foreach block $FPOS($feature,blocks) {
	set yoffset [second $block]
	layout-system-block [car $block] $newx [expr $y + $yoffset]
    }
}

proc layout-system-block {system x y} {
    global SPOS 
    set SPOS($system,x) $x
    set SPOS($system,y) [expr $y + $SPOS($system,yoffset)]
    set SPOS($system,xend) [expr $SPOS($system,x) + $SPOS($system,width)]
    set SPOS($system,ymid) [expr $SPOS($system,y) + ($SPOS($system,height) / 2)]
    set SPOS($system,yend) [expr $SPOS($system,y) + $SPOS($system,height)]
    set SPOS($system,prespace0) 0
    set newx [expr $x + $SPOS($system,prespace)]

    foreach block $SPOS($system,blocks) {
	set yoffset [second $block]
	layout-feature-block [car $block] $newx [expr $y + $yoffset]
    }
}


proc draw-profile {profile} {
    global GCANV
    set p1 [pop profile]
    set x1 [car $p1]
    while { $profile != {} } {
	set p2 [pop profile]
	set y1 [second $p2]
	if [info exists x2] {draw-line $GCANV $x2 $y2 $x1 $y1}
	set x2 [car $p2]
	set y2 $y1
	draw-line $GCANV $x1 $y1 $x2 $y2
	set x1 $x2
    }
}



#######################################################
## SHOW-FEATURE/SYSTEM

# Moves text to final position, and draws connecting lines

proc block-show-feature {feature} {
    global  FPOS FEATURE SPOS GCANV SHEIGHT ROOT SYSTEM

    set color "black"
    if [info exists FEATURE($feature,system)] {
	if { $SYSTEM($FEATURE($feature,system),state) == "disabled"} {
	    set color "gray"
	    $GCANV  itemconfigure $FPOS($feature,wgt) -fill $color
	}
    }

    # Position the feature
    # (shift feature up 2 pixels because the actual centre is that of capitals,
    # and the bulk of letters (acegijmnopqrsuvwxyz) fall under the midpoint.

    $GCANV coords $FPOS($feature,wgt) $FPOS($feature,x) [expr $FPOS($feature,y) - 2]

   
    # Move the Realisations
    if {[fetch-feature-realisations $feature] != {}} {
	$GCANV coords $FPOS($feature,rwgt) \
		[expr $FPOS($feature,x) + 5] [expr $FPOS($feature,yend) + 3]
	$GCANV itemconfigure $FPOS($feature,rwgt)  -fill $color

	# draw a rectangle around
	set bbox [$GCANV bbox $FPOS($feature,rwgt)]
	draw-rect $GCANV [expr [lindex $bbox 0] -1]  [expr [lindex $bbox 1] -1]\
		[expr [lindex $bbox 2] + 1]  [expr [lindex $bbox 3] +1]
    }
    
    # display the extending systems
    foreach block $FPOS($feature,blocks) {
	set system [car $block]
	block-show-system $system
	if { $FPOS($feature,cbn) == 0 } {
	     block-draw-link $feature $system
	}
    }

    # Now, draw a curly bracket if needed
    if { $FPOS($feature,cbn) == 1 } {
	set sys1 [car [car $FPOS($feature,blocks)]]
	set ymin [expr $SPOS($sys1,y) - 2]
	set ymax [expr $ymin + $FPOS($feature,cbheight)]
	set xmin [expr [end-of-feature $feature] + 5]
	set xmax [expr $xmin + $FPOS($feature,cbwidth)]
	draw-open-curly-bracket $GCANV $xmin $ymin $xmax $ymax
    }

    # If there are hyperlinks on this feature not yet shown, draw them
    if [info exists FPOS($feature,to_hyperlink)] {
	foreach sys $FPOS($feature,to_hyperlink) {
	    add-hypermark $feature $sys
	}
    }
}


proc block-show-system {system} {
    global SYSTEM SYSTEM_YRANGE SPOS FPOS GCANV

    set features $SYSTEM($system,features)
    if { $features == {} } {return}

    if { $SYSTEM($system,state) == "disabled"} {
	set color "gray"
	$GCANV itemconfigure $SPOS($system,wgt) -fill $color
    } else {
	set color "black"
    }

    # if a single line, shift the system up a half-width
    $GCANV coords $SPOS($system,wgt) $SPOS($system,x) $SPOS($system,y)
    if { $SYSTEM($system,string) == $system} {
	$GCANV move $SPOS($system,wgt) 0 [expr - ($SPOS($system,height) / 2)]
    }

    # Draw any complex ec
    if [complex-ec-p $SYSTEM($system,ec)] {
	# to be completed
	# block-draw-ec $system
    }
    # Define the x position of the upbar (using one feature)
    set xmid [expr $FPOS([car $features],x) -6]
    
    # Draw the system underline
    set ymid $SPOS($system,ymid) 
    draw-line $GCANV\
                    $SPOS($system,x) $ymid\
                    $xmid $ymid "-fill $color"

    # Draw the system up bar
    # the +1 here is to fix a minor display error
    if { [llength $features] > 1 } {
	set feat1 [car $features]
	set feat2 [last $features]
	set ymin $FPOS($feat1,ymid)
	set ymax $FPOS($feat2,ymid)
	draw-line $GCANV $xmid $ymin $xmid $ymax "-fill $color"
    }

    # Draw each feature
    foreach feature $features {
	block-show-feature $feature
	draw-line $GCANV\
                     [expr $FPOS($feature,x) -2] $FPOS($feature,ymid)\
		     $xmid $FPOS($feature,ymid)\
		     "-fill $color"
    }
}

   

proc block-draw-link {feature system} {
    global FPOS SPOS GCANV SYSTEM EC_HEIGHT
    if { $SYSTEM($system,state) == "disabled"} {
	set color "gray"
    } else {
	set color "black"
    }
    
    set xend [expr $SPOS($system,x) - $SPOS($system,prespace0)]

    # If we end at a complex ec, work out the end pos
    
    # For now, no drawing of complex ecs
    if { 1 == 0 && [complex-ec-p $SYSTEM($system,ec)]}  {

	# Work out in what pos the feature is in the ec
	set pos [lsearch -exact $SYSTEM($system,entry_features) $feature]
	set count [llength $SYSTEM($system,entry_features)]
	# Divide EC_HEIGHT by count+1 to get the interval size
	# Multiply by pos+1 to get the distance from the top,
	# then subtract half EC_HEIGHT to place it relative to the middle
	set interval [expr $EC_HEIGHT / ($count + 1)]
	set offset [expr $interval * ( $pos + 1) - ($EC_HEIGHT / 2)]
	set yend [expr $SPOS($system,ymid) + $offset]
    } else {
	set yend $SPOS($system,ymid)
    }

    draw-line $GCANV\
       [expr $FPOS($feature,xend) + 2]  $FPOS($feature,ymid)\
       $xend $yend "-fill $color"
}



proc block-draw-ec {system} {
    # only one level at present
    global SHEIGHT SPOS SYSTEM GCANV REALISATION_FONT EC_HEIGHT
    
    switch -- [lindex $SYSTEM($system,ec) 0] {
	and { set Symb "\}" }
	or { set Symb "\]" }
    }
    set x [expr $SPOS($system,x) - $SPOS($system,prespace0)]
    set y [expr $SPOS($system,ymid) - ( $EC_HEIGHT / 2 ) -5]
    draw-text $GCANV $Symb $x $y "-font [fetch-font "Times" $EC_HEIGHT]"
}


proc add-hypermark {feature system} {
    global FEATURE SYSTEM FPOS RHEIGHT_DFLT REALISATION_FONT GCANV 

    # if the feature is not yet drawn, add it to the hyperlink queue
    if ![info exists FPOS($feature,xend)] {
	lappend FPOS($feature,to_hyperlink) $system
	return
    }


    # Work out where hyperlinks for this feature start
    set xpos $FPOS($feature,xend)
    set ypos $FPOS($feature,y)

    # Work out the space beforehand
    if [info exists FPOS($feature,hyperspace)] {
	set prespace $FPOS($feature,hyperspace)
    } else { 
	set prespace 1
    }

    # Place & draw the feature
    incr xpos $prespace
    set wgt [draw-text $GCANV "*" $xpos $ypos  "-font $REALISATION_FONT  -fill green"]

    # Bind the event
    $GCANV bind $wgt <Button-1> "goto-system $system"

    # Work out the widget size
    set bbox [$GCANV bbox $wgt]
    set FPOS($feature,hyperspace) [expr $prespace + [lindex $bbox 2] - [lindex $bbox 0]]
}

proc scroll-to {x y} {
    global GCANV
   
    set bbox [$GCANV bbox all]
    set xfrac [expr $x / [lindex $bbox 2]]
    set yfrac [expr $y / (1.000000 * [lindex $bbox 3])]
    $GCANV xview moveto $xfrac
    $GCANV yview moveto $yfrac
}

proc tmp {} {
    global GCANV
    $GCANV bind <Motion> {
	show-closest %x %y
    }
}


proc show-closest {x y} {
    global GCANV OLDSET

    if ![info exists OLDSET] {set OLDSET {}}
    set cands [$GCANV find overlapping [expr $x  - 100] [expr $y - 100] [expr $x  + 100] [expr $y + 100]]

    # Unset the wigets out of distance
    set OLDSET [set-difference $OLDSET $cands]
    foreach wgt $OLDSET {
	if [member system [$GCANV gettags $wgt]] {
	    $GCANV itemconfig $wgt -font [font create -size 8  -family {Times New Roman}]
	} else {
	    $GCANV itemconfig $wgt -font [font create -size 12  -family {Times New Roman}]
	}
    }
    set OLDSET {}
    foreach cand $cands {
	if {[$GCANV type $cand] == "text" } {

	    lappend OLDSET $cand
	    set bbox [$GCANV bbox $cand]
	    set x2 [expr [lindex $bbox 0] + ([lindex $bbox 2] - [lindex $bbox 0])/2]
	    set y2 [expr [lindex $bbox 1] + ([lindex $bbox 3] - [lindex $bbox 1])/2]
	    set dist [expr sqrt( (( $x2-$x ) * ( $x2-$x )) + (( $y2-$y ) * ( $y2-$y )))]
	    if [member system [$GCANV gettags $cand]] {
		set base 8
	    } else {
		set base 12
	    }
	    set fontincr [expr $base + int((100 - $dist)/ 5)]
	    if { $fontincr > $base } {
		$GCANV itemconfig $cand -font [font create -size $fontincr -family {Times New Roman}]
	    }
	}
    }
   
}

	
