####################################
## System EDITOR


# Routines for system network editing
# Initially defines popup menus
# then defines these operations

################################
# POPUP MENUS


proc feature-click {pane wid} {
    global FEATURE ROOT

    set feature [$pane itemcget $wid -text]

    # take off " ..." added by the grapher  (if present)
    if { [string first " ..." $feature] != -1 } {
	set feature [string range $feature 0 end-4]
    }
    set choices { "Add System"      "gr-add-system-under"
	"Rename Feature"  "gr-rename-feature"
	"Delete Feature"  "gr-delete-feature"
	"Edit Realisations"  "gr-edit-realisations"
    }

    if { $feature != $ROOT } {
	 lappend choices "Move Up" "gr-move-feature-up"	"Move Down" "gr-move-feature-down"\
		 "Graph from here" "graph-from-here" "Graph from parent" "graph-from-parent"
    }
     
    if { $FEATURE($feature,state) == "disabled"} {
	lappend choices "Unignore Feature" "gr-unignore-feature"
    } else {
	lappend choices "Ignore Feature" "gr-ignore-feature"
    }


    switch -- [current-tool] {
	parser  {
	    lappend choices "Describe Feature" "show-feature-documentation"
	    lappend choices "Show Lexemes" "show-lexemes-with-feature"
	}

	grapher {
	     lappend choices "Describe Feature" "show-feature-documentation"
	}

	coder {
	    global DOCUMENT_FEATURES
	    if [member $feature $DOCUMENT_FEATURES] {
		lappend choices "Remove from Document Features" "gr-delete-doc-feature"
	    } else {
		lappend choices "Set as Document Feature" "gr-add-doc-feature"
	    }
	    lappend choices "Show Examples" "show-examples-with-feature"
	}
    }
    menu-select $pane $wid  $feature $choices
#	"Hide Subnet"     "gr-hide-children"
}



proc system-click {pane wid} {
    global SYSTEM CURRENT_WID 
    set CURRENT_WID $wid
    set system [$pane itemcget $wid -text]
    regsub "\n" $system {} system
    set choices {	"Add Feature"    "gr-add-feature"
	"Rename System"  "gr-rename-system"
	"Delete System"  "gr-delete-system"
	"Change Entry Condition" "gr-change-ec"
	"Move Up" "gr-move-system-up"
	"Move Down" "gr-move-system-down"
    }
    if { $SYSTEM($system,state) == "disabled"} {
	lappend choices "Unignore Subnet" "gr-unignore-system"
    } else {
	lappend choices "Ignore Subnet" "gr-ignore-system"
    }

    if [member [current-tool] {"parser" "grapher"}] {
	lappend choices "Describe System" "show-system-documentation"
    }
   
    menu-select $pane $wid $system $choices
}


proc realisation-click {pane wid} {
    global SYSTEM CURRENT_WID
    set CURRENT_WID $wid
    set feature [car [ldelete [$pane gettags $wid] realisation]]
    gr-edit-realisations $feature
}

proc menu-select {pane wid arg options} {
    # puts "menu-select: $options"
    global  GCANVX GCANVY
    set coords [screen-coords $wid $pane]
    set opts {}
    set cmds {}
    
    for {set x 0} {$x<[llength $options]} {set x [expr $x + 2]} {
	lappend opts [lindex $options $x]
	lappend cmds [lindex $options [expr $x + 1]]
    }
    set GCANVX  [expr int([lindex $coords 0])]
    set GCANVY [expr int([lindex $coords 1])]
    set selected [popup-choose-from-list $opts $GCANVX $GCANVY]
    if {$selected == "cancelled"} {return}
    update
    set optpos [lsearch -exact  $opts $selected]
    eval "[lindex $cmds $optpos] $arg"
}

###############################################
# Edit Functions

# 1. Feature ops

proc gr-add-system-under {feature} {
    add-system-under $feature
    mark-scheme-modified
    graph-scheme   
}

proc gr-edit-realisations {feature} {
    global FEATURE
    if ![info exists FEATURE($feature,realisations)] {
	set FEATURE($feature,realisations) {}
    }

    set result [edit-realisations $feature  $FEATURE($feature,realisations)]
    if { $result == "cancelled"} {return}

    set result [string trim $result]

    # Check that a change has in fact occurred
    if {$result == $FEATURE($feature,realisations) } {
	return
    }

    # Change the realisations
    if {$result == ""} {
	unset FEATURE($feature,realisations)
    } else {
	set FEATURE($feature,realisations) $result
    }

    mark-scheme-modified
    graph-scheme   
}

proc edit-realisations {feature string} {
    
    #  position this near top of screen
    global f
    set f [modal-dialog .asker ]
    wm geometry $f +20+20
    frame $f.top 
    wm title $f "Realisations for feature: $feature"
    text $f.top.entry -bg white -relief sunken\
	    -width 60 -borderwidth 1 -height 6
    $f.top.entry insert 1.0 $string
    pack  $f.top.entry -side left -padx 5

    frame $f.bot 
    button $f.bot.done -text "Done" -command {
	 global f; return-from-modal .asker [$f.top.entry get 1.0 end]
    }
    button $f.bot.cancel -text "Cancel" -command  "return-from-modal .asker cancelled"
    pack $f.bot.done  $f.bot.cancel -side left -padx 15
    
    bind $f <Control-c> "return-from-modal .asker cancelled"
   
    pack $f.top $f.bot -side top

    post-modal $f
}



proc gr-rename-feature {feature} {
    global FEATURE ROOT GCANVX GCANVY SYSTEM
    set newfeature [standardise-name [GetValue "Rename feature to: " $feature $GCANVX $GCANVY]]
    if { $newfeature == {} } {return}
    set newfeature [string tolower $newfeature]
    if { $newfeature == $feature } {return }
    if  [feature-exists-p $newfeature] { 
	
	if { $newfeature == $ROOT || $FEATURE($feature,extenders) != {} } {
	    tell-user "Feature $newfeature already exists. Cannot rename."
	    return 0
	}
	
	set thissystem $FEATURE($feature,system)
	set system $FEATURE($newfeature,system)
	set response [ask-user "Feature already exists in system $system. Should I move it?"]
	switch -- $response {
	    no {return 0}
	    yes {

		set pos [lsearch -exact $SYSTEM($thissystem,features) $feature]
		delete-feature $feature
		change-feature-system $newfeature $thissystem
		set features [ldelete $SYSTEM($thissystem,features) $newfeature]
		set SYSTEM($thissystem,features)\
			[concat [lrange $features 0 [expr $pos - 1]]\
			   $newfeature [lrange $features $pos end] ]
	    }
	}
    } else {
	rename-feature $feature $newfeature
	
	# Change all ecs which refer to this feature
	foreach idx [array names FEATURE "*,realisations"] {
	    if { [string first $feature $FEATURE($idx)] > -1 } {
		set count [regsub -all -line -lineanchor "{\[ \{\]}$feature{\[ \}$\]}" $FEATURE($idx) HELLO text1]
		if { $count > 0} {
		    puts "OLD: $FEATURE($idx)\nNEW: $text1"
		    # set FEATURE($idx) $text1
		}
	    }
	}
    }
   
    mark-scheme-modified
    graph-scheme
}



proc gr-hide-children {feature} {
}


#2. System ops

proc gr-rename-system {system} {
    global FEATURE ROOT SYSTEM GCANVX GCANVY
    set newsystem [standardise-name [GetValue "Rename system to: " $system $GCANVX $GCANVY]]
    if { $newsystem == {} } {return}
    set newsystem [string toupper $newsystem]
    if { $system == $newsystem } {return}

    regsub -all " " $newsystem "-" newsystem
    if  [info exists SYSTEM($newsystem,ec)] { 
	tell-user "System: $newsystem  already exists."
	return 0
    }
    rename-system $system $newsystem
    mark-scheme-modified
   graph-scheme
}



proc standardise-name {name} {
    set name [string trim $name]
    
    # Replace spaces   
    regsub -all " " $name "-" name
    
    # remove the bad chars
    regsub -all {[^a-zA-Z0-9\-_]} $name "" name

    return $name
}

proc gr-add-feature {system} {
    global FEATURE SYSTEM ROOT
    set feature [GetValue "Type in name of feature"]
    set feature [standardise-name $feature]
    if { $feature == {} } {return}
    set feature [string tolower $feature]

    if { $feature == $ROOT} {
	tell-user "Cannot insert: feature $feature is the root of the network!"
    }
    if  [feature-exists-p $feature] { 
	set response [ask-user "Feature already exists in system $FEATURE($feature,system). Should I move it?"]
	switch -- $response {
	    no {return 0}
	    yes {
		change-feature-system $feature $system
		
	    }
	}
    } else {
	add-feature $feature $system
    }
    
    mark-scheme-modified
    graph-scheme
}

proc gr-delete-feature {feature} {
    global ROOT FEATURE SYSTEM 
     if { $feature == $ROOT } {
	  tell-user "Can't delete root feature!"
        return
     }
     if { $FEATURE($feature,extenders) == {}} {
	 if { [current-tool] == "coder"} {
	     set response [ask-user "The coding of  feature '$feature' will be erased. Are you sure?"]
	 } else {
	     set response [ask-user "Are you sure you want to delete feature '$feature'?"]
	 }
     } else {
	 set response [ask-user "This will delete all systems below feature '$feature'. Are you sure?"]
     }
     switch -- $response {
	 no {return}
     }
    set system $FEATURE($feature,system)
    delete-feature $feature
    if { $SYSTEM($system,features) == {}} {delete-system $system}
    mark-scheme-modified
    graph-scheme
}

proc gr-delete-system {system} {
   set response [ask-user "This will delete all systems below this system. Are you sure?"]
    switch -- $response {
	no {return}
    }
    delete-system $system
    mark-scheme-modified
    graph-scheme
}

proc gr-hide-system {system} {
}

proc gr-ignore-system {system} {

    ignore-systems-from $system
    if { [current-tool] == "coder"} {
	calculate-all-node-completeness resolve_gates
    }
    graph-scheme
}


proc gr-unignore-system {system} {

    unignore-systems-from $system
    if { [current-tool] == "coder"} {
	calculate-all-node-completeness resolve_gates
    }
    graph-scheme
}

proc gr-move-system-up {system} {
    move-system-up $system
    mark-scheme-modified
    graph-scheme
}

proc gr-move-system-down {system} {
    move-system-down $system
    mark-scheme-modified
    graph-scheme
}

proc gr-move-feature-up {feature} {
    move-feature-up $feature
    mark-scheme-modified
    graph-scheme
}

proc gr-move-feature-down {feature} {
    move-feature-down $feature
    mark-scheme-modified
    graph-scheme
}


proc gr-add-gloss {system} {
}



proc gr-unignore-feature {feature} {
    unignore-from-feature $feature
    mark-scheme-modified
    if { [current-tool] == "coder"} {
	calculate-all-node-completeness resolve_gates
    }
    graph-scheme
}

proc gr-ignore-feature {feature} {
    ignore-from-feature $feature
    mark-scheme-modified
    if { [current-tool] == "coder"} {
	calculate-all-node-completeness resolve_gates
    }
    graph-scheme
}



proc complex-click {pane wid} {

    set coords [screen-coords $wid $pane]
    set x [expr int([lindex $coords 0])]
    set y [expr int([lindex $coords 1])]

    set complex [$pane itemcget $wid -text]
    set features [split [string range $complex 4 end] {&/}]
    set options {}
    foreach feature $features {
	lappend options [string trim $feature]
    }
    set choice [popup-choose-from-list $options $x $y]
    if { $choice == "cancelled"} {return}

    hypershow-feature $choice $pane
}

proc hypershow-feature {feature pane {type feature}} {
    set bbox [$pane bbox all]
    set xtot [expr [lindex $bbox 2] - [lindex $bbox 0]]
    set ytot [expr [lindex $bbox 3] - [lindex $bbox 1]]
    foreach wgt [$pane find withtag $type] {
	set wgttext  [$pane itemcget $wgt -text]
	regsub -all "\n" $wgttext "" wgttext
	if { $wgttext == $feature } {
	    set bbox1 [$pane bbox $wgt]
	    set xfrac [expr [lindex $bbox1 0] - [lindex $bbox 0] - 30 ]
	    set yfrac [expr [lindex $bbox1 1] - [lindex $bbox 1] -30]
	    $pane xview moveto [expr (1.00 * $xfrac / $xtot)]
	    $pane yview moveto [expr (1.00 * $yfrac / $ytot)]
	    update
	    $pane itemconfig $wgt -fill "red"
	    update
	    after 4000 "$pane itemconfig $wgt -fill black"
	}
    }
}


proc graph-from-here {feature} {
    global TEMPROOT
    set TEMPROOT $feature
    graph-scheme
}

proc graph-from-parent {feature} {
    global TEMPROOT SYSTEM FEATURE
    set system $FEATURE($feature,system)
    set ec [flatten $SYSTEM($system,ec)]
    set parent [car $ec]
    set TEMPROOT $parent
    graph-scheme
}
    
proc gr-add-doc-feature {feature} {
    add-doc-feature $feature
    
    # redraw 
    graph-scheme
    calculate-all-node-completeness resolve_gates
    mark-edited
}

proc add-doc-feature {feature} {
    global DOCUMENT_FEATURES FEATURE

    # 1. Add to the list
    lappend-new DOCUMENT_FEATURES $feature
    set FEATURE($feature,state) active

    # Ignore all co-features of this one
    foreach feat2 [co-features $feature] {
	delete DOCUMENT_FEATURES $feat2 ;# stop two feats from same system being selected
	ignore-from-feature $feat2
    }
}


proc gr-delete-doc-feature {feature} {
    global DOCUMENT_FEATURES FEATURE

    # 1. remove from the list
    delete DOCUMENT_FEATURES $feature

    # Enable cofeatures
    foreach feat2 [co-features $feature] {
	unignore-from-feature $feat2

    }

    # Ensure the graph shows these specially
    # (done in grapher)

    # redraw 
    graph-scheme
    mark-edited
}
