

####################################################		   
# Define a GRAPH Pane

proc GraphPane {path args} {
    
    # Define the canvas (using defaults)
    canvas $path -bg white -borderwidth 3 -relief sunken
    
    # Add any of the args
    for {set i 0} {$i < [llength $args]} {incr i 2} {
	$path config  [lindex $args $i] [lindex $args [expr $i + 1]]
    }
    
    # Add some bindings
    $path bind feature <ButtonRelease> "grapher-show-menu feature $path"
    $path bind system  <ButtonRelease> "grapher-show-menu system $path"
    $path bind realisation <ButtonRelease> "grapher-show-menu realisation $path"
    $path bind feature <Enter> {rollover-message "Click on this feature for a list of options"}
    $path bind feature <Leave> "rollover-message {}"
    $path bind system <Enter> {rollover-message "Click on this system for a list of options"}
    $path bind system <Leave> "rollover-message {}"
    $path bind realisation <Enter> {rollover-message "Click on the realisations to edit them"}
    $path bind realisation <Leave> "rollover-message {}"
    $path bind complex  <ButtonRelease> "grapher-show-menu complex $path"
    $path bind complex <Enter> {rollover-message "Click on the complex to goto one of the features"}
    $path bind complex <Leave> "rollover-message {}"

    # Return the path
    return $path
}


proc grapher-show-menu {type pane} {
    set wgt [$pane find withtag current]
    switch -- $type {
	feature {feature-click $pane $wgt}
	system {system-click $pane $wgt}
	realisation {realisation-click $pane $wgt}
	complex {complex-click $pane $wgt}
    }
}


##############################
## ZOOMING

proc zoom-in {} {
    global ZOOM
    incr ZOOM
    set-zoom $ZOOM
    graph-scheme
}

proc zoom-out {} {
    global ZOOM
    incr ZOOM -1
    set-zoom $ZOOM
    graph-scheme
}


proc set-zoom-old {zoom} {
    global FONT ZOOM SHEIGHT FHEIGHT RHEIGHT SHEIGHT_DFLT FHEIGHT_DFLT RHEIGHT_DFLT\
	    GR_FONT_FAMILY SYS_FONT FEAT_FONT REALISATION_FONT EC_FONT EC_HEIGHT
    set ZOOM $zoom
    set SYS_FONT  [fetch-font $GR_FONT_FAMILY [expr $SHEIGHT_DFLT + $ZOOM]]
    set FEAT_FONT [fetch-font $GR_FONT_FAMILY [expr $FHEIGHT_DFLT + $ZOOM]]
    set BFEAT_FONT [fetch-font $GR_FONT_FAMILY [expr $FHEIGHT_DFLT + $ZOOM] bold]
    set REALISATION_FONT [fetch-font "Times"  [expr $RHEIGHT_DFLT + $ZOOM] italic]
    set SHEIGHT [font-height $SYS_FONT]
    set FHEIGHT [font-height $FEAT_FONT]
    set RHEIGHT [font-height $REALISATION_FONT]
    set EC_HEIGHT [expr 18 + $ZOOM]
    set EC_FONT [fetch-font $GR_FONT_FAMILY $EC_HEIGHT]
}


proc set-zoom {zoom} {
    global FONT ZOOM SHEIGHT FHEIGHT RHEIGHT SHEIGHT_DFLT FHEIGHT_DFLT RHEIGHT_DFLT\
	    GR_FONT_FAMILY SYS_FONT FEAT_FONT BFEAT_FONT REALISATION_FONT EC_FONT \
	    EC_HEIGHT XINCR_DFLT YINCR_DFLT XINCR  YINCR
    set ZOOM $zoom
    set factor [expr $ZOOM / 100.00]

    set sfontsize [expr int($SHEIGHT_DFLT * $factor)]
    if { $sfontsize < 1 } {set sfontsize 1}
    set SYS_FONT  [fetch-font $GR_FONT_FAMILY $sfontsize]

    set ffontsize [expr int($FHEIGHT_DFLT * $factor)]
    if { $ffontsize < 1 } {set ffontsize 1}
    set FEAT_FONT [fetch-font $GR_FONT_FAMILY $ffontsize]
    set BFEAT_FONT [fetch-font $GR_FONT_FAMILY $ffontsize bold]
    set rfontsize [expr int($FHEIGHT_DFLT * $factor)]
    if { $rfontsize < 1 } {set rfontsize 1}
    set REALISATION_FONT [fetch-font "Times"  $rfontsize italic]

    set EC_HEIGHT [expr int(18 * $factor)]
    set EC_FONT [fetch-font $GR_FONT_FAMILY $EC_HEIGHT]

    set SHEIGHT [font-height $SYS_FONT]
    set FHEIGHT [font-height $FEAT_FONT]
    set RHEIGHT [font-height $REALISATION_FONT]

    set XINCR [expr int($XINCR_DFLT * $factor)]
    if { $XINCR < 1 } {set XINCR 1}

    set YINCR [expr int($YINCR_DFLT * $factor)]
    if { $YINCR < 1 } {set YINCR 1}
}


##############################
## SAVE PDX

proc savepdx {canv} {
    set default [scheme-default-file pdx]
    set initialfile [file tail $default]
    set initialdir  [file dirname $default]
    set file [tk_getSaveFile -title "Save PDX file to"\
		-initialfile $initialfile -initialdir $initialdir ]
    if { $file != ""} {
	canvas-to-pdx $canv $file
    }
}

##############################
## SAVE POSTSCRIPT

proc saveps {canv} {
    global  DIR
    set default [scheme-default-file eps]
    set initialfile [file tail $default]
    set initialdir  [file dirname $default]
    set file [tk_getSaveFile -initialfile $initialfile -initialdir $initialdir]
    if { $file != {} } {
	$canv postscript -file $file
    }
}

