###################################################
# 1. DEFINE WINDOW ELEMENTS
###################################################

proc make-scheme-frame {} {
    global PLATFORM GCANV
    if [winfo exists .scheme] {destroy .scheme}
    frame .scheme
    make-scheme-toolbar
    make-graph-canvas
    set-zoom 0
    pack-graphframe
    pack .scheme.rtbar -side left  -anchor nw 
    set-zoom 100
    return .scheme
}

proc pack-graphframe {} {
    if [winfo exists .scheme.treeframe] {destroy .scheme.treeframe}
    pack .scheme.graphframe -side right -fill both -expand true -anchor n -padx 2
}

proc unpack-graphframe {} {
    pack forget .scheme.graphframe
}




####################################################		   
# Define The GRAPH Frame


proc make-graph-canvas {} {
    global GCANV

    frame .scheme.graphframe -height 10 -width 10  
    
    set GCANV [GraphPane .scheme.graphframe.canvas -height 999\
	    -yscrollcommand ".scheme.graphframe.yscroll set"\
	    -xscrollcommand ".scheme.graphframe.xscroll set"]
    scrollbar .scheme.graphframe.yscroll -orient vertical\
	    -command ".scheme.graphframe.canvas yview"
    scrollbar .scheme.graphframe.xscroll -orient horizontal\
	    -command ".scheme.graphframe.canvas xview"
   

    pack .scheme.graphframe.yscroll -side right -fill y
    pack .scheme.graphframe.xscroll -side bottom -fill x 
    pack $GCANV -fill both -expand true -side left
    
    return $GCANV
}

proc make-scheme-toolbar {} {
    global GRAPH_DEPTH TEMPROOT DEPTH_OPTIONS GRAPH_VERSION GRAPH_REGISTER GRAPH_TYPES
    frame .scheme.rtbar
    set GRAPH_DEPTH all

    
    label .scheme.rtbar.versionlab  -text "Layout Version" -fg red
    if { ![info exists GRAPH_VERSION] ||  ![info exists GRAPH_REGISTER($GRAPH_VERSION)] } {
	set GRAPH_VERSION [car $GRAPH_TYPES]
    }

    button .scheme.rtbar.version -textvariable GRAPH_VERSION 
    bind .scheme.rtbar.version <ButtonRelease-1> {
	global  GRAPH_VERSION GRAPH_TYPES
	set result [popup-choose-from-list $GRAPH_TYPES %x %y]
	update
	switch -- $result {
	    cancelled {}
	    default {
		set GRAPH_VERSION $result
		graph-scheme 
	    }
	}
    }
    
    add-rollover-message .scheme.rtbar.version\
	    "Select the graph layout manager to use."
    

    # Define The ACTION toolbar
    label .scheme.rtbar.actiontext -text "Actions" -fg red  
    Toolbar .scheme.rtbar.action
    
    # ToolbarItem .scheme.rtbar.action "Zoom In" "Increases Graph size" zoom-in
    # ToolbarItem .scheme.rtbar.action "Zoom Out" "Decreases Graph size" zoom-out
    ToolbarItem .scheme.rtbar.action "ReDraw" "Redraws the graph" graph-scheme 
    
    
    ToolbarItem .scheme.rtbar.action "Print Scheme" "Prints the scheme (only for some systems)" {
	global GCANV
	myPrintCanvas $GCANV
    }

    ToolbarItem .scheme.rtbar.action "Find Feature" "Locates a feature in the graph" {
	find-feature
    }

    ToolbarItem .scheme.rtbar.action "Find System" "Locates a system in the graph" {
	find-system
    }


    
    label .scheme.rtbar.presellabel  -text "StartFeat" -fg red
    # ChooseButton .scheme.rtbar.presel TEMPROOT ALL_ACTIVE_SYSTEMS_SORTED
    button .scheme.rtbar.presel -textvariable TEMPROOT 
      bind .scheme.rtbar.presel <ButtonRelease-1> {
	global ROOT TEMPROOT
	set feats [start-features]
	lappend feats "Other..."
	set result [popup-choose-from-list $feats %x %y]
	switch -- $result {
	    cancelled {}
	    
	    "Other..." {
		set feats [lsort [all-active-features]]
		set result [popup-choose-from-list $feats %x %y]
		set TEMPROOT $result
		graphf $result
	    }
	    
	    default {
		set TEMPROOT $result
		graphf $result
	    }
	}
    }

    add-rollover-message .scheme.rtbar.presel\
	    "The graph will only display systems under the selected feature"
    
    label .scheme.rtbar.depthtext -text "Depth" -fg red  

    button .scheme.rtbar.depthentry -textvariable GRAPH_DEPTH\
	    -command {
	global GRAPH_DEPTH
	set xpos [winfo pointerx .scheme.rtbar.depthentry]
	set ypos [winfo pointery .scheme.rtbar.depthentry]
	set result [popup-choose-from-list {all 1 2 3 4 5 6 7 8 9}  $xpos $ypos]
	if {  $result != "cancelled" } {
	    set GRAPH_DEPTH $result
	    graph-scheme
	}
    }

    add-rollover-message .scheme.rtbar.depthentry\
	    "Specify how many systems from the root are to be displayed (0 for all)"


   label .scheme.rtbar.zoomlabel  -text "Zoom %" -fg red
    button .scheme.rtbar.zoom -textvariable ZOOM
    bind .scheme.rtbar.zoom <ButtonRelease-1> {
	  set choices {10 20 30 40 50 60 70 80 90 100 120 150 200}
	  set result [popup-choose-from-list $choices %x %y]
	  switch -- $result {
	    cancelled {}
	    default {
		set-zoom $result
		graph-scheme
	    }
	}
    }


    add-rollover-message .scheme.rtbar.zoom\
	    "Changes the size of the graph"
    pack .scheme.rtbar.zoomlabel .scheme.rtbar.zoom  -side top -pady 0.1c

    pack .scheme.rtbar.versionlab .scheme.rtbar.version\
	    .scheme.rtbar.actiontext .scheme.rtbar.action \
	    .scheme.rtbar.presellabel .scheme.rtbar.presel\
	    .scheme.rtbar.depthtext .scheme.rtbar.depthentry\
	    -side top -pady 0.1c
}



proc add-scheme-menu {path} {
    global TOOL GCANV PLATFORM HIDE_SYSTEM_NAMES
    Menu $path Scheme 
    MenuItem $path Scheme "Load Scheme" {load-scheme;graph-scheme}
    MenuItem $path Scheme "Reload Scheme" {
	global GRAMMAR_FILE; 
	load-scheme $GRAMMAR_FILE
	graph-scheme
    }
    if {$TOOL == "coder" } {
	MenuItem $path Scheme "Copy Scheme From..." {copy-scheme-from}
    }
    MenuItem $path Scheme "Save Scheme" {save-scheme}
    MenuItem $path Scheme "Save Scheme As..." {save-scheme-as}
    MenuSeparator $path Scheme 
    MenuItem $path Scheme "Print Scheme" {global GCANV; myPrintCanvas $GCANV}
    MenuItem $path Scheme "Save Diagram As..." {global GCANV; mySaveCanvas $GCANV}  
    
    if {$PLATFORM == "windows"} {
	MenuItem $path Scheme "Capture Scheme to Clipboard"\
		{global GCANV; myCaptureCanvas $GCANV}
    }

    MenuCheck .menuBar Scheme  "Hide System Names" HIDE_SYSTEM_NAMES  {graph-scheme}  
}


proc save-scheme-diag-as {} {
    global GCANV
    set file [tk_getSaveFile -title "Save Scheme"\
	    -filetypes { {"Windows MetaFile" {.wmf}} {"Windows Enhanced Metafile" {.emf}}\
	    {"Encapsulated Postscript" {.eps}}  {"PDX file" {.pdx}}}]
    if { $file == {}} {return 0}
    
    mySaveCanvas $cnv $file
}


proc scheme-toggle-button {mode dir} {
  .scheme.rtbar.action.$mode configure -relief $dir
}


proc toggle-graph-version {} {
    global GRAPH_VERSION VERSION_BUTTON
    switch -- [$VERSION_BUTTON cget -text] {
	"Version 2" {
	    set GRAPH_VERSION 1
	    $VERSION_BUTTON config -text "Version 3"
	}
	"Version 1" {
	    set GRAPH_VERSION 0
	    $VERSION_BUTTON config -text "Version 2"
	}

	"Version 3" {
	    set GRAPH_VERSION 2
	    $VERSION_BUTTON config -text "Version 1"
	}
    }
    graph-scheme
}


proc start-features {} {
    global ROOT FEATURE SYSTEM
    set feats {}
    foreach system $FEATURE($ROOT,extenders) {
	set feats2 $SYSTEM($system,features)
	set feats [concat $feats $feats2]
	foreach feat $feats2 {
	    foreach system1 $FEATURE($feat,extenders) {
		set feats [concat $feats $SYSTEM($system1,features)]
	    }
	}
    }
    return [concat $ROOT [lsort [delete-duplicates $feats]]]
}

proc find-feature {} {
    global GCANV TEMPROOT
    
    set feature [ask-user-for-string  "Feature name:"]
    if {$feature == 0} {return}

    if {[string first "*" $feature] > -1} {
	set feature [wildcard-match $feature feature $TEMPROOT]
	if { $feature == "cancelled"} {return}
    }
    
    goto-feature $feature
}

proc find-system {} {
    global GCANV TEMPROOT
    
    set system [ask-user-for-string  "System name:"]
    if {$system == 0} {return}

    if {[string first "*" $system] > -1} {
	set system [wildcard-match $system system $TEMPROOT]
	if { $system == "cancelled"} {return}
    }
    
    goto-system $system 
}

proc goto-system {system} {
    global GCANV
    hypershow-feature $system $GCANV system
}

proc goto-feature {feature} {
    global GCANV
    hypershow-feature $feature $GCANV feature
}



