##################################################
# NEGATION.tcl

# Code dealing with negation of features and LEs



proc negate-le {le} {
    global FEATURE SYSTEM ROOT

    switch -- [ec-type $le] {

	simple {
	    set result [negate-feature $le]
	    return $result
	}


	and {
	    # translates to {or {not le1} {not le2} ...}
	    set result {}
	    foreach le1 [cdr $le] {
		lappend result [negate-le $le1]
	    }
	    return [disjoin-many-fds $result]
	}

	or {

	    # Translates to: {and {not le1} {not le2} ...}
	    set disjuncts [cdr $le]

	    # Ensure each disjunct has a result (if not, fail)
	    set results {}
	    foreach le1 $disjuncts {
		set result [negate-le $le1]
		if { $result == {} } {return {}}
		lappend results $result
	    }

	    # Unify the results
	    set result [pop results]
	    foreach fd $results {
		set result [unify-fds $result $fd]
		if { $result == {} } {return {}}
	    }
	    return $result
	}

	not { 
	    # Negation of negation
	    switch -- [llength $le] {
		1 {
		    tell-user "negate-le: operator 'not' with no arguments: $le"
		    error
		}
		
		2 { 
		    return [path [second $le]]
		}
		
		default  {
		    tell-user "negate-le: operator 'not' with more than one argument: $le"
		    error
		}
	    }
	}
    }
}


proc negate-feature {feature} {
    global ROOT FEATURE

    if { $feature == $ROOT} {
	return {}
    }
    set cofeats [co-features $feature]
    if { $cofeats == {} } {
	return [negate-system $FEATURE($feature,system)]
    }
    set path2 [negate-system $FEATURE($feature,system)]
    switch -- [llength $cofeats] {
	0 {return $path2}
	1 {set path1 [path [car $cofeats]]}
	default {set path1 [path [concat "or" $cofeats]]}
    }

    return [disjoin-fds $path1 $path2]
}



proc negate-system {system} {
    global SYSTEM
    if ![info exists SYSTEM($system,negation)] {
	set path [negate-le $SYSTEM($system,ec)]
	set SYSTEM($system,negation) [simplify-fd $path]
    }
    return $SYSTEM($system,negation)
}
    


proc restrict-feature {feature fd} {
  # Unifies the negation of  the Feature and the FD

    set defs [definite $fd]
    
    # If the feature is definite, fail
    if [member $feature $defs] {return {} }

   # feature is already excluded in the definite - return the fd
    if [intersect-p [co-features $feature] $defs]  { return $fd}

    return [unify-fds [negate-feature $feature] $fd]
}



proc restrict-le {le fd} {
    # Unifies the negation of  the Feature and the FD
    #this function hasn't been optimised for complex objects as assert-le

    if { [ec-type $le] == "simple" } {
	return [restrict-feature $feature $fd]
    }

    return [unify-fds [negate-le $le] $fd]
}

