###################################################
# 1. DEALING WITH SYSTEMS
###################################################

# Change to two arrays
# SYSTEM($snam,<field>)
#   SYSTEM(x,features): the features of the system
#   SYSTEM(x,ec): the ec of the system
#   SYSTEM($system,entry_features) : the features in the ec (flattened if it is complex)
#   SYSTEM(x,gloss): a description of the system
#   SYSTEM(x,state): either active or disabled
#   SYSTEM(x,visible): 1 if visible, 0 if hidden (for graph)


# FEATURE($Fnam,<field>)
#   FEATURE(x,system): the system to which this feature belongs
#   FEATURE(x,extenders): the systems which have this feature as ec
#   FEATURE(x,realisations): a list of realisations of the unit.
#   FEATURE(x,state): either active or disabled

# ROOT             -- the leftmost feature
# SCHEME_MODIFIED  -- 1 if scheme modified since last save/load, 0 otherwise
# EDIT_STACK - list of changes to the resources
# SYSTEM_LIST -- ordered list of the systems


proc clear-systems {} {
    global FEATURE SYSTEM ROOT SYSTEM_LIST SCHEME_MODIFIED EDIT_STACK TEMPROOT TOOL
    
    if { $TOOL == "parser"} {
	global INSERTCOND PRESELS ROLE_REQUIREMENTS
	if [info exists INSERTCOND] {unset INSERTCOND}
	if [info exists PRESELS] {unset PRESELS}
	if [info exists ROLE_REQUIREMENTS] {unset ROLE_REQUIREMENTS}
	clear-paths
    }
    if [info exists FEATURE] {unset FEATURE}
    if [info exists SYSTEM] {unset SYSTEM}
    set ROOT {}
    set TEMPROOT {}
    set SYSTEM_LIST {}
    set SCHEME_MODIFIED 0
    set EDIT_STACK {}
}

proc clear-system {system} {
    global SYSTEM
    unset SYSTEM($system,features)
    unset SYSTEM($system,ec)
    unset SYSTEM($system,entry_features)
    unset SYSTEM($system,state)
    unset SYSTEM($system,gloss)
    unset SYSTEM($system,visible)
}

proc defroot {feature} {
    global ROOT TEMPROOT FEATURE
    set ROOT $feature
    set TEMPROOT $ROOT
    set FEATURE($feature,state) active
}

proc defsystem {system ec features {state active}} {
    global FEATURE SYSTEM
    set system [string toupper $system]
    if [info exists SYSTEM($system,features)] {
	retract-system $system
    }
    
    set SYSTEM($system,features) $features
    set SYSTEM($system,state) $state
    set SYSTEM($system,gloss) ""
    set SYSTEM($system,visible) 1
    
    # deal with the features
    foreach feature $features {
	if ![info exists FEATURE($feature,extenders)] {set FEATURE($feature,extenders) {}}
	# wipe any prior registration if exists
	if [info exists FEATURE($feature,system)] {
	    delete SYSTEM($FEATURE($feature,system),features) $feature
	}
	set FEATURE($feature,system) $system
	set FEATURE($feature,state) active
    }

    # process the ec
    set-system-ec $system $ec
}

proc def-feature {feature realisations {state active}} {
    global FEATURE TOOL
    set FEATURE($feature,realisations) $realisations
    set FEATURE($feature,state) $state
    if { $TOOL == "parser"} {
	register-realisations $feature $realisations
    }
}


proc defrealisation {feature realisations} {
   def-feature $feature $realisations
}


proc defconstraint {feature realisations} {
   # def-feature $feature $realisations
}

proc set-system-ec {system ec} {
    global SYSTEM FEATURE
    set SYSTEM($system,ec) $ec
    set SYSTEM($system,entry_features) [flatten $ec]
    foreach feature $SYSTEM($system,entry_features) {
	if [info exists FEATURE($feature,extenders)] {
	    lappend FEATURE($feature,extenders) $system
	} else {
	    set FEATURE($feature,extenders) "$system"
	}
    }
}

proc retract-system {system} {
  global FEATURE SYSTEM
  foreach feature $SYSTEM($system,features) {
     retract-feature $feature
  }
  foreach feature $SYSTEM($system,entry_features) {
     delete FEATURE($feature,extenders) $system
  }
  clear-system $system
}

proc retract-feature {feature} {
    global  SYSTEM FEATURE 
    set system $FEATURE($feature,system)
    delete SYSTEM($system,features) $feature
    unset FEATURE($feature,system)
    unset FEATURE($feature,state)
}


proc delete-system {system} {
    global  FEATURE SYSTEM 
    # More destructive form of system retraction

    # 1. Delete all outgoing features
    foreach feature $SYSTEM($system,features) {
	delete-feature $feature
    }

    #1. Delete pointers to this system
    foreach feature $SYSTEM($system,entry_features) {
	delete FEATURE($feature,extenders) $system 
    }

    #2. Delete the system definition
    clear-system $system
}

proc delete-feature {feature} {
    global  FEATURE SYSTEM TOOL
    # More destructive form of feature retraction

    #1. For each extender, simplify the EC
    foreach system $FEATURE($feature,extenders) {
	set SYSTEM($system,ec) [remove-from-ec $feature $SYSTEM($system,ec)]
	if { $SYSTEM($system,ec) == {} } {
	    delete-system $system 
	} else {
	    set SYSTEM($system,entry_features) [flatten $SYSTEM($system,ec)]
	}

    }
    unset FEATURE($feature,extenders)
    unset FEATURE($feature,state)
    if [info exists FEATURE($feature,realisations)] {unset FEATURE($feature,realisations)}

    #3. Delete the feature from the System, and visa versa
    set system $FEATURE($feature,system)
    delete SYSTEM($system,features) $feature
    unset FEATURE($feature,system)

    # Application specific processing
    switch -- $TOOL {
	coder {
	    # Delete the feature from the coding
	    global TEXT_NODES NODE
	    foreach coding $TEXT_NODES {
		delete NODE($coding,features) $feature
	    }
	}
    }
}


proc add-feature {feature system {state active}} {
  global FEATURE SYSTEM
  set system [string toupper $system]
  lappend SYSTEM($system,features) $feature
  set FEATURE($feature,system) $system
  if ![info exists FEATURE($feature,extenders)] {
    set FEATURE($feature,extenders) {}
  }
  set FEATURE($feature,state) $state
}



proc rename-feature {feature newfeature} {
    global FEATURE SYSTEM ROOT TEMPROOT  TOOL

    # TEMPROOT is used by the grapher
    if { $feature == $TEMPROOT } {
	set TEMPROOT $newfeature
    }

  # a) delete all old entries
    if { $feature == $ROOT } {
	defroot $newfeature
    } else {
	set system $FEATURE($feature,system)
	set pos [lsearch $SYSTEM($system,features) $feature]
	set SYSTEM($system,features)\
		[lreplace $SYSTEM($system,features) $pos $pos $newfeature]
	
	# Change the current feature's entry
	set FEATURE($newfeature,system) $system
	unset FEATURE($feature,system)
    }
    set FEATURE($newfeature,extenders) $FEATURE($feature,extenders)
    unset FEATURE($feature,extenders)

    set FEATURE($newfeature,state) $FEATURE($feature,state)
    unset FEATURE($feature,state)

    # For each extender, change the entry-cond
    foreach system $FEATURE($newfeature,extenders) {
	set SYSTEM($system,ec) [list-replace $feature $newfeature $SYSTEM($system,ec)]
	set SYSTEM($system,entry_features) [flatten $SYSTEM($system,ec)]
    }
    

    # Change the realisation entry
    if [info exists  FEATURE($feature,realisations)] {
	set FEATURE($newfeature,realisations) $FEATURE($feature,realisations)
	unset FEATURE($feature,realisations)
    }

    # Do some application-specific processing
    switch -- $TOOL {
	coder {
	    global TEXT_NODES NODE
	    
	    # For each coding, change the feature (ensure current is reloaded)
	    foreach coding $TEXT_NODES {
		substitute NODE($coding,features) $feature $newfeature
	    }
	}
    }
}


proc rename-system {system newsystem} {
    global FEATURE SYSTEM 

    #1. Change the ref from feature to system
    foreach feature $SYSTEM($system,features) {
	set FEATURE($feature,system) $newsystem
    }

    #2. Change all extender references
    foreach feature $SYSTEM($system,entry_features) {
	set pos [lsearch -exact $FEATURE($feature,extenders) $system]
	set FEATURE($feature,extenders)\
		[lreplace $FEATURE($feature,extenders) $pos $pos $newsystem]
    }

    #3 Create the new system's entry
    set SYSTEM($newsystem,features) $SYSTEM($system,features)
    set SYSTEM($newsystem,ec)      $SYSTEM($system,ec)
    set SYSTEM($newsystem,entry_features) $SYSTEM($system,entry_features)
    set SYSTEM($newsystem,state)   $SYSTEM($system,state)
    set SYSTEM($newsystem,gloss)   $SYSTEM($system,gloss)
    set SYSTEM($newsystem,visible) $SYSTEM($system,visible)

    #4. clear the old system
    unset SYSTEM($system,features)
    unset SYSTEM($system,ec)
    unset SYSTEM($system,entry_features)
    unset SYSTEM($system,state)
    unset SYSTEM($system,gloss)
    unset SYSTEM($system,visible)

}

proc defgloss {system gloss} {
  global SYSTEM
  set SYSTEM($system,gloss) $gloss
}

proc entry-condition {system} {
  global SYSTEM
  return $SYSTEM($system,ec)
}

proc fetch-system-features {system} {
  global SYSTEM
  return $SYSTEM($system,features)
}

proc fetch-system-state {system} {
  global SYSTEM
  return $SYSTEM($system,state)
}

proc who-extends {feature} {
  global FEATURE
  return $FEATURE($feature,extenders)
}

proc who-has-as-input {feature} {
  return [who-extends $feature]
}


proc fetch-feature-system {feature} {
  global FEATURE ROOT
  if { $feature == $ROOT } {return {}}
  return $FEATURE($feature,system)
}

proc unique-system-name {} {
    global SYSTEM
    set indx 1
    while {[info exists SYSTEM(SYSTEM$indx,features)] == 1} {
	incr indx
    }
    return SYSTEM$indx
}

proc unique-system-name1 {feature} {
    global SYSTEM
    set sysname [string toupper "$feature-type"]
    if ![info exists SYSTEM($sysname,features)] {return $sysname}
    set indx 2
    while {[info exists SYSTEM($sysname$indx,features)] == 1} {
	incr indx
    }
    return $sysname$indx
}

proc unique-feature-name {} {
  global FEATURE
  set indx 1
  while {[info exists FEATURE(feature$indx,system)] == 1} {
	incr indx
  }
  return feature$indx
}


proc add-system-under {feature} {
    global  FEATURE SYSTEM ROOT TOOL

    set system [unique-system-name1 $feature] 
    if { $feature == $ROOT} {
	set state "active"
    } else {
	set state $SYSTEM($FEATURE($feature,system),state)
    } 
    
    defsystem $system $feature "[unique-feature-name]" $state
    add-feature [unique-feature-name] $system $state

    # Application specific processing
    switch -- $TOOL {
	coder {
	    global TEXT_NODES NODE
	    foreach nid $TEXT_NODES {
		if [member $feature $NODE($nid,features)] {
		    set NODE($nid,completed-p) 0
		}
	    }
	}
    }
}

proc feature-exists-p {feature} {
    global FEATURE ROOT
    if { $feature == $ROOT} {return 1}
    if { [info exists FEATURE($feature,system)] == 1} {return 1}
    return 0
}


proc system-exists-p {system} {
    global SYSTEM
    if { [info exists SYSTEM($system,ec)] == 1} {return 1}
    return 0
}

proc simple-ec-p {ec} {
    if { [lindex $ec 1] == {} } {
	return 1
    }
    return 0
}

proc complex-ec-p {ec} {
    if { [lindex $ec 1] == {} } {
	return 0
    }
    return 1
}

proc system-has-complex-ec {system} {
  global SYSTEM
    if [member [car $SYSTEM($system,ec)] {and or}] {
	return 1
    }
    return 0
}

proc ec-type {ec} {
    if { [lindex $ec 1] == {} } {
	return simple
    }
    set op [lindex $ec 0]
    if ![member $op {and or not fd}] {
	# This is a simple list of features
	return "list"
    }
    return $op
}

proc parse-complex-ec {ec} {
    return $ec
}

proc remove-from-ec {feature ec} {
    if [simple-ec-p $ec] {
	# a symbol
	if {$feature == $ec} {
	    return {}
	} else {
	    return $ec
	}
    } else {
	set newlist [lindex $ec 0]
	foreach itm [cdr $ec] {
	    set new  [remove-from-ec $feature $itm]
	    if { $new != {} } {
		lappend newlist $new
	    }
	}
	switch -- [llength $newlist] {
	    1 {return {}}
	    default {return $newlist}
	}
    }
}


proc move-feature-up {feature} {
    global FEATURE SYSTEM
    set system $FEATURE($feature,system)
    set pos [lsearch $SYSTEM($system,features) $feature]
    if { $pos == 0} {bell;return}
    set prior [lindex $SYSTEM($system,features)  [expr $pos - 1]]
    set SYSTEM($system,features)\
	    [lreplace $SYSTEM($system,features) [expr $pos - 1] $pos $feature $prior]
}

proc move-feature-down {feature} {
    global FEATURE SYSTEM
    set system $FEATURE($feature,system)
    set pos [lsearch $SYSTEM($system,features) $feature]
    if { $pos == [expr [llength $SYSTEM($system,features)] - 1]} {bell;return}
    set next [lindex $SYSTEM($system,features)  [expr $pos + 1]]
    set SYSTEM($system,features)\
	    [lreplace $SYSTEM($system,features)  $pos [expr $pos + 1] $next $feature]
}

proc move-system-up {system} {
    global FEATURE SYSTEM
    foreach feature $SYSTEM($system,entry_features) {
	set pos [lsearch -exact $FEATURE($feature,extenders) $system]
	if { $pos > 0} {
	    set prior [lindex $FEATURE($feature,extenders)  [expr $pos - 1]]
	    set FEATURE($feature,extenders)\
		    [lreplace $FEATURE($feature,extenders) [expr $pos - 1] $pos $system $prior]
	}
    }
}

proc move-system-down {system} {
    global FEATURE SYSTEM
    foreach feature $SYSTEM($system,entry_features) {
	set pos [lsearch $FEATURE($feature,extenders) $system]
	if { $pos < [expr [llength $FEATURE($feature,extenders)] - 1]} {
	    set next [lindex $FEATURE($feature,extenders)  [expr $pos + 1]]
	    set FEATURE($feature,extenders)\
		    [lreplace $FEATURE($feature,extenders) $pos [expr $pos + 1] $next  $system]
	}
    }
}


proc flatten {arg} {

    if { [llength $arg] > 1 } {
	if [member [lindex $arg 0] {and or not}] {
	   set arg [lrange $arg 1 end]
	}
	set result {}
	foreach itm $arg {
	    set result [concat $result [flatten $itm]]
	}
	return $result
    } else {
	return [list $arg]
    }
}

proc all-features {} {
	global SYSTEM ROOT
	set result [list $ROOT]
	foreach system [all-systems] {
	  set result [concat $result $SYSTEM($system,features)]
	}
	return $result
}


proc all-features-under {feature} {
    global SYSTEM
    set features {}
    foreach system [all-systems-under $feature] {
      concat! features $SYSTEM($system,features)
    }
    return $features
}



proc all-features-under-system {system} {
    global SYSTEM FEATURE 
    set features {}
    set systems [list $system]
    set toprocess $SYSTEM($system,features)
    while {$toprocess != {} } {
	set feat [pop toprocess]
        lappend features $feat
	foreach system $FEATURE($feat,extenders) {
	    if ![member $system $systems] {
		lappend systems $system
		set toprocess [concat $SYSTEM($system,features) $toprocess]
	    }
	}
    }
    return $features
}


proc all-active-features {} {
    global SYSTEM ROOT FEATURE
    if ![info exists ROOT] {return {}}
    set result [list $ROOT]
    foreach system [all-active-systems] {
	foreach feature $SYSTEM($system,features) {
	    if { $FEATURE($feature,state) == "active"} {
		lappend result $feature
	    }
	}
    }
    return $result
}

proc all-features-sorted {} {
    return [lsort -ascii [all-features]]
}

proc all-systems {} {
    global ROOT
    return [all-systems-under $ROOT]
}

proc all-active-systems {} {
    global ROOT
    return [all-active-systems-under $ROOT]
}

proc all-systems-sorted {} {
    return [lsort -ascii [all-systems]]
}

proc all-systems-under {feature} {
    global FEATURE SYSTEM
    set systems {}
    set toprocess [list $feature]
    while {$toprocess != {} } {
	set feat [pop toprocess]
	foreach system $FEATURE($feat,extenders) {
	    if ![member $system $systems] {
		lappend systems $system
		set toprocess [concat $SYSTEM($system,features) $toprocess]
	    }
	}
    }
    return $systems
}


proc all-active-systems-under.old {feature} {
    global FEATURE SYSTEM
    set systems {}
    set toprocess [list $feature]
    while {$toprocess != {} } {
	set feat [pop toprocess]
	set newFeats {}
	foreach system $FEATURE($feat,extenders) {
	    if { $SYSTEM($system,state) != "disabled" && ![member $system $systems]} {
		lappend systems $system
		set newFeats [concat $newFeats $SYSTEM($system,features)]
	    }
	}
	set toprocess [concat $newFeats $toprocess]
    }
    return $systems
}

proc all-active-systems-under {feature {systems {}}} {
    global FEATURE SYSTEM 
    foreach system $FEATURE($feature,extenders) {
	if { $SYSTEM($system,state) != "disabled" && [member $system $systems] == 0} {
	    lappend systems $system
	    foreach feat $SYSTEM($system,features) {
		set systems [all-active-systems-under $feat $systems]
	    }
	}   
    }
    return $systems
}


proc change-ec {system newec} {
    global SYSTEM FEATURE TOOL
        
    foreach feature $SYSTEM($system,entry_features) {
	delete FEATURE($feature,extenders) $system 
    }
    
    set-system-ec $system $newec
    

    # Application specific processing
    switch -- $TOOL {
	coder {
	    correct-after-ec-change $system $newec
	}
    }    
}


proc ec-satisfied-p {ec features} {
    # note diff from compat: compatible vs satisfied
    if [simple-ec-p $ec] {
	return [member $ec $features]
    }

    switch --  [lindex $ec 0] {
	and {
	    foreach term [cdr $ec] {
		if ![ec-satisfied-p $term $features] {
		    return 0
		}
	    }
	    return 1
	}

	or {
	    foreach term [cdr $ec] {
		if [ec-satisfied-p $term $features] {
		    return 1
		}
	    }
	    return 0
	}
    }
}

proc compat {le features} {
    global SYSTEM FEATURE
    
    if [list-p $le] {
	switch -- [first $le] {
	    and {
		foreach le1 [cdr $le] {
		    if ![compat $le1 $features] {return 0}
		}
		return 1
	    }
	    or {
		foreach le1 [cdr $l] {
		    if [compat $le1 $features] {return 1}
		}
		return 0
	    }
	}
    } else {
	if [member $le $features] {return 1}
	if [intersect-p [co-features $le] $features] {return 0}
	set system $FEATURE($le,system)
	return [compat $SYSTEM($system,ec) $features]
    }
}

proc co-features {feature} {
    global SYSTEM FEATURE ROOT
    if { $feature == $ROOT} {return {}}
    
    set system $FEATURE($feature,system)
    return [ldelete $SYSTEM($system,features) $feature]
}



proc find-root {} {
    global SYSTEM FEATURE 
    set system [second  [array get FEATURE "*,system"]]
    
    while 1 {
	set feature [first $SYSTEM($system,entry_features)]
	if ![info exists FEATURE($feature,system)] {
	    defroot $feature
	    return $feature
	}
	set system $FEATURE($feature,system)
    }
}


proc ec-depth {ec} {
    if [simple-ec-p $ec] {
	return 0
    }
    set maxdepth 0
    foreach term [cdr $ec] {
	set maxdepth [max $maxdepth [ec-depth $term]]
    }
    
    return [incr maxdepth]
}

proc mark-scheme-modified {} {
    global SCHEME_MODIFIED TOOL ORDERED_FEATURES
    set SCHEME_MODIFIED 1
   
    #  clear the path information
    reset-all-paths
    
    if [info exists ORDERED_FEATURES] {unset ORDERED_FEATURES}

    # recompute the preselection backward index
    compile-out-realisations
    
    # Application specific processing
    switch -- $TOOL {
	coder {
	    MenuItemEnable .menuBar "Codings" "Save Codings"
	}
    }
}


proc mark-scheme-unmodified {} {
    global SCHEME_MODIFIED TOOL
    set SCHEME_MODIFIED 0

    # Application specific processing
    switch -- $TOOL {
	coder {
	    global MODIF_FLAG
	    if { $MODIF_FLAG == 0} {
		MenuItemDisable .menuBar "Codings" "Save Codings"
	    }
	}
    }
}


proc scheme-modified-p  {} {
    global SCHEME_MODIFIED
    if [info exists SCHEME_MODIFIED] {return $SCHEME_MODIFIED}
    return 0
}

proc ignore-systems-from {system} {
    global SYSTEM FEATURE

    #1. Go through subsystems finding which systems to ignore
    ignore-system $system
    
    foreach feature $SYSTEM($system,features) {
	ignore-from-feature $feature
    }
}


proc ignore-from-feature {feature} {
    global SYSTEM FEATURE

    #1. Go through subsystems finding which systems to ignore
    set FEATURE($feature,state) "disabled"
    
    foreach system $FEATURE($feature,extenders) {
	if { $SYSTEM($system,state) == "active" } {
	    if ![enterable-system-p $system] {
		ignore-systems-from $system
	    }
	}
    }
}

proc ignore-system {system} {
    global SYSTEM FEATURE
    set SYSTEM($system,state) "disabled"
    foreach feature $SYSTEM($system,features) {
	set FEATURE($feature,state) "disabled"
    }
}


proc unignore-system {system} {
    global SYSTEM
    set SYSTEM($system,state) "active"
    foreach feature $SYSTEM($system,features) {
	set FEATURE($feature,state) "active"
    }
}

proc unignore-systems-from {system} {
    global SYSTEM FEATURE

    #1. Go through subsystems finding which systems to ignore
    unignore-system $system

    foreach feature $SYSTEM($system,features) {
	unignore-from-feature $feature
    }
}


proc unignore-from-feature {feature} {
    global SYSTEM FEATURE

    #1. Go through subsystems finding which systems to ignore
    set FEATURE($feature,state) "active"
    
    foreach system $FEATURE($feature,extenders) {
	if { $SYSTEM($system,state) == "disabled" } {
	    if [enterable-system-p $system] {
		unignore-systems-from $system
	    }
	}
    }
}

proc ignored-feature-p {feature} {
    global FEATURE
    if [info exists FEATURE($feature,state)] {
	if {$FEATURE($feature,state) == "disabled"} {
	    return 1
	}
    }
    return 0
}

proc enterable-system-p {system} {
    # Returns 1 if at least one feature to the system is currently enabled.
    global SYSTEM FEATURE
    foreach feature $SYSTEM($system,entry_features) {
	if { $FEATURE($feature,state) == "active"} {
	    return 1
	}
    }
    return 0
}




proc change-feature-system {feature system} {
    global SYSTEM FEATURE TOOL
    
    set ec $SYSTEM($system,ec)
    set oldsystem $FEATURE($feature,system)
    delete SYSTEM($oldsystem,features) $feature
    add-feature $feature $system
    if { $SYSTEM($oldsystem,features) == {}} {delete-system $oldsystem}

     # Application specific processing
    switch -- $TOOL {
	coder {
	    coder-change-feature-system $feature $system
	}
    }
}


proc merge-paths {path1 path2} {
    if { $path1 == {} || $path2 == {} } {
	return {}
    }

    # ensure path1 is a list of paths
    if { [car $path1] == "or"} {
	set path1 [cdr $path1]
    } else {
	set path1 [list $path1]
    }

    # ensure path2 is a list of paths
    if { [car $path2] == "or"} {
	set path2 [cdr $path2]
    } else {
	set path2 [list $path2]
    }

    # multiply out paths
    set result {}
    foreach p1 $path1 {
	foreach p2 $path2 {
	    set p [merge-simple-paths $p1 $p2]
	    if { $p != {} } { 
		lappend result $p
	    }
	}
    }

    switch -- [llength $result] {
	0 { return {} }
	1 { return [car $result] }
	default { return [concat "or" $result] }
    }
}

proc merge-simple-paths {p1 p2} {
    
    foreach feature $p2 {

	# if already contained, skip it
	if [member $feature $p1] {
	    continue
	}

	# if incompat, return {}
	foreach feat2 $p1 {
	    if [member $feature [co-features $feat2]] {
		return {}
	    }
	}

	# else add to p1
	set p1 [concat $p1 $feature]
    }
    return $p1
}
    
proc disjunctive-path {disjuncts} {
    set results {}
    foreach disjunct $disjuncts {
	set path [path $disjunct]
	if { $path != {} } {
	    lappend results $path
	}
    }
    
    # find the common over all disjuncts
    set common [lindex $results 0]
    foreach result [cdr $results] {
	set common [intersection $common $result]
    }

}


proc most-delicate-features {features} {
    global SYSTEM FEATURE ROOT
    set fred 0
    set to_delete {}
    foreach feat $features {
	if { $feat != $ROOT} {
	    set ec $SYSTEM($FEATURE($feat,system),entry_features)
	    set new  [intersection $features $ec]
	    set to_delete [concat $to_delete $new]
	}
    }
    return [set-difference $features $to_delete]
}


proc sort-features {features} {
    global FEATURE SYSTEM ROOT
    set feature $ROOT
    set sorted [list $ROOT]
    set waiting_systems $FEATURE($feature,extenders)
    set processed_systems {}
    while { $waiting_systems != {} && $features != {} } {
	set system [pop waiting_systems]
	lappend processed_systems $system
	
	set feature [intersection $SYSTEM($system,features) $features]
	if { $feature != {} } {
	    lappend sorted $feature
	    set features [ldelete $features $feature]
	    foreach system1 $FEATURE($feature,extenders) {
		if ![member $system1 [concat $waiting_systems $processed_systems]] {
		   set waiting_systems [concat $system1 $waiting_systems]
		}
	    }
	}
    }
    return $sorted	
}


proc split-realisations {reals} {
    # Takes in a string-form realisation, and returns converted to a list of reals
    return [split $reals "\n;"]
}


proc convert-reals {realisations} {
    global TOOL
    regsub -all  {\(} $realisations "\{" realisations
    regsub -all  {\)} $realisations "\}" realisations
    set realisations [split $realisations ";\n"]
    if { $TOOL != "parser"} {return $realisations}
    set result {}
    foreach real $realisations {
	regsub -all "  " $real " " real
	set real [string trim $real]
	set pos [string first ":" $real]
	if {$pos == -1 } {
	    # this is a slot-assigment realisation
	    set real [split $real " "]
	    set funct [lindex $real 0]

	    if { [string index $funct end] == "*" } {
		# "end-1" doesnt work in early versions of tcl
		set butlast [expr [string length $funct] -1]
		set funct [string range $funct 0 $butlast]
		set multiple 1
	    } else {
		set multiple 0
	    }

	    set slot [lindex $real 1]
	    if { [llength $real] != 2} {
		tell-user "Badly formed realisation: $real"
	    }
	    
	    lappend result [list insert $funct $slot $multiple]
	} else {
	    # This is a class assigment
	    set funct [string range $real 0 [expr $pos -1]]
	    set classes [string trim [string range $real [expr $pos + 1] end]]
	    if { [llength $real] != 2} {
		tell-user "Badly formed realisation: $real"
	    }
	    lappend result [list type $funct $classes]
	}
    }
    return $result
}




proc compat-sets {set1 set2} {
    if { $set2 == {} } {return 1}
    if { [llength $set1] > [llength $set2]} {
	set set3 $set1
	set set1 $set2
	set set2 $set3
    }
    foreach feat $set1 {
	if [intersect-p [co-features $feat] $set2] {
	    return 0
	}
    }
    return 1
}


proc compile-out-realisations {} {
    global FEATURE PRESEL_LOOKUP
    
    if [info exists PRESEL_LOOKUP] {unset PRESEL_LOOKUP}
    foreach indx [array names FEATURE "*,realisations"] {
	set feature [car [split $indx ","]]
	set reals  [convert-reals $FEATURE($indx)]
	foreach real $reals {
	    switch -- [car $real] {
		type {
		    set funct [second $real]
		    set feats [car [third $real]]
		    foreach feat [flatten $feats] {
			lappend PRESEL_LOOKUP($feat) [list $feature $funct $feats]
		    }
		}
	    }
	}
    }
}

proc repeatable-function-p {funct path} {
    global FINSERTS
    set wc [word-class $path]
    foreach insert $FINSERTS($wc,$funct) {
	if { [third $insert] == 1} {
	    if { [require-le [car $insert] $path] != {} } {
		return 1
	    }
	}
    }
    return 0
}


proc find-all-mentions {feature} {
    global PRESEL_LOOKUP
    if [info exists PRESEL_LOOKUP($feature)] {
	return $PRESEL_LOOKUP($feature)
    }
    return {}
}

proc super-feature-p {feature1 feature2} {
    global TOOL
    set path [feature-path $feature2]
    if {$TOOL == "parser"} {
	set path [path-to-fd $path]
    }
    set features [flatten $path]
    if [member $feature1 $features] {
	return 1
    } 
    return 0
}

proc less-delicate-rule-p {rule1 rule2} {
    return [super-feature-p [car $rule1] [car $rule2]]
}


proc register-realisations {feature realisations} {
    global FEATURE  INSERTCOND FORMALISM ROLE_REQUIREMENTS PRESELS
    
    switch -- $FORMALISM {
	
	dependency {
	    set reals  [convert-reals $realisations]
	    foreach real $reals {
		switch -- [car $real] {
		    type {
			set funct [second $real]
			set feats [car [third $real]]
			
			lappend-new ROLE_REQUIREMENTS($feature) $funct
			lappend PRESELS($feature) [list $funct $feats]
		    }
		    insert {
			set funct [second $real]
			lappend INSERTCOND($funct) $feature
			lappend-new ROLE_REQUIREMENTS($feature) $funct
		    }
		}
	    }
	}
    }
}



proc least-delicate-feature {feats} {
    global ORDERED_FEATURES
    
    if ![info exists ORDERED_FEATURES] {
	build-ordered-features-list
    }
    
    set min 9999
    set ldf {}
    foreach feat $feats {
	set pos [lsearch -exact $ORDERED_FEATURES $feat]
	if { $pos == -1} {
	    puts "WARNING: feature $feat not defined."
	} else {
	    if { $pos < $min } {
		set ldf $feat
		set min $pos
	    }
	}
    }
    return $ldf
}


proc build-ordered-features-list {} {
    global ORDERED_FEATURES FEATURE SYSTEM ROOT

    # Take each feature in the queue in turn, and
    # add its systems features to the end of the queue.
    # This gives a breadth-first list

    set ORDERED_FEATURES {}
    set queue [list $ROOT]
    set seen {}
    while { $queue != {} } {
	set head [pop queue]
	lappend ORDERED_FEATURES $head
	foreach system $FEATURE($head,extenders) {
	    if ![member $system $seen] {
		set queue [concat $queue $SYSTEM($system,features)]
		lappend seen $system
	    }
	}
    }
}

proc ordered-features {{features {}}} {
    global ORDERED_FEATURES
    if ![info exists ORDERED_FEATURES] {
	build-ordered-features-list
    }

    if { $features == {}} {
	return $ORDERED_FEATURES
    }
    return [intersection $ORDERED_FEATURES $features]
}



proc wildcard-match {symb type {start {}}} {
    global FEATURE SYSTEM ROOT

    # Find the relevant options
    switch -- $type {
	feature {
	    set matches {}
	    set options [array names FEATURE "$symb,system"]
	    foreach option $options {
		regsub ",system" $option "" option
		lappend matches $option
	    }
	}
	system {
	    set matches {}
	    set options [array names SYSTEM "$symb,ec"]
	    foreach option $options {
		regsub ",ec" $option "" option
		lappend matches $option
	    }
	}
	default {return}
    }
    

    # if a start-point, eliminate invalids
    if { $matches != {}  && $start != {}} {
	switch -- $type {
	    feature {
		set valid [all-features-under $start]
	    }
	    system {
		set valid [all-active-systems-under $start]
	    }
	}
	set matches [intersection $matches $valid]
    }

    switch --  [llength $matches] {
	0 { tell-user "No $type match for wildcard: $symb"; return "cancelled" }
	1 { return [lindex $matches 0] }
	default	{ 
	    return [popup-choose-from-list $matches 50 50 "Select $type to match: $symb"]
	}
    }
}

proc screen-realisation {real} {
    switch -- [car $real] {
	insert {
	    if { [lindex $real 3] == 1} {
		return "[second $real]* [third $real]"
	    } else {
		return "[second $real] [third $real]"
	    }
	}

	type {
	    return "[second $real]: [third $real]"
	}
    }
}

proc insertion-condition {funct} {
    global INSERTCOND
    if [info exists INSERTCOND($funct)] {
	return $INSERTCOND($funct)
    }
}

  
proc gate-p {system} {
    global FEATURE SYSTEM
    set count 0
    foreach feature $SYSTEM($system,features) {
	if { $FEATURE($feature,state) == "active"} {
	    incr count
	    if {$count == 2} {return 0}
	}
    }
    return 1
}

proc fetch-feature-realisations {feature} {
    global FEATURE
    if [info exists FEATURE($feature,realisations)] {
	return $FEATURE($feature,realisations)
    }
}

proc system-active-features {system} {
    global SYSTEM FEATURE
    set result {}
    foreach feature $SYSTEM($system,features) {
	if {$FEATURE($feature,state) == "active"} {
	    lappend result $feature
	}
    }
    return $result
}
