######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


proc cell-analysis {nodes} {
    global SYSTEM  NODE ROOT cellcount STATSBOX 
    
    #1. Build up the cells
    set systems [all-active-systems]

    if [info exists cellcount] {unset cellcount}
    set cells {}
    foreach nid $nodes {
	set index "I"
	set features $NODE($nid,features) 
	set included {}
	foreach system $systems {
	    set selected [intersection $features [system-active-features $system]]
	    set pos [lsearch -exact [system-active-features $system] $selected]
	    if { $selected != {} } {lappend included $selected}
	    incr pos
	    set index "$index,$pos"
	}
	lappend cellcount($index) $nid
	if ![member $index $cells] {
	    lappend cells $index
	    set cellfeats($index)  $included
	}
    }
    
    #2. Print the forms
    set cells [lsort -command cell-size-greater $cells]
    foreach cell $cells {
	set example [lindex $cellcount($cell) 0]
	$STATSBOX insert end "\nCELL: $cellfeats($cell)"
	$STATSBOX insert end "\nCount: [llength $cellcount($cell)]"
	$STATSBOX insert end "\nMembers: \{$cellcount($cell)\}"
	$STATSBOX insert end "\nExample: $NODE($example,text)\n"
    }
    pad-title
}

proc cell-size-greater {a b} {
    global cellcount
    if { [llength $cellcount($a)] < [llength $cellcount($b)] } { return 1}
    return 0
}


