######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#; COMPARATIVE STATS
#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

# Two sorts of compstats 
# a) Global: Each feature is measured against all codings
# b) Local: Each feature is compared only over its own system


# Eliminate systems totally dependent on one choice from report_systems

###########################################
## COMP STATS
proc comparative-stats {nodes} {
    global  STATS_SYSTEM SYSTEM TITLEBOX FEATURE

    # Ignore ignored features
    set split_features [system-active-features $STATS_SYSTEM]

    $TITLEBOX delete 1.0 end
    $TITLEBOX insert end [format " %-21s |"  {}]\n
    $TITLEBOX insert end [format " %-21s |"  Feature]
    foreach col $split_features {
	# note %s places things on the right
	set col [centre-string $col 21]
	$TITLEBOX insert 1.end [format "%21s|"  $col ]
	$TITLEBOX insert end [format "  %4s %5s %7s |" Mean N TStat]
    }

    set report_systems [ldelete [all-active-systems] $STATS_SYSTEM]
    show-comparison-results [calc-compstats $split_features $report_systems $nodes]
    update-scrollbar .stats.main.table.yscroll

}

proc show-comparison-results {results} {
    global STATSBOX
    set sysformat "%-14s %7i %7s"
    set featform " %5.2f  %4i %5.2f%3s"
    foreach itm  $results {

	# Present with the System Line
	set sysresult [pop itm]
	set sys [car $sysresult]
	if {[string length $sys] > 22} {
	    set sys [string range $sys 0 21]
	}
	set line [format "%-22s |" $sys]
	set border "------------------------"

	foreach col [second $sysresult] {
	    append line [format "       %4i          |" $col]
	    append border "----------------------"
	}
	$STATSBOX insert end $border\n
	$STATSBOX insert end $line\n
    
	# Now the feature lines
	foreach featresult $itm {
	    set feat [car $featresult]
	    if {[string length $feat] > 19} {
		set feat [string range $feat 0 18]
	    }
	    set str "   [format "%-19s" $feat] |"
	    foreach col [lindex $featresult 1] {
		set m1 [expr [lindex $col 0] * 100.0]
		append str "[format " %4.0f%s %4i %5.2f%3s |" $m1 "%" [lindex $col 1] [lindex $col 2] [lindex $col 3]]"
	    }
	    $STATSBOX insert end $str\n
	}
    }
    pad-title
}

 

##########################################################
## Calculate-Compstat-Stats
##########################################################


proc calc-compstats {columns report_systems codings} {
    global NODE SYSTEM COUNT STATS_LOCGLOB STATS_MODE LAST_RESULT
    set colcount [llength columns]

    set active_features [all-active-features]
    foreach col $columns {
	set COUNT($col,N) 0
	 # setup the counts
	foreach feat $active_features {
	    set COUNT($col,$feat) 0
	}
    }
   
    # Do the actual counting
    foreach nid $codings {
	switch -- $STATS_MODE {
	   "Comparative" {
		set features [intersection $NODE($nid,features) $active_features]
		set col [first [intersection $columns $features]]
		if { $col != {} } {
		    incr COUNT($col,N)
		    foreach feat $features {
			if {$feat != $col} {incr COUNT($col,$feat)}
		    }
		}
	    }

	    "Compare Files" {
		global TEXT_NODES COMPARE_NODES
		set features [intersection $NODE($nid,features) $active_features]
		if [member $nid $TEXT_NODES] {
		    set col this_file
		} else {
		    set col other_file
		}
		incr COUNT($col,N)
		foreach feat $features {
		    if [info exists COUNT($col,$feat)] {
			incr COUNT($col,$feat)
		    }
		}
	    }
	}
    }
    
    # Calc the stats

    set Results {}

    foreach system $report_systems {
	
	#i) Discover the total for each set
	set totN 0
	set systemNs {}
	foreach col $columns {
	    if { $STATS_LOCGLOB == "Local" } {
		set systemN($col) 0
		foreach feat [system-active-features $system] {
		    incr systemN($col) $COUNT($col,$feat)
		}
	    } else {
		set systemN($col) $COUNT($col,N)
	    }
	    incr totN $systemN($col)
	    lappend systemNs $systemN($col)
	}
	
	# ii) Save the System result line
	set LocResults [list [list $system $systemNs]]
	# lappend Results [list system $system $systemNs]

      
	#iii) Process each feature in the system
	foreach feature [system-active-features $system] {

	    set totCount 0
	    foreach col $columns {
		incr totCount $COUNT($col,$feature)
	    }
	    set result {}

	    # - Get the counts for each set 
	    foreach col $columns {
		set N1 $systemN($col)
		set N2 [expr $totN - $N1]
		set count1 $COUNT($col,$feature)
		set count2 [expr $totCount - $count1]
		if { $N1 > 0 } {
		    set m1 [div $count1  $N1]
		} else {
		    set m1 0
		}
		if { $N2 > 0 } {
		    set m2 [div $count2  $N2]
		} else {
		    set m2 0
		}
		set stdev1 [stdev $count1 $N1 $m1]
		set stdev2 [stdev $count2 $N2 $m2]
		if { $N1 > 0 && $N2 > 0 } {
		    set stdevcomb [combined-stdev $N1 $stdev1 $N2 $stdev2]
		} else {
		    set stdevcomb 0
		}
		
		if { $stdevcomb != 0 } {
		    set tstat [expr ($m1 - $m2) / $stdevcomb]
		    set tstat [expr abs($tstat)]
		    set signif [signif $tstat [expr $N1 + $N2 - 2]]
		} else {
		    set tstat 0
		    set signif " "
		}
		lappend result [list $m1 $count1 $tstat $signif $stdev1]
	    }
	    # store away the features results
	    lappend LocResults [list $feature $result]
	}
	lappend Results $LocResults
    }
    set LAST_RESULT $Results
    return $Results
}



proc get-counts {cols feature} {
    global COUNT
    set result {}
    foreach col $cols {
	lappend result $COUNT($col,$feature)
    }
    return $result
}

proc get-system-Ns {system cols} {
  # I want to add the total occurence of a system for each set
  # so add each features counts
    global SYSTEM COUNT
    set results
    foreach col $cols {
	set n 0
	foreach feat [system-active-features $system] {
	    incr n $COUNT($col,$feat)
	}
	lappend results $n
    }
    return $results
}


proc stdev {sumx n {m {}}} {
    if { $n == 1} {return 0}

    if { $m == {} } {set m [div $sumx $n]}

    set var [div [expr $sumx * (1 - $m)] [expr $n - 1]]
    return [expr sqrt($var)]
}

proc combined-stdev {n1 sd1 n2 sd2} {
    if { [expr $n1 + $n2] <= 2} {return 0}
    set z1 [expr [div 1 $n1] + [div 1 $n2]]
    set z2 [expr ($n1 - 1) * ($sd1 * $sd1)]
    set z3 [expr ($n2 - 1) * ($sd2 * $sd2)]
    set z4 [expr $z1 * ( ($z2 + $z3) / ($n1 + $n2 - 2))]
    return [expr sqrt($z4)]
}

#(defun stdev (sumx N &optional (M (/ sumx N)))
#  (sqrt (/ (* sumx (- 1 m) )
#             (1- n))))

#(defun combined-stdev ( n1  sd1  n2  sd2)
#  (sqrt (* (+ (/ 1 N1) (/ 1 N2)) 
#           (/ (+ (* (1- N1) (* Sd1 Sd1))
#                 (* (1- N2) (* Sd2 Sd2))) 
#              (+ n1 n2 -2)))))






