######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


# Code to compare another file to this one

proc compare-against-file {current_nodes} {
    global COMPARE_FILE CODINGS_FILE  STATS_FILTER DISPLAY_TYPE  COMPARE_NODES UNUSED_IDS
    
    if { [string trim $COMPARE_FILE] == ""} {
	tell-user "Please select a file to compare to first!"
	return 0
    }

    if ![file exists $COMPARE_FILE] {
	tell-user "The selected file does not exist. Please select a real file."
	return
    }

    set loaded_nodes [load-nodes-for-comparison $COMPARE_FILE]

    set filter [.stats.tbar.filter.text get 1.0 end]
    if { [string trim $filter] != ""} {
	set loaded_nodes [apply-filter $filter $loaded_nodes]
    }

    if { $loaded_nodes == "cancelled"} {return "cancelled"}
    if { $loaded_nodes == {} } {
	tell-user "No codings in loaded file match filter: $STATS_FILTER"
	return
    }

    set fname1 [centre-string [file tail $CODINGS_FILE] 21]
    set fname2 [centre-string [file tail $COMPARE_FILE] 21]
    if { $fname1 == $fname2} {
	set fname1 "Current File"
	set fname2 "Other File"
    }

    if { $DISPLAY_TYPE == "Table"} {
	table-compare-against-file $fname1 $fname2 $current_nodes $loaded_nodes
    } else {
	system-compare-against-file $fname1 $fname2  $current_nodes $loaded_nodes
    }

    # Clear the loaded nodes
    foreach nid $COMPARE_NODES {
	lappend UNUSED_IDS $nid
	clear-node $nid
    }
    unset COMPARE_NODES
}

proc table-compare-against-file {fname1 fname2 current_nodes loaded_nodes} {
    global TITLEBOX
    stats-show-table
    $TITLEBOX delete 1.0 end
    $TITLEBOX insert end [format " %-21s |%21s|%21s"  "File:" $fname1 $fname2]\n
    $TITLEBOX insert end [format " %-21s |"  {}]
    $TITLEBOX insert end [format "  %4s %5s %7s |" Mean N TStat]
    $TITLEBOX insert end [format "  %4s %5s %7s |" Mean N TStat]

    set report_systems [all-active-systems]
    set sysformat "%-14s %7i %7s"
    set featform " %5.2f  %4i %5.2f%3s"
    set results [calc-compstats {this_file other_file}\
	    $report_systems [concat $current_nodes $loaded_nodes]]
    show-comparison-results $results
}


proc system-compare-against-file {fname1 fname2  current_nodes loaded_nodes} {
    global STATS_CURR_SYSINDX COMPARE_SETS
    stats-show-by-system
    set STATS_CURR_SYSINDX 0
    set COMPARE_SETS [list "Current" "Other"]
    calc-compstats {this_file other_file} [all-active-systems]\
	    [concat $current_nodes $loaded_nodes]
    compare-current-system
  
}


proc load-nodes-for-comparison {file} {
    global XMLSEG LASTSEG CURRSEG NODE COMPARE_NODES UNUSED_TAGS LASTTAG

    # Expects a heirarchically structured xml form.
    # Only Segment tags can include free text.
    set COMPARE_NODES {}

    load-xml $file

    set codingsform [parse-xml {system encoding}]

    ################################
    # PARSE the body
    set body [get-xml-element $codingsform {codings body}]
   
    #3. Load in segment defins
    foreach seg [get-xml-elements $body {body segment}] {
	if {[get-xml-element $seg {segment ignore}] != 1} {
	    set id [new-node]
	    set NODE($id,features) [get-xml-element $seg {segment features}]
	    regsub -all {&lt;} $NODE($id,comment) "<" NODE($id,comment)
	    regsub -all {&gt;} $NODE($id,comment) ">" NODE($id,comment)
	    lappend COMPARE_NODES $id
	    set text [lindex $seg 3]
	    regsub -all {&lt;} $text "<" text
	    regsub -all {&gt;} $text ">" text
	    set NODE($id,text) [string trim $text]
	}
    }
    return $COMPARE_NODES
}

