######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#; COMPARATIVE STATS - system by system
#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

# Two sorts of compstats 
# a) Global: Each feature is measured against all codings
# b) Local: Each feature is compared only over its own system


# Eliminate systems totally dependent on one choice from report_systems

# This version precalculates the stats, then runs through them
# result by result

proc comparative-stats-by-system {nodes split_features} {
    global  STATS_SYSTEM STATS_CURR_SYSINDX COMPARE_SETS

    set COMPARE_SETS $split_features

    set report_systems [ldelete [all-active-systems] $STATS_SYSTEM]
    calc-compstats $split_features $report_systems $nodes
    set STATS_CURR_SYSINDX 0
    compare-current-system
}


proc compare-current-system {} {
    global LAST_RESULT STATS_CURR_SYSINDX COMPARE_SETS
 
    set result [lindex $LAST_RESULT $STATS_CURR_SYSINDX]
    if { $result == ""} {
	tell-user "Problem: no result for index: $STATS_CURR_SYSINDX"
	return
    }

    .stats.main.results.text delete 1.0 end

    set systeminfo [pop result]
    set system [pop systeminfo]
    set counts [pop systeminfo]
    set count1 [lindex $counts 0]
    set count2 [lindex $counts 1]

    # Now, print the results
    # Two sets to compare
    set set1name [lindex $COMPARE_SETS 0]
    set set2name [lindex $COMPARE_SETS 1]
    set set1Name [centre-string "[my-trim-string $set1name 10](N=$count1)" 15]
    set set2Name [centre-string "[my-trim-string $set2name 10](N=$count2)" 15]
    compstats-showresult "Comparison for system: $system\n" boldtag
    compstats-showresult [format "%-20s%s\n" ""\
	    [string repeat "-" 50]]
    compstats-showresult [format "%-18s | %-15s | %-15s |  %-11s |\n"\
	    ""   $set1Name $set2Name ""] boldtag
    .stats.main.results.text insert end\
	    [format "%-18s |  Mean  |  Stddv |  Mean  |  Stddv | Tstat  | Lvl |\n"\
	    "Feature"]  boldtag
    compstats-showresult [format "%-20s%s\n" ""\
	    [string repeat "-" 50]]
    
    set sigdiffs {}
    foreach itm $result {
	set feature [car $itm]
	set set1 [car [second $itm]]
	set set2 [second [second $itm]]
	set m1 [car $set1]
	set m2 [car $set2]
	set std1 [lindex $set1 4]
	set std2 [lindex $set2 4]
	set tstat [lindex $set1 2]
	set signif [lindex $set1 3]
	.stats.main.results.text insert end\
		[format "%-18s | %6.3f | %6.3f | %6.3f | %6.3f | %6.3f | %-3s |\n"\
		[my-trim-string $feature 18]\
		$m1 $std1 $m2 $std2 $tstat $signif]

	if { $signif != " "} {
	    if { $m1 > $m2 } {
		lappend sigdiffs [list $feature "higher" $m1 $m2]
	    } else { 
		lappend sigdiffs [list $feature "lower" $m1 $m2]
	    }
	}
    }
    
    compstats-showresult [format "%-20s%s\n" "" [string repeat "-" 50]]
    
    
    # INTERPRETATION
    compstats-showresult "\nInterpretation\n\n" underlinetag
    
    # Insuff. data
    if { $count1 < 5 || $count2 < 5 } {
	compstats-showresult "There is insufficient data to make any interpretation."
	return
    }
    
    # No signif differences
    if { $sigdiffs == {} } {
	compstats-showresult "There are no significant differences between these sets."
	return
    }
    
    # Signif differences
    foreach sigdiff $sigdiffs {
	set feature [pop sigdiff]
	set h_or_l [pop sigdiff]
	set m1 [pop sigdiff]
	set m2 [pop sigdiff]
	
	compstats-showresult "* Set1 ($set1name) exhibits significantly $h_or_l use\n"
	compstats-showresult "  of the feature: "
	compstats-showresult "$feature" italictag
	set m1 [expr 100.00 * $m1]
	set m1 [format "%5.2f%%" $m1]
	set m2 [expr 100.00 * $m2]
	set m2 [format "%5.2f%%" $m2]
	
	compstats-showresult " ($m1 vs. $m2).\n\n"
    }

    if { [llength $sigdiffs] < [llength $result] } {
	compstats-showresult "The use of the remaining features are not significantly different."
    }
}


proc compstats-showresult {text {tag {}}} {
    .stats.main.results.text insert end  $text $tag
}


proc stats-next-system {} {
    global STATS_CURR_SYSINDX LAST_RESULT
    if { $STATS_CURR_SYSINDX == [expr [llength $LAST_RESULT] -1] } {
	bell
    } else {
	incr STATS_CURR_SYSINDX 
	compare-current-system
    } 
}

proc stats-prev-system {} {
    global STATS_CURR_SYSINDX 
    if { $STATS_CURR_SYSINDX == 0 } {
	bell
    } else {
	incr STATS_CURR_SYSINDX -1
	compare-current-system
    } 
}

