######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


# Code for working out descriptive stats

proc descriptive-stats {nodes} {
    global SYSTEM STATS_LABEL STATS_LOCGLOB STATSBOX

    # FUNCTION for presenting the descriptive statistics

    set results [calc-descriptive-stats $nodes]
   
    set form " %-14s %-19s %7i %7s"
    foreach result $results {
	set system [car $result]
	if {[string length $system] > 14} {
	    set system [string range $system 0 13]
	} 
	foreach fresult [cdr $result] {
	    set feature [car $fresult]
	    set fcount [second $fresult]
	    set mean [lindex $fresult 2]
	    if {$mean != "--"} {
		set mean [format "%7.1f%%" $mean]
	    }

	    if {[string length $feature] > 19} {
		set feature [string range $feature 0 18]
	    }
	    $STATSBOX insert end "[format $form $system $feature $fcount $mean]\n"
	    
	    # only show the system on the first feature
	    set system ""
	}    
	$STATSBOX insert end "------------------------------------------------------\n"
    }
    pad-title
}


proc calc-descriptive-stats {nodes} {
    global SYSTEM STATS_LOCGLOB LAST_RESULT LAST_RESULT_TYPE

    set tcount [llength $nodes]

    foreach system [all-active-systems] {
	if { $SYSTEM($system,state) == "disabled"} {continue}
	set sys_results [list $system]
	if { $STATS_LOCGLOB == "Local" } {
	    set scount [count-sys-occurences $system $nodes]
	}	
	
	foreach feature [system-active-features $system] {
	    set fcount [count-occurences $feature $nodes]

	    if { $STATS_LOCGLOB == "Global" } {
		set count $tcount
	    } else {
		set count $scount
	    }
	    if {$count == 0} {
		set mean "--"
	    } else {
		set mean [expr 100.00 * ($fcount + 0.00) / $count]
	    }
	    
	    lappend sys_results [list $feature $fcount $mean]
	}
	lappend result $sys_results
    }

    set LAST_RESULT_TYPE "descriptive"
    set LAST_RESULT $result
    return $result
}

proc count-occurences {feature nodes} {
    global NODE 
    set count 0
    foreach node $nodes {
	    if [member $feature $NODE($node,features)] {incr count}
    }
    return $count
}


proc count-sys-occurences {system nodes} {
    global NODE SYSTEM
    set count 0
    set features [system-active-features $system]
    foreach node $nodes {
	if [intersect-p $features $NODE($node,features)] {incr count}
    }
    return $count
}



