######################################
## Systemic Coder
#
#  File: StatsSave.tcl
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Systemic Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.txt
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


proc stats-save-study {} {
    global STATS_IS_CURRENT
    
    # ensure the display is up to date.
    #    if { $STATS_IS_CURRENT == 0 } 
    if { [stats-do-study] == 0} {
	return 0
    }
   

    set file [tk_getSaveFile -title "Save Results as"\
	    -initialfile [default-file "html"] -filetypes { {HTML *.html} {"Plain Text" *.txt}}]

    if { $file == {} } { return }
    
    if [member [file extension $file] {.html .htm}] {
	save-study-as-html $file
    } else {
	save-study-as-text $file
    }
}

proc save-study-as-text {file} {
   global STATS_MODE STATS_FILTER  STATS_LOCGLOB STATS_SYSTEM STATSBOX TITLEBOX 

    set str [open $file w]
    switch -- $STATS_MODE {
	"Descriptive" {
	    puts $str "Descriptive Statistics for file: [default-file]."
	    catch "puts $str \"Date: [date]\""
	    puts $str "Filter: [string trim $STATS_FILTER]"
	    puts $str "Counting: $STATS_LOCGLOB"
	    puts $str "------------------------------------------------------"
	    puts $str [$TITLEBOX get 1.0 end]
	    puts $str [$STATSBOX get 1.0 end] 

	    close $str
	}
	"Compare Files" {
	    global COMPARE_FILE
	    puts $str "Comparison between files:  "
	    puts $str "  File1: [default-file]"
	    puts $str "  File2: $COMPARE_FILE"
	    catch "puts $str \"Date: [date]\""
	    puts $str "Filter: [string trim $STATS_FILTER]"
	    puts $str "Counting: $STATS_LOCGLOB"
	    puts $str "------------------------------------------------------"
	    puts -nonewline $str [$TITLEBOX get 1.0 end]
	    puts $str "[$STATSBOX get 1.0 end]"
	    close $str
	}

	"Cell Analysis" {
	    puts $str "Cell Analysis for file: [default-file]."
	    catch "puts $str \"Date: [date]\""
	    puts $str "Filter: [string trim $STATS_FILTER]"
	    puts $str "------------------------------------------------------"
	    puts $str "[$STATSBOX get 1.0 end]"
	    close $str
	}

   	"Comparative" {
	    puts $str "Comparative Statistics for file: [default-file]."
	    puts $str "Data split on system: $STATS_SYSTEM"
	     catch "puts $str \"Date: [date]\""
	    puts $str "Filter: [string trim $STATS_FILTER]"
	    puts $str "Counting: $STATS_LOCGLOB"
	    puts $str "------------------------------------------------------"
	    puts -nonewline $str [$TITLEBOX get 1.0 end]
	    puts $str "[$STATSBOX get 1.0 end]"
	    close $str
	}
    }
}

proc save-study-as-html {file} {
   global STATS_MODE STATS_FILTER TOOL_VERSION

    set str [open $file w]

    set filter [string trim $STATS_FILTER]
    if { $filter == ""} {
	set filter "none"
    }
    puts $str "<html><title>Coder $TOOL_VERSION Statistical study: $STATS_MODE</title><body BGCOLOR=\"#FFCC66\"><center>"
    switch -- $STATS_MODE {
	"Descriptive" {
	   save-descriptive-study $str $filter
	}
	
	"Comparative" {
	    save-comparative-study $str $filter 
	}

	"Compare Files" {
	    save-comparative-study $str $filter
	}


	"Cell Analysis" {
	    puts $str "Cell Analysis for file: [default-file]."
	    catch "puts $str \"Date: [date]\""
	    puts $str "Filter: [string trim $STATS_FILTER]"
	    puts $str "------------------------------------------------------"
	    puts $str "[$STATSBOX get 1.0 end]"
	}

   	
    }
    puts $str "</body></html>"
    close $str
}

proc save-descriptive-study {str filter} {
    global STATS_LOCGLOB LAST_RESULT
    puts $str "<h2>Descriptive Statistics for file: [file tail [default-file]]</h2></center>"
    catch "puts $str \"<p><b>Date</b>: [date]</p>\""
    puts $str "<p><b>Filter</b>: $filter</p>"
    puts $str "<p><b>Counting</b>: $STATS_LOCGLOB</p>"
    puts $str "<br><center><table BORDER>"
    puts $str "<tr><td align=\"center\"><b>System</b><td align=\"center\"><b>Feature</b><td align=\"center\"><b>Count</b><td align=\"center\"><b>Mean</b></tr>"
    foreach result $LAST_RESULT {
	set system [car $result]
	foreach fresult [cdr $result] {
	    set feature [car $fresult]
	    set fcount [second $fresult]
	    set mean [lindex $fresult 2]
	    if {$mean != "--"} {
		set mean [format "%7.1f%%" $mean]
	    }
	    puts $str "<tr><td>$system<td>$feature<td align=\"right\">$fcount<td align=\"right\">$mean</tr>"
	    # only show the system on the first feature
	    set system ""
	}    
    }
    puts $str "</table></center>"
}

proc save-comparative-study {str filter} {
    global STATS_MODE STATS_SYSTEM STATS_LOCGLOB LAST_RESULT SYSTEM COMPARE_FILE DISPLAY_TYPE


    switch -- $STATS_MODE {
	"Compare Files" {
	    set file1 [file tail [default-file]]
	    set file2 [file tail $COMPARE_FILE]
	    set splitFeatures [list $file1 $file2]
	    puts $str "<h2>Comparison between files: $file1 and $file2</h2>"
	}

	"Comparative" {
	    set splitFeatures [system-active-features $STATS_SYSTEM]
	    puts $str "<h2>Comparative Statistics for file: [file tail [default-file]]</h2>"
	    puts $str "<p><b>Data split on system</b>: $STATS_SYSTEM"
	}
    }

    puts $str "</center>"
    catch "puts $str \"<b>Date</b>: [date]<br>\""
    puts $str "<b>Filter</b>: $filter<br>"
    puts $str "<b>Counting</b>: $STATS_LOCGLOB<p>"

    if { $DISPLAY_TYPE == "Table"} {
	save-comparative-table-study $str $splitFeatures
    } else {
	save-comparative-by-system-study $str $splitFeatures
    }
}

proc save-comparative-table-study {str splitFeatures} {
    global LAST_RESULT
    puts $str "<br><center><table BORDER=\"1\" bgcolor=\"white\">"

    # Put the Table Header
    puts $str "<tr><td align=\"center\">&nbsp;"
    foreach feature $splitFeatures {
	puts $str "<td align=\"center\" colspan=4><b>$feature</b>"
    }
    puts $str "</tr>"
    # The second row of the header
    puts $str "<tr><td align=\"center\"><b>Feature</b>"
    foreach feature $splitFeatures {
	puts $str "<td align=\"center\"><b>Mean</b><td align=\"center\"><b>N</b><td align=\"center\"><b>TStat</b><td align=\"center\">&nbsp;"
    }
     puts $str "</tr>"

    # Now the results
    foreach itm $LAST_RESULT {
	
	# Present with the System Line
	set sysresult [pop itm]
	set sys [car $sysresult]
	puts  $str "<tr><td>$sys"
	foreach col [second $sysresult] {
	    puts $str "<td colspan=4 align=\"center\">$col"
	}
	puts $str "</tr>"
	
	# Now the feature lines
	foreach featresult $itm {
	    set feat [car $featresult]
	    puts $str "<tr><td align=\"right\">$feat"
	    foreach col [lindex $featresult 1] {
		set m1 [expr [lindex $col 0] * 100.0]
		set m1 [format "%7.1f%%" $m1]
		set n [lindex $col 1]
		set tstat [format "%7.2f" [lindex $col 2]]
		set signif [lindex $col 3]
		if { [string trim $signif] == {} } {
		    set signif "&nbsp;"
		}
		puts $str "<td align=\"right\">$m1<td align=\"right\">$n<td align=\"right\">$tstat<td align=\"center\">$signif"
	    }
	    puts $str "</tr>"
	}
    }
    
    puts $str "</table></center>"
}



proc save-comparative-by-system-study {str splitFeatures} {
    global LAST_RESULT  COMPARE_SETS

    foreach result $LAST_RESULT {
	set systeminfo [pop result]
	set system [pop systeminfo]
	set counts [pop systeminfo]
	set count1 [lindex $counts 0]
	set count2 [lindex $counts 1]
	
	# Now, print the results
	# Two sets to compare
	set set1name [lindex $COMPARE_SETS 0]
	set set2name [lindex $COMPARE_SETS 1]
	set set1Name [centre-string "[my-trim-string $set1name 10](N=$count1)" 15]
	set set2Name [centre-string "[my-trim-string $set2name 10](N=$count2)" 15]
	puts $str "<hr><h2>System: <i>$system</i></h2>"
	puts $str "<table border=\"1\" align=center bgcolor=\"white\">"

	puts $str "<tr align=center><td><b>Feature</b><td colspan=2><b>$set1Name</b><td colspan=2><b>$set2Name</b><td colspan=2>&nbsp;</tr>"

	puts $str "<tr align=center><td>&nbsp;<td><b>Mean</b><td><b>Stddv</b><td><b>Mean</b><td><b>Stddv</b><td><b>Tstat</b><td><b>Lvl</b></tr>"

	set sigdiffs {}
	foreach itm $result {
	    set feature [car $itm]
	    set set1 [car [second $itm]]
	    set set2 [second [second $itm]]
	    set m1 [format "%6.3f" [car $set1]]
	    set m2 [format "%6.3f" [car $set2]]
	    set std1 [format "%6.3f" [lindex $set1 4]]
	    set std2 [format "%6.3f" [lindex $set2 4]]
	    set tstat [format "%6.3f" [lindex $set1 2]]
	    set signif [lindex $set1 3]

	    if { [string trim $signif] == {} } {
		set signif "&nbsp;"
	    }

	    set feat [my-trim-string $feature 18]
	    puts $str "<tr align=center><td>$feat<td>$m1<td>$std1<td>$m2<td>$std2<td>$tstat<td>$signif</tr>"
	    # [format "%-18s | %6.3f | %6.3f | %6.3f | %6.3f | %6.3f | %-3s |\n"

	    
	    if { $signif != "&nbsp;"} {
		if { $m1 > $m2 } {
		    lappend sigdiffs [list $feature "higher" $m1 $m2]
		} else { 
		    lappend sigdiffs [list $feature "lower" $m1 $m2]
		}
	    }
	}
	puts $str "</table>"
	
	
	
	# INTERPRETATION
	puts $str "<p><h3>Interpretation</h3>"
	
	# Insuff. data
	if { $count1 < 5 || $count2 < 5 } {
	    puts $str "There is insufficient data to make any interpretation."
	    continue
	}
	
	# No signif differences
	if { $sigdiffs == {} } {
	    puts $str "There are no significant differences between these sets."
	    continue
	}
	
	# Signif differences
	puts $str "<ul>"
	foreach sigdiff $sigdiffs {
	    set feature [pop sigdiff]
	    set h_or_l [pop sigdiff]
	    set m1 [pop sigdiff]
	    set m2 [pop sigdiff]
	    set m1 [expr 100.00 * $m1]
	    set m1 [format "%5.2f%%" $m1]
	    set m2 [expr 100.00 * $m2]
	    set m2 [format "%5.2f%%" $m2]

	    if { $h_or_l == "higher"} {
		puts $str "<li> Set1 ($set1name) exhibits significantly higher use"
		puts $str "of the feature: <i>$feature</i> ($m1 vs. $m2)."
	    } else {
		puts $str "<li> Set2 ($set2name) exhibits significantly higher use"
		puts $str "of the feature: <i>$feature</i> ($m2 vs. $m1)."
	    }
	}
	puts $str "</ul>"
	if { [llength $sigdiffs] < [llength $result] } {
	    puts $str "<p>The use of the remaining features are not significantly different."
	}
    }
}


# If a binary system, and counts add up to 100, then say
# Set 1 has a higher x while set 2 ..."


