######################################
## Systemic Coder
#
#  Author: Mick O'Donnell micko@wagsoft.com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/Copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.


#########################################
# Stats.tcl

proc make-stats-frame {} {
    global STATS_FILTER STATS_LABEL STATS_MODE
    if [winfo exists .stats] {destroy .stats}
    frame .stats
    set STATS_MODE ""

    make-stats-tbar
    pack .stats.tbar -side top  -anchor nw -padx 0.5c
    
    ##################
    ## The CONTENT Frame
    frame .stats.main
    make-display-space
    
    #    bind  .stats.main.table.choices <ButtonRelease> {stats-set-current %x %y}
    pack .stats.main.table -side top -expand 1 -fill both
    pack .stats.main -expand 1 -fill both -pady 0.2c -padx 0.5c
    install-stats-mode descriptive
    return .stats
}

proc make-display-space {} {
    global STATSBOX TITLEBOX
    set name .stats.main.table
    frame $name 
    frame $name.leftframe 
   
    set TITLEBOX $name.title
    text $TITLEBOX -relief sunken -font "Courier 12" -height 1 -wrap none\
	    -xscrollcommand "$name.leftframe.xscroll set" -bg white
    set STATSBOX $name.leftframe.choices
    text $STATSBOX -height 40\
	    -bg white -relief sunken -font "Courier 12" -wrap none\
	    -xscrollcommand "$name.leftframe.xscroll set"\
	    -yscrollcommand "$name.yscroll set"
    scrollbar $name.yscroll -command "$STATSBOX yview"
    scrollbar $name.leftframe.xscroll -orient horizontal -command stats-double-scroll
	
    pack $name.yscroll -side right -fill y  -expand 1  -anchor e 
    pack $TITLEBOX -side top -fill x  -expand 1  -anchor n 
    pack $name.leftframe.xscroll -fill x -expand 1  -side bottom
    pack $STATSBOX -fill both -expand 1 -side top

    pack $name.leftframe -fill both -expand 1 -side left
}

proc stats-double-scroll {args} {
    eval ".stats.main.table.leftframe.choices xview" $args
    eval ".stats.main.table.title xview" $args
}

proc pad-title {} {
    global TITLEBOX STATSBOX
    # Adds spaces to the title to make it as long
    # as the longest line of text (to make the scroll function properly)
    set titletext [$TITLEBOX get 1.0 end]
    set titlelength 0
    foreach line [split $titletext "\n"] {
	if { [string length $line] > $titlelength} {
	    set titlelength [string length $line]
	}
    }

    set statstext [$STATSBOX get 1.0 end]
    set maxwidth 0
    foreach line [split $statstext "\n"] {
	if { [string length $line] > $maxwidth} {
	    set maxwidth  [string length $line]
	}
    }
    if { $maxwidth > $titlelength} {
	set addition [string repeat " " [expr $maxwidth - $titlelength]]
	$TITLEBOX insert end $addition
    }
}


proc install-stats {} {
    global STATS_MODE
    switch -- $STATS_MODE {
	"Comparative" {
	    stats-show-syschoose
	}
	"Compare Files" {
	    stats-show-filechoose
	}
    }
}

proc uninstall-stats {} {
}

proc make-stats-tbar {} {

    # Define a toolbar
    if {[winfo exists .stats.tbar]} {destroy .stats.tbar}
    
    frame .stats.tbar -relief sunken

    frame .stats.tbar.controls
    button  .stats.tbar.controls.apply -text "Go!" -command {stats-do-study} -height 2
    add-rollover-message .stats.tbar.controls.apply\
       "Show the results of the study"

    button  .stats.tbar.controls.save -text "Save Results" -command {stats-save-study} -height 1
    add-rollover-message .stats.tbar.controls.save\
       "Save the study as html or plain text"
    pack .stats.tbar.controls.apply .stats.tbar.controls.save  -side top
    pack  .stats.tbar.controls -side right -expand 1 -fill y

    frame .stats.tbar.top

    frame .stats.tbar.top.type
    label .stats.tbar.top.type.label -text "Type:"


    tk_optionMenu .stats.tbar.top.type.choice STATS_MODE "Descriptive" "Comparative" "Compare Files" "Cell Analysis" 
    .stats.tbar.top.type.choice.menu entryconfigure 0 -command {install-stats-mode descriptive}
    .stats.tbar.top.type.choice.menu entryconfigure 1 -command {install-stats-mode comparative}
    .stats.tbar.top.type.choice.menu entryconfigure 2 -command {install-stats-mode compare-files}
    .stats.tbar.top.type.choice.menu entryconfigure 3 -command {install-stats-mode cell-analysis}
     add-rollover-message .stats.tbar.top.type.choice\
       "Choose the type of study you want done"


    help-button .stats.tbar.top.type.help stats_type
    pack  .stats.tbar.top.type.label .stats.tbar.top.type.choice .stats.tbar.top.type.help -side left

    frame .stats.tbar.locglob
    label .stats.tbar.locglob.label -text "Counting:"
    tk_optionMenu .stats.tbar.locglob.choice STATS_LOCGLOB  "Global" "Local"    
    help-button .stats.tbar.locglob.help stats_locglob
    pack  .stats.tbar.locglob.label .stats.tbar.locglob.choice\
	    .stats.tbar.locglob.help -side left
    add-rollover-message .stats.tbar.locglob.choice\
       "Use local or global counting (see manual)"

    # The filter
    frame .stats.tbar.filter
    label .stats.tbar.filter.label -text "Only use codings which match:"
    text .stats.tbar.filter.text -relief sunken -bg white -height 1
    bind .stats.tbar.filter.text <Return> {stats-do-study;break} 
    add-rollover-message .stats.tbar.filter.text\
       "Enter here a boolean expression to limit which codings are included in the study"
    help-button .stats.tbar.filter.help stats_filter

    pack .stats.tbar.filter.label -side left
    pack .stats.tbar.filter.text -side left -expand 1 -fill x
    pack .stats.tbar.filter.help -side left
    pack .stats.tbar.top.type -side left
    pack .stats.tbar.top .stats.tbar.locglob -side top -expand 1 -fill x -padx 0.1c
    pack .stats.tbar.filter  -side top -expand 1 -fill x -padx 0.1c
}


proc install-stats-mode {mode} {
    global TITLEBOX STATS_MODE DISPLAY_TYPE

    clear-stats    
    $TITLEBOX delete 1.0 end
    switch -- $mode {
	descriptive {
	    set STATS_MODE "Descriptive"
	    $TITLEBOX config -height 1
	    $TITLEBOX insert end [format " %-14s %-19s %7s %7s"  System Feature N Mean]
	    stats-show-locglob
	    stats-hide-syschoose
	    stats-hide-filechoose
	    stats-hide-table-or-system
	    stats-show-table
	}
	comparative {
	    set STATS_MODE "Comparative"
	    $TITLEBOX config -height 2
	    stats-show-locglob
	    stats-show-syschoose
	    stats-hide-filechoose
	    stats-show-table-or-system
	    if { $DISPLAY_TYPE == "Table"} {
		stats-show-table
	    } else {
		stats-show-by-system 
	    }
	}

	cell-analysis {
	    set STATS_MODE "Cell Analysis"
	    $TITLEBOX config -height 0
	    stats-hide-locglob
	    stats-hide-syschoose
	    stats-hide-filechoose
	    stats-hide-table-or-system
	    stats-show-table
	}

	compare-files {
	    set STATS_MODE "Compare Files"
	    global TITLEBOX
	    $TITLEBOX config -height 2
	    stats-show-locglob
	    stats-hide-syschoose
	    stats-show-filechoose
	    stats-show-table-or-system
	    if { $DISPLAY_TYPE == "Table"} {
		stats-show-table
	    } else {
		stats-show-by-system
	    }
	}
    }
}

proc clear-stats {} {
    global STATSBOX
    $STATSBOX delete 1.0 end
}

proc stats-do-study {} {
    global STATS_MODE TEXT_NODES STATS_FILTER DISPLAY_TYPE
    clear-stats
    set selected [stats-filtered-nodes $TEXT_NODES]
    if { $selected == "cancelled"} {return 0}
    if { $selected == {} } {
	tell-user "No codings match filter: $STATS_FILTER"
	return 0
    }

    switch -- $STATS_MODE {
	"Descriptive" {
	     return [descriptive-stats $selected]
	}
	"Comparative" {
	    global SYSTEM STATS_SYSTEM
	    set splitFeats [system-active-features $STATS_SYSTEM]
	    # 1. Ensure the system has suficient features
	    switch -- [llength $splitFeats] {
		0 { 
		    tell-user "That system has no features"
		    return 0
		}
		
		1 {
		    tell-user "That system has only 1 feature. Cannot split data to compare."
		    return 0
		}

		2 {}
		
		default {
		     if { $DISPLAY_TYPE != "Table"} {
			 tell-user "At present, system-by-system results only available for binary systems."
			 return 0
		     }
		 }
	    }

	    
	    if { $DISPLAY_TYPE == "Table"} {
		stats-show-table
		return [comparative-stats $selected]
	    } else {
		stats-show-by-system
		 return [comparative-stats-by-system $selected $splitFeats]
	    }
	}

	"Cell Analysis" {
	    return [cell-analysis $selected]
	}

	"Compare Files" {
	    return [compare-against-file $selected]
	}
    }
}

proc stats-show-by-system {} {

    
    stats-show-locglob
    pack forget .stats.main.table
    if [winfo exists .stats.main.results] {destroy .stats.main.results}
    frame .stats.main.results
    
    frame .stats.main.results.buttons
    button .stats.main.results.buttons.prev -text "Previous" -command {stats-prev-system}
    button .stats.main.results.buttons.next -text "Next" -command {stats-next-system}
    pack  .stats.main.results.buttons.prev  .stats.main.results.buttons.next\
	    -side left -expand t -fill x
    pack .stats.main.results.buttons -side bottom -anchor n
    pack .stats.main.results -side bottom  -expand 1 -fill both
    label .stats.main.results.label -text "Comparison for system:"
    pack .stats.main.results.label -side top
    text .stats.main.results.text -bg white -font "Courier 11" -height 1 -wrap none
    .stats.main.results.text tag config boldtag -font "Courier 11 bold"
    .stats.main.results.text tag config italictag -font "Courier 11 italic"
    .stats.main.results.text tag config underlinetag -underline 1
    pack .stats.main.results.text -expand 1 -fill both
}


proc centre-string {str width} {
    if { [string length $str] > $width } {set str [string range $str 0 [expr $width - 1]]}
    set padlength [expr int(($width - [string length $str])/2)]
    return  "$str[string repeat " " $padlength]"
}

proc my-trim-string {str width} {
    if { [string length $str] > $width } {set str [string range $str 0 [expr $width - 1]]}
    return  "$str"
}

proc stats-show-table {} {
    pack forget .stats.main.results .stats.main.table
    pack .stats.main.table -side top -expand 1 -fill both
}



proc stats-hide-syschoose {} {
    # DESTROY THE SYSTEM CHOOSER (COMPSTATS) IF EXISTS
    if {[winfo exists .stats.tbar.system]} {
	destroy .stats.tbar.system 
    }
}

proc stats-show-filechoose {} {
    global COMPARE_FILE
    set f .stats.tbar.filename
    if {[winfo exists $f]} {
	destroy $f
    }

    frame $f -borderwidth 2 
    label $f.label -text "Compare to File:" 
    entry $f.entry  -bg white -relief sunken -textvariable COMPARE_FILE\
	     -borderwidth 1
    button $f.button -text "Locate" -command {select-compare-file}
    help-button $f.help select_compare_file
    pack $f.label -side left  -padx 3 -expand 0 -fill none
    pack  $f.help $f.button -side right -expand 0 -fill none -padx 5
    pack $f.entry -fill x -expand 1 -padx 3 -side left
    pack $f -side left -after .stats.tbar.locglob.help -in .stats.tbar.locglob -expand 1 -fill x
}

proc stats-show-table-or-system {} {
   global DISPLAY_TYPE
    set f .stats.tbar.top.displtype
    if {[winfo exists $f]} {
	destroy $f
    }

    frame $f -borderwidth 2 
    label $f.label -text "Display as:" 
    tk_optionMenu $f.choice DISPLAY_TYPE "Table" "Individual System"
    help-button $f.help stats_display_type

    pack $f.label $f.choice $f.help -side left
    pack $f -side left
}

defhelp stats_display_type "Select \"Table\" to show results for all systems
in tabular form. Select \"Individual Systems\" to show results for each
system in turn (in more detail)."

proc stats-hide-table-or-system {} {
    set f .stats.tbar.top.displtype
    if {[winfo exists $f]} {
	destroy $f
    }
}


proc select-compare-file {} {
  global COMPARE_FILE CODINGS_FILE  
  set COMPARE_FILE [tk_getOpenFile -title "Select Codings File" -parent .\
	  -initialfile $CODINGS_FILE\
	  -filetypes [file-save-types]]
}


defhelp select_compare_file "Select a saved codings file (.cd2) which you wish 
compared statistically to the current file. These files should have the same
coding network as the current one, or largely identical."


proc stats-hide-filechoose {} {
    if [winfo exists .stats.tbar.filename] {destroy .stats.tbar.filename}
}

proc stats-hide-locglob {} {
    # Hide THE LOCGLOB wgts IF EXISTS
    pack forget .stats.tbar.locglob.label .stats.tbar.locglob.choice .stats.tbar.locglob.help
}

proc stats-show-locglob {} {
    pack  .stats.tbar.locglob.label .stats.tbar.locglob.choice\
	    .stats.tbar.locglob.help -side left
}

proc stats-show-syschoose {} {
    global STATS_SYSTEM ALL_ACTIVE_SYSTEMS_SORTED
    # make the system chooser
    # destroy any old version
    if {[winfo exists .stats.tbar.system]} {
	destroy .stats.tbar.system
    }

    # for comparative only
    frame .stats.tbar.system
    label .stats.tbar.system.label -text "System to split on:"

    set ALL_ACTIVE_SYSTEMS_SORTED [lsort [all-active-systems]]    
    if { ![info exists STATS_SYSTEM] || ![system-exists-p $STATS_SYSTEM] } {
	set STATS_SYSTEM [car $ALL_ACTIVE_SYSTEMS_SORTED]
    }


    ChooseButton .stats.tbar.system.choice STATS_SYSTEM ALL_ACTIVE_SYSTEMS_SORTED
    help-button .stats.tbar.system.help stats_system

    pack .stats.tbar.system.label -side left
    pack .stats.tbar.system.choice -side left -expand 1 -fill x
    pack .stats.tbar.system.help -side left

    pack .stats.tbar.system -after .stats.tbar.locglob.help -in .stats.tbar.locglob\
	    -side left -padx 1c
}



proc stats-filtered-nodes  {nodes} {
    global STATS_FILTER  DISPLAYED_FILTER
    set filter [.stats.tbar.filter.text get 1.0 end]
    set matched [apply-filter $filter $nodes]
    
    # store the filter to be reinstated when the Stats is reentered
    set STATS_FILTER [string trim $DISPLAYED_FILTER]

    # Redisplay the (possibly) revised filter
    .stats.tbar.filter.text delete 1.0 end
    .stats.tbar.filter.text insert end "$STATS_FILTER"
    return $matched
}
    
defhelp stats_locglob "GLOBAL vs. LOCAL COUNTING

  In the normal case, the mean occurence of a feature is derived by dividing the number of codings which have that feature by the total number of codings in the set. This is called GLOBAL counting. If one root-level feature scored 20%, then the mean-scores of features of a system which inherits from this feature will add up to 20%.
    
  However, for systemic data, we are often interested in seeing the probability of selecting a feature from a system GIVEN that the system is entered. In this case, we divide the number of codings with the feature by the number of codings which select any feature from that system. The means of all features in the system thus add up to 100%."


defhelp stats_type "STASTICAL ANALYSIS TYPES

You can obtain 3 statistcal views of your codings:

  - Descriptive Statistics: provides the means, standard deviation, etc.
    for  each feature;
  - Comparative Statistics: you divide the codings into two or more subsets,
    which are then contrasted.
  - Cell Analysis: all codings with the same features are grouped. Each group
    is presented, allong with the count, and an example. To stop systems being
    used in the group formation, disable them in the scheme window."

    defhelp stats_filter\
"FILTERING
=========

  We often want to obtain statistics on a sub-corpus, for instance, to see the statistics for only finite clauses.

  SIMPLE FILTERING: To select only those codings with a particular feature, type in the name of the feature in the \"Only use codings which match\" box.

  COMPLEX FILTERING: You can select codings matching a complex combination
of features by typing in expressions like:

     finite and material

     mental or verbal

     not passive

  If you want to use 'and' and 'or' in the same filter, use parentheses to state clearly what you mean:

     finite and (mental or verbal) and not passive
     (finite and mental) or verbal

  WILDCARDS: If you are not sure of the name of the feature, type an asterisk in the name, e.g., \"fin*\" will match the feature \"finite\". If the wildcard matches more than one feature in your scheme, you will be asked to select the one you want.  Wildcards can also be used in complex filters, e.g., 

   fin* or ment*

  An asterisk by itself will show you ALL features in your scheme.
"
  
defhelp stats_system\
"COMPARATIVE STATS -- System Selection
=====================================

  Choose one of the systems offered here. Your codings will be divided into a number os sets, one for each feature in the system. Each of these sets will then be compared statistically.

Note: if a system has more than two features in it and you only want to compare two of them, use the Filter box (labelled: \"Only use codings which match\") to filter out the codings with the features you do not want. For instance, if we have a system: GENDER: male/female/neuter, and we want to compare male vs. female, select the GENDER system, and type (not neuter) in the filter box."



